package com.las2witsml;

//import classes
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JOptionPane;
import javax.swing.JTable;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Title: ButtonEditor.java 
 * Description:  This class is the default editor for table
 * Copyright: Copyright (c) 2006
 * Company: Satyam Computer Services Ltd.
  * @version 1.0
 * @author Ravi Domakonda, Lalitha Yengaldas
 */
public class ButtonEditor extends DefaultCellEditor {
	//declare variables
	protected JRadioButton rbButton;
	private String    label;
	private boolean   isPushed;
	
	/** 
	*  Constructor ButtonEditor specifying check box
	*/
	public ButtonEditor(JCheckBox checkBox) {
		super(checkBox);
		//Group the radio buttons.
		ButtonGroup group = new ButtonGroup();
		//set properties for radio button
		rbButton = new JRadioButton();
		rbButton.setName(label);
		rbButton.setOpaque(true);
		//add the button to group
		group.add(rbButton);
		//add a listener to button
		rbButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			  
			fireEditingStopped();
			}
		});
	}

	/**
	* Method getTableCellRendererComponent
	*This method returns the component used for drawing the cell. This method is used to configure the component based on
the coordinates and value in the cell.
	*
	* @param table
	* @param value
	* @param isSelected
	* @param row
	* @param column
	* @return Component
	*
	*/
	public Component getTableCellEditorComponent(JTable table, Object value,
		   boolean isSelected, int row, int column) {
			
		if (isSelected) {
			rbButton.setForeground(table.getSelectionForeground());
			rbButton.setBackground(table.getSelectionBackground());
		} else{
			rbButton.setForeground(table.getForeground());
			rbButton.setBackground(table.getBackground());
		}
		label = (value ==null) ? "" : value.toString();
		rbButton.setText( label );
		isPushed = true;
		
		return rbButton;
	}

	/**
	* Method getCellEditorValue
	*This method returns the value contained in the editor.
	*
	* @return Object
	*
	*/
	public Object getCellEditorValue() {
		if (isPushed)  {
		// 
		// 
		JOptionPane.showMessageDialog(rbButton ,label + ": Ouch!");
		// System.out.println(label + ": Ouch!");
		}
		isPushed = false;
		return new String( label ) ;
	}
 
	/**
	* Method stopCellEditing
	*This method forwards the message from the CellEditor to the delegate
	*
	* @return Object
	*
	*/ 
	public boolean stopCellEditing() {
		isPushed = false;
		return super.stopCellEditing();
	}
	
	/**
	* Method getCellEditorValue
	*This method notifies all listeners that have registered interest for notification on this event type.
	*
	*
	*/ 
	protected void fireEditingStopped() {
		//isPushed = false;
		//super.fireEditingStopped();
	}
}
