package com.las2witsml;

/**
 * Title: FileDetails.java
 * Description: A class holding the file selection details
 * Copyright: Copyright (c) 2006
 * Company: Satyam Computer Services Ltd.
  * @version 1.0
 * @author Ravi Domakonda, Lalitha Yengaldas
 */ 
public class FileDetails {
	//declare variables
	private int intSNo;
	private String strInputFile;
	private String strOutputFile;
	private boolean blnIsSelected;
	
	/** 
	*  Constructor FileDetails
	*/
	public FileDetails() {
	}

	/**
	* Constructor FileDetails specifying the serial no, input file, output file and file selection 
	*/
	public FileDetails(int intSNo, String strInputFile, String strOutputFile, boolean blnIsSelected) {
		this.intSNo = intSNo;
		this.strInputFile = strInputFile;
		this.strOutputFile = strOutputFile;
		this.blnIsSelected = blnIsSelected;
  }
	
	/**
	* Method setSNo
	*This method is used to set serial number for displaying file details
	* @param intSNo Serial Number for displaying file details
	*
	*/
	public void setSNo(int intSNo) {
		this.intSNo = intSNo;
	}

	/**
	 * Method getSNo
	 * This method is used to get serial number for displaying file details
	 * @return SNo for displaying file details
	 *
	 */
	public int getSNo() {
		return intSNo;
	}
	
	/**
	 * Method setInputFile
	 * This method sets the input LAS file name
	 * @param strInputFile InputFile of LASFile name
	 *
	 */
	public void setInputFile(String strInputFile) {
		this.strInputFile	= strInputFile;
	}

	/**
	 * Method getInputFile
	 * This method gets the input LAS file name
	 * @return Input LAS File name
	 *
	 */
	public String getInputFile() {
		return strInputFile;
	}
	
	/**
	 * Method setOutputFile
	* This method sets the destination WITSML file name
	* @param strOutputFile Destination WITSML file name
	 *
	 */
	public void setOutputFile(String strOutputFile) 	{
		this.strOutputFile	= strOutputFile;
	}

	/**
	 * Method getOutputFile
	 * This method gets the destination WITSML file name
	 * @return destination WITSML file name
	 *
	 */
	public String getOutputFile() 	{
		return strOutputFile;
	}

	/**
	 * Method setIsSelected
	 * This method selects the input LAS file 
	 * @param blnIsSelected File selection status
	 *
	 */
	public void setIsSelected(boolean blnIsSelected) {
		this.blnIsSelected	= blnIsSelected;
	}

	/**
	 * Method getIsSelected
	 * This method get the file selection status of  the input LAS file 
	 * @return File selection status
	 *
	 */
	public boolean getIsSelected() {
		return blnIsSelected;
	}
	
	/**
	 * Method createRow
	 * This method create a file selection row for displaying in file selection grid
	 * @return File Details object[] 
	 *
	 */
	public Object[] createRow() {
	
		Object[] newRow = new Object[4];
		//set serial number
		newRow[0] = new Integer(intSNo); 
		//set input LAS file
		newRow[1] = strInputFile;
		//set destination file
		newRow[2] = strOutputFile;
		//set file selection flag
		newRow[3] = "";
		//return the file details row
		return newRow;
	}
} 