package com.las2witsml;

//import classes
import java.awt.Component;
import java.awt.AWTEvent;
import java.awt.event.MouseEvent;
import java.lang.Boolean;
import java.util.EventObject;
import java.io.Serializable;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellEditor;
import javax.swing.event.EventListenerList;
import javax.swing.event.ChangeEvent;
import javax.swing.event.CellEditorListener;
import javax.swing.tree.TreeCellEditor;

/**
 * Title: JComponentCellEditor.java
 * Description: A class for displaying combo boxes in  a table
 * Copyright: Copyright (c) 2006
 * Company: Satyam Computer Services Ltd.
  * @version 1.0
 * @author Ravi Domakonda, Lalitha Yengaldas
 */
public class JComponentCellEditor implements TableCellEditor, TreeCellEditor, Serializable {
	//declare variables
	protected EventListenerList elListenerList = new EventListenerList();
	transient protected ChangeEvent ceChangeEvent = null;
	
	protected JComponent jcEditorComponent = null;
	protected JComponent jcContainer = null;		// Can be tree or table
	
	/**
	* Method getComponent
	* This method returns the component
	* @return jcEditorComponent
	*
	*/
	public Component getComponent() {
		return jcEditorComponent;
	}
	
	/**
	* Method getCellEditorValue
	* This method Returns the value contained in the editor
	* @return jcEditorComponent
	*
	*/
	public Object getCellEditorValue() {
		return jcEditorComponent;
	}
	
	/**
	* Method isCellEditable
	* This method asks the editor if it can start editing using anEvent
	* @return jcEditorComponent
	*
	*/
	public boolean isCellEditable(EventObject anEvent) {
		return true;
	}
	
	/**
	* Method shouldSelectCell
	* This method returns true if the editing cell should be selected, false otherwise
	* @return true
	*
	*/
	public boolean shouldSelectCell(EventObject anEvent) {
		if( jcEditorComponent != null && anEvent instanceof MouseEvent
			&& ((MouseEvent)anEvent).getID() == MouseEvent.MOUSE_PRESSED )
		{
		  Component dispatchComponent = SwingUtilities.getDeepestComponentAt(jcEditorComponent, 3, 3 );
			MouseEvent e = (MouseEvent)anEvent;
			MouseEvent e2 = new MouseEvent( dispatchComponent, MouseEvent.MOUSE_RELEASED,
				e.getWhen() + 100000, e.getModifiers(), 3, 3, e.getClickCount(),
				e.isPopupTrigger() );
			dispatchComponent.dispatchEvent(e2); 
			e2 = new MouseEvent( dispatchComponent, MouseEvent.MOUSE_CLICKED,
				e.getWhen() + 100001, e.getModifiers(), 3, 3, 1,
				e.isPopupTrigger() );
			dispatchComponent.dispatchEvent(e2); 
		}
		return false;
	}
	
	/**
	* Method stopCellEditing
	* This method tells the editor to stop editing and accept any partially edited value as the value of the editor
	* @return true
	*
	*/
	public boolean stopCellEditing() {
		fireEditingStopped();
		return true;
	}
	
	/**
	* Method cancelCellEditing
	* This method Tells the editor to cancel editing and not accept any partially edited value
	* @return true
	*
	*/
	public void cancelCellEditing() {
		fireEditingCanceled();
	}
	
	/**
	* Method addCellEditorListener
	* This method Adds a listener to the list that's notified when the editor stops, or cancels editing.
	* @return void
	*
	*/
	public void addCellEditorListener(CellEditorListener l) {
		elListenerList.add(CellEditorListener.class, l);
	}
	
	/**
	* Method removeCellEditorListener
	* This method Removes a listener from the list that's notified
	* @return void
	*
	*/
	public void removeCellEditorListener(CellEditorListener l) {
		elListenerList.remove(CellEditorListener.class, l);
	}
	
	/**
	* Method fireEditingStopped
	* This method stops the editing of a cell
	* @return void
	*
	*/
	protected void fireEditingStopped() {
		Object[] objListeners = elListenerList.getListenerList();
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = objListeners.length-2; i >= 0; i-=2) {
			if (objListeners[i]==CellEditorListener.class) {
				// Lazily create the event:
				if (ceChangeEvent == null)
					ceChangeEvent = new ChangeEvent(this);
				((CellEditorListener)objListeners[i+1]).editingStopped(ceChangeEvent);
			}	       
		}
	}
	
	/**
	* Method fireEditingStopped
	* This method cancels the editing of a cell
	* @return void
	*
	*/
	protected void fireEditingCanceled() {
		// Guaranteed to return a non-null array
		Object[] objListeners = elListenerList.getListenerList();
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = objListeners.length-2; i >= 0; i-=2) {
			if (objListeners[i]==CellEditorListener.class) {
				// Lazily create the event:
				if (ceChangeEvent == null)
					ceChangeEvent = new ChangeEvent(this);
				((CellEditorListener)objListeners[i+1]).editingCanceled(ceChangeEvent);
			}	       
		}
	}
	
	// implements javax.swing.tree.TreeCellEditor
	/**
	* Method fireEditingStopped
	* This method Sets an initial value for the editor
	* @return void
	*
	*/
	public Component getTreeCellEditorComponent(JTree tree, Object value,
		boolean isSelected, boolean expanded, boolean leaf, int row) {
		String         stringValue = tree.convertValueToText(value, isSelected,
			expanded, leaf, row, false);
		
		jcEditorComponent = (JComponent)value;
		jcContainer = tree;
		return jcEditorComponent;
	}
	
	// implements javax.swing.table.TableCellEditor
	/**
	* Method fireEditingStopped
	* This method Sets an initial value for the editor
	* @return void
	*
	*/
	public Component getTableCellEditorComponent(JTable table, Object value,
		boolean isSelected, int row, int column) {
		
		jcEditorComponent = (JComponent)value;
		jcContainer = table;
		return jcEditorComponent;
	}
	
} // End of class JComponentCellEditor
