/**
 * Title: LasData.java
 * Description:  This class is the used to maintain the general information about the selected LAS file
 * Copyright: Copyright (c) 2006
 * Company: Satyam Computer Services Ltd.
  * @version 1.0
 * @author Ravi Domakonda, Lalitha Yengaldas
 */

package com.las2witsml;

//import packages
import java.util.ArrayList;
import java.util.Hashtable;

public class LasData {

	//declare variables
	//Selected LAS file
	private String strInputLasFile;
	//Destination file
	private String strDestFile;
	//LAS file version
	private String strVersion;
	//LAS file WRAP
	private String strWrap;
	//LAS file Units
	private String strUnit;
	//WELL name
	private String strWell;
	//Unique Well identifier
	private String strUWI;
	//Null value
	private double dbNullValue;
	//START DEPTH
	private Double dbStartValue;
	//STOP DEPTH
	private Double dbStopValue;
	//STEP VALUE
	private Double dbStepValue;
	//LAS File delimiter
	private int intDelimiter;
	//LAS parameter data
	private  LasParameterData objParameterData;
	
	//Well Information array list
	private ArrayList alWellData;
	//Run Parameter Information array list
	private ArrayList alRunParameterData;
	//Parameter Information array list
	private ArrayList alParameterData;
	
	//LOG Section (LAS3.0) Information or Curve & Parameters Information (LAS2.0)
	//Log definition array list
	private ArrayList alLogDefinitions;
	//Log definition parameters
	private ArrayList alLogParameters;
	//For changed mappings
	private Hashtable htChangedElements ;
	
	//well data count
	private int intWellDataCnt=0;

	/**
	* Constructor LasData 
	*/
	public LasData() {
		//initialize variables
		strInputLasFile = "";
		strVersion = "";
		strWrap = "";
		intDelimiter = 9; //TAB '32 - SPACE and 44 - COMMA
		objParameterData = new LasParameterData();
		alWellData = new ArrayList();
		alRunParameterData = new ArrayList();
		alParameterData = new ArrayList();
		alLogDefinitions = new ArrayList();
		alLogParameters = new ArrayList();
	}
	
	/**
	 * Method setInputLasFile
	 * This method sets the input LAS file name
	 * @param stInputLasFile  Input LAS file name
	 *
	 */
	public void setInputLasFile(String stInputLasFile)
	{
		this.strInputLasFile	= stInputLasFile;
	}

	/**
	 * Method getInputLasFile
	 * This method returns the input LAS file name
	 * @return  input LAS file name
	 *
	 */
	public String getInputLasFile()
	{
		return strInputLasFile;
	}
	
	/**
	 * Method setDestFile
	 * This method sets the destination folder name
	 * @param  stDestFile Destination Folder name
	 *
	 */
	public void setDestFile(String stDestFile)
	{
		this.strDestFile = stDestFile;
	}

	/**
	 * Method getDestFile
	 * This method retuns the destination folder name
	 * @return Destination Folder name
	 *
	 */
	public String getDestFile()
	{
		return strDestFile;
	}
	
	/**
	 * Method setVersion
	 * This method sets the version name
	 * @param stVersion Version  of the LAS file
	 *
	 */
	public void setVersion(String stVersion)
	{
		this.strVersion	= stVersion;
	}

	/**
	 * Method getVersion
	 * This method returns the version of the LAS file
	 * @return Version  of the LAS file
	 *
	 */
	public String getVersion()
	{
		return strVersion;
	}
	
	/**
	 * Method setWRAP
	 * This method sets the WRAP mode of the LAS file
	 * @param stWRAP WRAP  mode of the LAS file
	 *
	 */
	public void setWRAP(String stWRAP)
	{
		this.strWrap	= stWRAP;
	}

	/**
	 * Method getWRAP
	 * This method returns the WRAP mode of the LAS file
	 * @return WRAP  mode of the LAS file
	 *
	 */
	public String getWRAP()
	{
		return strWrap;
	}
	
	/**
	 * Method setUnit
	 * This method sets the units of the LAS file
	 * @param stUnit Unit of the LAS file
	 *
	 */
	public void setUnit(String stUnit)
	{
		this.strUnit = stUnit;
	}

	/**
	 * Method getUnit
	 * This method returns the units of the LAS file
	 * @return Unit of the LAS file
	 *
	 */
	public String getUnit()
	{
		return strUnit;
	}
	
	/**
	 * Method setWell
	 * This method sets the well name of the LAS file
	 * @param stWell Well Name
	 *
	 */
	public void setWell(String stWell)
	{
		this.strWell = stWell;
	}

	/**
	 * Method getWell
	 * This method returns the well name of the LAS file
	 * @return Well Name
	 *
	 */
	public String getWell()
	{
		return strWell;
	}
	
	/**
	 * Method setUWI
	 * This method sets the unique well identifier of the LAS file
	 * @param stUWI UWI of the LAS file
	 *
	 */
	public void setUWI(String stUWI)
	{
		this.strUWI = stUWI;
	}

	/**
	 * Method getUWI
	 * This method returns the unique well identifier of the LAS file
	 * @return UWI of the LAS file
	 *
	 */
	public String getUWI()
	{
		return strUWI;
	}
	
	/**
	 * Method setNullValue
	 * This method sets the null value of the LAS file
	 * @param dbNullValue NullValue of the LAS file
	 *
	 */
	public void setNullValue(double dbNullValue)
	{
		this.dbNullValue = dbNullValue;
	}

	/**
	 * Method getNullValue
	 * This method returns the null value of the LAS file
	 * @return NullValue of the LAS file
	 *
	 */
	public double getNullValue()
	{
		return dbNullValue;
	}
	
	/**
	 * Method setStart
	 * This method sets the start depth of the LAS file
	 * @param dbStartValue StartValue of the LAS file
	 *
	 */
	public void setStart(Double dbStartValue)
	{
		this.dbStartValue = dbStartValue;
	}

	/**
	 * Method getStartValue
	 * This method returns the start depth of the LAS file
	 * @return dbStartValue
	 *
	 */
	public Double getStart()
	{
		return dbStartValue;
	}
	
	/**
	 * Method setStop
	 * This method sets the stop depth of the LAS file
	 * @param dbStopValue StopValue of the LAS file
	 *
	 */
	public void setStop(Double dbStopValue)
	{
		this.dbStopValue = dbStopValue;
	}

	/**
	 * Method getStopValue
	 * This method returns the stop depth of the LAS file
	 * @return StopValue of the LAS file
	 *
	 */
	public Double getStop()
	{
		return dbStopValue;
	}
	
	/**
	 * Method setStep
	 * This method sets the step value of the LAS file
	 * @param dbStepValue StepValue of the LAS file
	 *
	 */
	public void setStep(Double dbStepValue)
	{
		this.dbStepValue = dbStepValue;
	}

	/**
	 * Method getStepValue
	 * This method returns the step value of the LAS file
	 * @return StepValue of the LAS file
	 *
	 */
	public Double getStepValue()
	{
		return dbStepValue;
	}
	
	/**
	 * Method setDelimiter
	 * This method sets the delimiter of the LAS file
	 * @param intDelimiter Delimiter of LAS file
	 *
	 */
	public void setDelimiter(int intDelimiter)
	{
		this.intDelimiter =  intDelimiter;
	}

	/**
	 * Method getDelimiter
	 * This method returns the delimiter of the LAS file
	 * @return Delimiter of LAS file
	 *
	 */
	public int getDelimiter()
	{
		return intDelimiter;
	}
	
	/**
	 * Method setWellDataCnt
	 * This method sets the well count of the LAS file
	  * @param numRows WellData C ount
	 *
	 */
	public void setWellDataCnt(int numRows)
	{
		this.intWellDataCnt	= numRows;
	}

	/**
	 * Method getWellDataCnt
	 * This method returns the well count of the LAS file
	 * @return WellData C ount
	 *
	 */
	public int getWellDataCnt()
	{
		return intWellDataCnt;
	}
	
	/**
	 * Method setParameterData
	 * This method sets the Parameter data of the LAS file
	 * @param objParameterData ParameterData Object
	 *
	 */
	public void setParameterData(LasParameterData objParameterData)
	{
		this.objParameterData	= objParameterData;
	}
	
	/**
	 * Method getParameterData
	 * This method returns the Parameter data of the LAS file
	 * @return ParameterData
	 *
	 */
	public LasParameterData getParameterData()
	{
		return objParameterData;
	}
	
	/**
	 * Method setLogDefns
	 * This method sets the log definitions data of the LAS file
	 * @param alLogDefinitions ArrayList of LogDefinitions
	 *
	 */
	public void setLogDefns(ArrayList alLogDefinitions)
	{
		this.alLogDefinitions	= alLogDefinitions;
	}
	
	/**
	 * Method getLogDefns
	 * This method returns the log definitions data of the LAS file
	 * @return ArrayList of LogDefinitions
	 *
	 */
	public ArrayList getLogDefns()
	{
		return alLogDefinitions;
	}
	
	/**
	 * Method setWellSectionData
	 * This method sets the well data of the LAS file
	 * @param alWellData Well Parameter Data
	 *
	 */
	public void setWellSectionData(ArrayList alWellData)
	{
		this.alWellData = alWellData;
	}
	/**
	 * Method getWellSectionData
	 *This method returns the ArrayList of WellSectionData
	 * @return Well Parameter Data
	 *
	 */
	public ArrayList getWellSectionData()
	{
		return alWellData;
	}
	
	/**
	 * Method setRunParameterSectionData
	 * This method sets the Run parameter data of the LAS file
	 * @param alRunParameterData Run Parameter Data
	 *
	 */
	public void setRunParameterSectionData(ArrayList alRunParameterData)
	{
		this.alRunParameterData = alRunParameterData;
	}
	/**
	 * Method getRunParameterSectionData
	 *This method returns the ArrayList of RunParameterSection
	 * @return  Parameter Data
	 *
	 */
	public ArrayList getRunParameterSectionData()
	{
		return alRunParameterData;
	}
	
	/**
	 * Method setParameterSectionData
	 *This method sets the parameter data of the LAS file
	 * @param alParameterData Parameter Data
	 *
	 */
	public void setParameterSectionData(ArrayList alParameterData)
	{
		this.alParameterData = alParameterData;
	}
	/**
	 * Method getParameterSectionData
	 *This method returns the ArrayList of ParameterSectionData
	 * @return ArrayList
	 *
	 */
	public ArrayList getParameterSectionData()
	{
		return alParameterData;
	}
  	
	/**
	 * Method setLogParameters
	 * This method sets the log parameters of the LAS file
	 * @param alLogParameters ArrayList of LogParameters
	 *
	 */
	public void setLogParameters(ArrayList alLogParameters)
	{
		this.alLogParameters = alLogParameters;
	}
	
	/**
	 * Method getLogParameters
	 * This method returns the log parameters data of the LAS file
	 * @return ArrayList of LogParameters
	 *
	 */
	public ArrayList getLogParameters()
	{
		return alLogParameters;
	}
	
	/**
	 * Method setChangedMappings
	 * This method sets the changed mappings
	 * @param htChangedElements Hashtable of changed mappings
	 *
	 */
	public void setChangedMappings(Hashtable htChangedElements){
		this.htChangedElements = htChangedElements;
	}
	
	/**
	 * Method getChangedMappings
	 * This method returns the changed mappings
	 * @return Hashtable of changed mappings
	 *
	 */
	public Hashtable getChangedMappings() {
		//check for the section and get the mappings
		return htChangedElements;
	}
		
}

