package com.las2witsml;

//import classes
import java.io.File;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileWriter;
import java.io.PrintWriter;

import java.lang.Math;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Enumeration;

//date formatting related classes
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

//xml related classes
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.OutputKeys;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.Text;

import org.apache.xpath.XPathAPI;
import org.apache.xpath.objects.XObject;

/**
* Title: LasDataFile.java
* Description: This is the main class that reads the LAS data and performs the WITSML conversion.
* Copyright: Copyright (c) 2005
* Company: Satyam Computer Services Ltd.
* @author Ravi Domakonda, Lalitha Yengaldas
* @version 1.0
*/
public class LasDataFile {
	//Declare variables
	LasData objLasData = null;
	LasSectionInfo objLasSectionInfo = null;
	//delimiter of the LAS file
	int intDelimiter = 32; // 32 -- SPACE By Default , 44 -- COMMA
	int intWellParamSize = -1;
	int intRunNumber = 0;
	String strVersion = "";
	String strWrap = "";
	String strLineData = null;
	String strMnemonic = null;
	String strUnits = null;
	String strValue = null;
	String strDescription = null;
	String strFormat = null;
	String strArrayAsocs = null;
	String strMappingFile = null;
	String strSelectionFile = null;
	//array list to hold well section data, curve section data and data array
	ArrayList alWellData = null;
	ArrayList alSelectedCurves = null;
	ArrayList alDataArray = null;
	
	FileInputStream inFileInputStream = null;
	DataInputStream inDataInputStream = null;
	
	DocumentBuilderFactory domFactory = null; 
	DocumentBuilder domBuilder = null; 
	Document newDoc = null;
	Element root = null;
	Node documentNode = null;
	//hash table to hold changed mapping elements
	Hashtable htMappingChangedElements = null;
	//vector to hold generated WITSML output  files
	Vector vecGeneratedFiles = null;
	//hash table to hold units mapping
	Hashtable htUnitMapping = null;
	//printWriter
	PrintWriter pwPrintWriter = null;
	
	/** 
	*  Constructor LasDataFile
	*/
	public LasDataFile() {
		try{
			vecGeneratedFiles = new Vector();
			htUnitMapping = new Hashtable();
			String[] strLasUnits = {"F", "M", "DEC", "G/C3", "GAPI", "HRS" , "IN", "LBS", "MV", "NAPI", "OHMM", "US/F"};
			String[] strWitsmlUnits = {"ft", "m", "Euc", "g/cm3", "gAPI", "3600.s" , "(1/12).ft'", "lbf", "mV", "nAPI", "ohm.m", "Us/ft"};
			
			for (int intUnitLen=0; intUnitLen < strLasUnits.length; intUnitLen++) {
				htUnitMapping.put(strLasUnits[intUnitLen], strWitsmlUnits[intUnitLen]);
			}
			//System.out.println(" htUnitMapping size ... "+htUnitMapping.size());
		}catch(Exception e)
		{
			System.out.println("The error in creating a log file " + e);
		}
		
	}
	
	/**
	* Method getLasData
	*Description: This will return LasData object for holding the LAS file data.
	* @return objLasData
	*
	*/
	public LasData getLasData() {
		return objLasData;
	}
	
	/**
	* Method setLasData
	* This method sets the LasData object
	* @param objLasData Data to be set from the LAS file
	*
	*/
	public void setLasData(LasData objLasData) {
		this.objLasData = objLasData;
	}
	
	/**
	* Method getVersion
	*Description: This will returns the version of the LAS file
	* @return version number
	*
	*/
	public String getVersion() {
		return strVersion;
	}
	
	/**
	* Method setVersion
	* This method sets the version of the LAS file
	* @param strVers version number
	*
	*/
	public void setVersion(String strVers) {
		this.strVersion = strVers;
	}
	
	/**
	* Method getWrap
	*Description: This will returns the WRAP mode of the LAS file, This can be Multiple lines per depth step or One line per depth step
	* @return WRAP mode of the LAS file
	*
	*/
	public String getWrap() {
		return this.strWrap;
	}
	
	/**
	* Method setWrap
	* This method sets the WRAP of the LAS file
	* @param strWr WRAP mode of the LAS file
	*
	*/
	public void setWrap(String strWr) {
		this.strWrap = strWr;
	}
	
	/**
	* Method getDelimiter
	*Description: This will returns the delimiter of the LAS file
	* @return Delimiter of the LAS file
	*
	*/
	public int getDelimiter() {
		return this.intDelimiter;
	}
	
	/**
	* Method setDelimiter
	* This method sets the delimiter
	* @param intDelim Delimiter of the LAS file
	*
	*/
	public void setDelimiter(int intDelim) {
		this.intDelimiter = intDelim;
	}
	
	
	/**
	* Method LoadFileData
	*Description: This will returns an instance of LasData object
	* @param strLasFileName LAS file name
	*
	*/
	public void LoadFileData(String strLasFileName)
	{
		int intLoopVar = 0;
		try{
			//System.out.println("The LAS file is ..."+strLasFileName);
			pwPrintWriter.println("The LAS file is ..."+strLasFileName);
			//pwPrintWriter.println("\n");
			objLasData = new LasData();
		
			alDataArray = new ArrayList();
			
			String strPattern  = "~A";
			String strSrcFileContents = getSrcFileDataBlock(strLasFileName, "");
			Pattern pattern = Pattern.compile(strPattern,Pattern.CASE_INSENSITIVE);
					
			Matcher matcher = pattern.matcher(strSrcFileContents);
			
			// Find all matches
			while (matcher.find()) {
				intLoopVar++;
				//System.out.println("intLoopVar in LoadFileData "+intLoopVar);
				
			}
			setRunNumber(intLoopVar);
			
			
		} catch(Exception e) {
			System.out.println(e.getMessage());
		}

	}
	
	/**
	* Method readRunParametersSectionData
	*Description: This will read details of PARAMETER INFORMATION block in the input LAS File.
	* @param strSrcFileFullName LAS file name
	* @param objLasData LAS data of the LAS file
	* @param strSearchString The parameter section to be read (starting with ~P) in LAS file
	*
	*/
	public void readRunParametersSectionData(String strSrcFileFullName, LasData objLasData, String strSearchString) {
		String strParamInfoBlock="";
		String strLineData = "";
		int intNextIndex  = -1;
		//get Parameter Information block starting from ~Parameter till next occurence of ~ character
		strParamInfoBlock = getSrcFileDataBlock(strSrcFileFullName, "~P");
		
		if (strParamInfoBlock.length() == 0) {
			if (strSearchString != null) {
				//File is Version 3.0, check for Log_Parameter and get the data block from LAS file
				strParamInfoBlock = getSrcFileDataBlock(strSrcFileFullName, strSearchString);
			}
			if (strParamInfoBlock.length() == 0) {
				return;
			}
		}
		
		intNextIndex = getLineFromBlock(strParamInfoBlock, strLineData, (intNextIndex + 1));
		strLineData = this.strLineData;
		
		// Skip through the block till you reach the first curve definition
		while ((strLineData.indexOf("#") != -1) || (strLineData.indexOf("~") != -1)) {
			intNextIndex = getLineFromBlock(strParamInfoBlock, strLineData, (intNextIndex + 1));
			strLineData = this.strLineData;
		}
		
		//clear the log parameters if already read
		if (objLasData.getLogParameters().size() > 0) {
			objLasData.setLogParameters(new ArrayList());
		}
		//System.out.println("objLasData.getLogParameters().size()........"+objLasData.getLogParameters().size());
		pwPrintWriter.println("objLasData.getLogParameters().size()........"+objLasData.getLogParameters().size());
		//get line of data from the above block of data
		while (intNextIndex != -1) {
			getLineData(strLineData, strMnemonic, strUnits, strValue, strDescription, strFormat, strArrayAsocs);
			//ignore the line that starts with #, as this is a comment
			if (!(strLineData.startsWith("#"))) {
				if (objLasData != null) {
					//set the line of data into an object
					objLasSectionInfo = new LasSectionInfo();
					//objLasSectionInfo.setIsSelected(new Boolean(false));
					objLasSectionInfo.setIsSelected(new Boolean(true));
					objLasSectionInfo.setMnemonic(strMnemonic);
					objLasSectionInfo.setUnit(strUnits);
					objLasSectionInfo.setValue(strValue);
					objLasSectionInfo.setDescription(strDescription);
					//copy the data to a hashtable
					objLasData.getParameterData().setRunParameterSectionData(objLasSectionInfo);
				}
				if (strMnemonic != "") {
					//add the data to Parameters array list
					objLasData.getLogParameters().add(objLasSectionInfo);
				} 
			}
			//read next line of data
			intNextIndex = getLineFromBlock(strParamInfoBlock, strLineData, intNextIndex + 1);
			strLineData = this.strLineData;
		}
		//set the LasData object
		setLasData(objLasData);
	}
	
	
	/**
	 * Method readWellSectionData
	 *Description: This will read details of WELL INFORMATION block in the input LAS File.
	 * @param strSrcFileFullName input LAS file name
	 * @param objLasData LAS data of the input LAS file
	 * @param alWellData The well section to be read (starting with ~W) in input LAS file
	 *
	*/
	public void readWellSectionData(String strSrcFileFullName, LasData objLasData, ArrayList alWellData) {
		String strLineData = "";
		int intNextIndex = -1;
		int intNumRows = 0;
		
		alWellData = new ArrayList();
		String strWellInfoBlock = "";
		this.strLineData = "";
		this.strMnemonic = "";
		this.strUnits = "";
		this.strValue = "";
		this.strDescription = "";
		this.strFormat = "";
		this.strArrayAsocs = "";
	
		// Get Well Information block starting from ~Well till next occurence of ~ character
		strWellInfoBlock = getSrcFileDataBlock(strSrcFileFullName, "~W");
		strWellInfoBlock.trim();
		//read the line from the block of data read above
		intNextIndex = getLineFromBlock(strWellInfoBlock, strLineData, (intNextIndex + 1));
		strLineData = this.strLineData;
		
		//Skip through the block till you reach START
		while (strLineData.indexOf("STRT") == -1) {
			intNextIndex = getLineFromBlock(strWellInfoBlock, strLineData, (intNextIndex + 1));
			strLineData = this.strLineData;
		}
		//get line of data from the above block of data
		while (intNextIndex != -1) {
			if (strLineData.length() != 0) {
				//get line of data from the above block of data
				getLineData(strLineData, strMnemonic, strUnits, strValue, strDescription, strFormat, strArrayAsocs);
				//ignore the line that starts with #, as this is a comment
				if (!(strLineData.startsWith("#"))) {
					intNumRows++;
					if (objLasData != null) {
						objLasSectionInfo = new LasSectionInfo();
						//set mnemonic
						objLasSectionInfo.setMnemonic(strMnemonic);
						//set units 
						if (strMnemonic.equals("STRT")) {
							objLasData.setUnit(strUnits);
							objLasSectionInfo.setIsSelected(new Boolean(true));
						}
						//set nullValue
						if (strMnemonic.equals("NULL")) {
							objLasData.setNullValue((new Double(strValue)).doubleValue());
							objLasSectionInfo.setIsSelected(new Boolean(true));
						}
						//set well name
						if (strMnemonic.equals("WELL")) {
							objLasData.setWell(strValue);
						}
						//set well identifier
						if (strMnemonic.equals("UWI") || strMnemonic.equals("API")) {
							objLasData.setUWI(strValue);
						}
						if (strMnemonic.equals("STOP")) {
							objLasSectionInfo.setIsSelected(new Boolean(true));
						}
						if (strMnemonic.equals("STEP")) {
							objLasData.setStep((new Double(strValue)));
							objLasSectionInfo.setIsSelected(new Boolean(true));
						}
						objLasSectionInfo.setUnit(strUnits);
						objLasSectionInfo.setValue(strValue);
						objLasSectionInfo.setDescription(strDescription);
						//add the details from the line read to a hash table
						objLasData.getParameterData().setWellSectionData(objLasSectionInfo);
						//add the details from the line read to an array list
						alWellData.add(objLasSectionInfo); 
					}
				} //end if (strLineData.Length != 0)
			}
			//read the next line from the blcok
			intNextIndex = getLineFromBlock(strWellInfoBlock, strLineData, intNextIndex + 1);
			strLineData = this.strLineData;
		}//end while (intNextIndex != -1) 
		//set the well section data
		objLasData.setWellSectionData(alWellData);
		//set the Las Data object
		setLasData(objLasData);
	}
	
	/**
	 * Method readParametersSectionData
	 *Description: This will read details of WELL INFORMATION and PARAMETER INFORMATION block in the input LAS File.
	 * @param strSrcFileFullName input LAS file name
	 * @param objLasData LAS data of the input LAS file
	 *
	*/
	public void readParametersSectionData(String strSrcFileFullName, LasData objLasData) {
		ArrayList alWellData = new ArrayList();
		ArrayList alRunParameterData = new ArrayList();
		Hashtable htRunData = null;
		Hashtable htWellData = null;
	
		//get Well parameter data
		readWellSectionData(strSrcFileFullName,objLasData,alWellData);
		alWellData = objLasData.getWellSectionData(); 
			
		//get Run parameter data
		readRunParametersSectionData(strSrcFileFullName,objLasData,null);
		alRunParameterData = objLasData.getLogParameters();
		
		//set the well section information to the parameter section data	
		if ((alWellData != null) && (alWellData.size() > 0)) {
			setWellParamSize(alWellData.size());
			objLasData.setParameterSectionData(alWellData);
		}
		
		//set the run information to the parameter section data	
		if ((alRunParameterData != null) && (alRunParameterData.size() > 0)) {
			for (int intParameter=0; intParameter<alRunParameterData.size(); intParameter++) {
				//set the run parameter information to the parameter section data	
				objLasData.getParameterSectionData().add(alRunParameterData.get(intParameter));
			}
		}
		//get well section data
		htWellData = objLasData.getParameterData().getWellSectionData();
		//get parameter section data
		htRunData = objLasData.getParameterData().getRunParameterSectionData();
		//set the data to a hashtable	
		if ((htWellData != null) && (htWellData.size() > 0)) {
			Enumeration e = htWellData.keys(); 
			//System.out.println(" Well data Enumeration  is .."+e);
			while(e.hasMoreElements() ) { 
				LasSectionInfo objLasSectionInfo = (LasSectionInfo) htWellData.get(e.nextElement()); 
				// LasSectionInfo objLasSectionInfo = (LasSectionInfo) e.nextElement(); 
				//htParameterData.put(objLasSectionInfo.getMnemonic(),objLasSectionInfo);
				objLasData.getParameterData().setParameterSectionData(objLasSectionInfo);
				     //System.out.println(" Well data getMnemonic:"+objLasSectionInfo.getMnemonic()); 
				     //System.out.println("getDescription:"+objLasSectionInfo.getDescription()); 
			} 
		}
		
		if ((htRunData != null) && (htRunData.size() > 0)) {
			Enumeration e = htRunData.keys(); 
			//System.out.println(" Run data Enumeration  is .."+e);
			while(e.hasMoreElements() ) { 
				 LasSectionInfo objLasSectionInfo = (LasSectionInfo) htRunData.get(e.nextElement()); 
				// LasSectionInfo objLasSectionInfo = (LasSectionInfo) e.nextElement(); 
				//htParameterData.put(objLasSectionInfo.getMnemonic(),objLasSectionInfo);
				objLasData.getParameterData().setParameterSectionData(objLasSectionInfo);
				//System.out.println(" Run data  getMnemonic:"+objLasSectionInfo.getMnemonic()); 
				//System.out.println("getDescription:"+objLasSectionInfo.getDescription()); 
			} 
		}
		//set the LasData	
		setLasData(objLasData);
	}
	
	
	/**
	* Method getDataFromLine
	*Description: This will read the different blocks of data from a line of LAS data.
	* @param strLineData Line of Data from the input LAS file
	* @param intDelimiter Delimiter of the input LAS file
	* @param alDataArray Array for storing data
	*
	*/
	public void getDataFromLine(String strLineData, int intDelimiter, ArrayList  alDataArray) {
		int intStartIndex = 0;
		int intStopIndex = -1;
		String strToken = "";
		String strCommaLineData = "";
		
		//intDelimiter = 32;
		char chDelimiter = (char) intDelimiter;
		
		intStartIndex = 0;
		intStopIndex = strLineData.indexOf(chDelimiter, intStartIndex);
		//while there is no delimiter
		while (intStopIndex != -1) {
			//get the token
			strToken = strLineData.substring(intStartIndex, (intStopIndex - intStartIndex)+intStartIndex);
			strToken.trim();
			
			// Some strings might be given in Quotes
			if (strToken.startsWith("\"" ) == true) {
				// Go to matching END QUOTE
				//intStopIndex = strLineData.indexOf("""", intStartIndex + 1);
				intStopIndex = strLineData.indexOf("\"", intStartIndex + 1);
				
				// Move till next occurence of delimiter character
				intStopIndex = strLineData.indexOf(chDelimiter, intStopIndex + 1);
				
				// Get the updated token
				strToken = strLineData.substring(intStartIndex, (intStopIndex - intStartIndex)+intStartIndex);
			}
			//add the token to the data array	
			if (!strToken.equals("")) {
				alDataArray.add(strToken);
			}
			intStartIndex = intStopIndex + 1;
			intStopIndex = strLineData.indexOf(chDelimiter, intStartIndex);
		} //end while
		
		//Last Element to be read
		strToken = strLineData.substring(intStartIndex);
		//add the token to data array
		alDataArray.add(strToken);
		//set the data array and line data
		this.alDataArray = alDataArray;
		this.strLineData=strLineData;
	}
	
	
	/**
	* Method readVersionInformation
	*Description: This will read the version information of the LAS file.
	* @param strSrcFileFullName LAS file name
	* @param strVers Version number of the input LAS file
	* @param strWrap WRAP mode of the input LAS file
	* @param intDelimiter Delimiter of the input LAS file
	*
	*/
	 public void readVersionInformation(String strSrcFileFullName , String  strVers, String strWrap, int intDelimiter) {
	 
		String strLineData = "";
		String strVersionInformation="";
		int intNextIndex = -1;
		// Get Version Information block starting from ~V till next occurence of ~ character
		strVersionInformation = getSrcFileDataBlock(strSrcFileFullName, "~V");
	
		// First line is Title of Version Information section, discard
		intNextIndex = getLineFromBlock(strVersionInformation, strLineData, (intNextIndex + 1));
		strLineData = this.strLineData;
	
		// Skip through the block till you reach VERS
		while (strLineData.indexOf("VERS") == -1) {
			//get line of data from the above block
			intNextIndex = getLineFromBlock(strVersionInformation, strLineData, (intNextIndex + 1));
			strLineData = this.strLineData;
		}
	
		//We have Reached VERS
		intNextIndex = getLineFromBlock(strVersionInformation, strLineData, (intNextIndex + 1));
		strLineData = this.strLineData;
		//get line data
		getLineData(strLineData, strMnemonic, strUnits, strValue, strDescription, strFormat, strArrayAsocs);
		//set version
		setVersion(this.strValue);
		strVers = this.strValue;
		
		// Next line will have WRAP
		intNextIndex = getLineFromBlock(strVersionInformation, strLineData, (intNextIndex + 1));
		//get line data
		getLineData(strLineData, strMnemonic, strUnits, strValue, strDescription, strFormat, strArrayAsocs);
		//set Wrap mode of the file
		setWrap(strValue);
		strLineData = this.strLineData;
		//set version
		setVersion(this.strValue);
		strVers = this.strValue;
	
		intDelimiter = 32 ;// By Default
		 
		if (!((strVers.trim().equalsIgnoreCase("2")) || (strVers.trim().equalsIgnoreCase("2.0")))) {
			// Delimiter mnemonic will be present only for LAS3.0 file
			intNextIndex = getLineFromBlock(strVersionInformation, strLineData, (intNextIndex + 1));
			//get line data
			getLineData(strLineData, strMnemonic, strUnits, strValue, strDescription, strFormat, strArrayAsocs);
			//set the delimiter
			if (strValue.equalsIgnoreCase("COMMA")) {
					setDelimiter(44);
			} else if (strValue.equalsIgnoreCase("SPACE")) {
					setDelimiter(32);
			} else if (strValue.equalsIgnoreCase("TAB")) {
					setDelimiter(9);
			}
		}
	}
	
	/**
	* Method getSrcFileDataBlock
	*Description: This method processes the source LAS file and reads the contents of the file to a string and returns it to calling function
	* @param strFileFullName input LAS file
	* @param strBlockStart The Block to be read from the LAS File (like ~V, ~W, ~P )
	*
	*/
	public String getSrcFileDataBlock(String strFileFullName, String strBlockStart){
		//String strFileContents = "";
		//String strLine = "";
		String strLine = null;
		StringBuffer strFileContents = new StringBuffer();
		
		try{
			//System.out.println("The file name is " + strFileFullName);
			pwPrintWriter.println("The file name is " + strFileFullName);
			inFileInputStream = new FileInputStream(strFileFullName);
			BufferedReader inBufferedReader = new BufferedReader(new InputStreamReader(inFileInputStream));
			//check if block start is empty, if block start is empty read all the data from LAS file
			
			if(strBlockStart == "") 	{
					
				strLine = inBufferedReader.readLine();
				//System.out.println("The file strLine is " + strLine);
				//pwPrintWriter.println("The file strLine is " + strLine);
				while (strLine!=null) {
					strFileContents.append(strLine + '\n');
					strLine = inBufferedReader.readLine();
					//System.out.println("The file strLine is 222 " + strLine);
					//pwPrintWriter.println("The file strLine is 222 " + strLine);
				}
				return strFileContents.toString();
			}
			strLine=inBufferedReader.readLine();
			// Skip till the start block
			while (strLine.indexOf(strBlockStart) == -1) {
				strLine = inBufferedReader.readLine();
				if (strLine == "") { 
					return "";
				}
			}
			//append the new line character and read the next line
			strFileContents.append(strLine + '\n');
			strLine = inBufferedReader.readLine();
			//if the above line is not null, read the block of data till we reach ~
			if (strLine != null) {
				while (strLine.indexOf("~") == -1) {
					strFileContents.append(strLine + '\n');
					strLine = inBufferedReader.readLine();
					if (strLine == null) {
						break;
					}
				}
			}
			// Reached the end of the data block We dont need further data from file
			inBufferedReader.close();
		} catch(NullPointerException npe) {
			System.out.println("Null Pointer Exception is ..."+npe);
			pwPrintWriter.println("Null Pointer Exception is ..."+npe);
		}
		catch(Exception e) {
			System.out.println("Exception is ..."+e);
			pwPrintWriter.println("Exception is ..."+e);
		}
		//System.out.println("The value is  :::::::::::::::::::::::::::: " + strFileContents);
		return strFileContents.toString();
	}//end getSrcFileDataBlock
	
	/**
	* Method getLineFromBlock
	*Description: This method will read a line of data from the passed input string
	* @param strInputString Block of data from the input LAS file
	* @param strRetString Line of data from the input LAS file
	* @param intStartIndex StartIndex from where the block starts
	*
	*/
	public int getLineFromBlock(String strInputString, String strRetString, int intStartIndex) {
		int intStopIndex = -1;
		char chNewLine;
		strRetString = "";
		chNewLine = (char) 10;//Newline character \n
		try {
			//search for new line character
			intStopIndex = strInputString.indexOf(chNewLine, intStartIndex);
			//if there is no new line character
			if (intStopIndex == -1) {
				chNewLine = (char) 13;  //Carriage Return Character \r
				//search for carriage return character
				intStopIndex = strInputString.indexOf(chNewLine);
				//pwPrintWriter.println("intStopIndex " +intStopIndex + " ???? ");
				if (intStopIndex == -1) {
					chNewLine = (char) 12; //Form Feed character \f
					//search for form feed character
					intStopIndex = strInputString.indexOf(chNewLine);
				}else{
					intStopIndex=strInputString.length();
				}
				
			}
			//get the substring
			/*pwPrintWriter.println("The intStartIndex after getLineFromBlock is " + intStartIndex);
			pwPrintWriter.println("The intStopIndex after getLineFromBlock is " + intStopIndex);*/
			if (intStopIndex != -1) 
				strRetString = strInputString.substring(intStartIndex, ((intStopIndex - intStartIndex)+intStartIndex));
			else 
				strRetString = strInputString;
			
			strRetString = strRetString.trim();
			this.strLineData = strRetString;
		} catch(Exception e) {
			System.out.println("GetLineFromBlock():::::: Exception is  " + e);
			pwPrintWriter.println("GetLineFromBlock():::::: Exception is  " + e);
		}
		return intStopIndex;
	}//end GetLineFromBlock
	
	/**
	* Method getLineData
	*Description: This method gets the different blocks from the line of LAS data read
	* @param strLineData A Line of  data from the input LAS file
	* @param strMnemonic Mnemonic from the Line
	* @param strUnits Units from the Line
	* @param strValue Value from the Line
	* @param strDescription Description from the Line
	* @param strFormat Format from the Line
	* @param strArrayAsocs ArrayAsocs from the Line
	*
	*/    
	public void getLineData(String strLineData, String strMnemonic,String  strUnits, String  strValue,String strDescription, String strFormat, String strArrayAsocs)  {
		int startPos;
		int stopPos;
		int intFirstIndex;
		int intSecondIndex;
		int intThirdIndex;
		
		//Reset the Reference params
		strMnemonic = "";
		strUnits = "";
		strValue = "";
		strDescription = "";
		strFormat = "";
		strArrayAsocs = "";
	
		if(strLineData.length() == 0) {
			return;
		}
		
		// Line will be in the following format
		// <MNEMONIC>.<UNIT>     <VALUE>  : <DESCRIPTION> {<FORMAT>} | <ASOC1>, <ASOC2>
		//if the line starts with #, ignore the line as this is a comment
		if(strLineData.substring(0,1) == "#") 	{
			return ;
		}
		
		//Get the Mnemonic - String between StartPos and first occurence of DOT (.)
		startPos = 0;
		stopPos = strLineData.indexOf(".", startPos + 1);
		if((startPos != -1) && (stopPos != -1)) {
			strMnemonic = strLineData.substring(startPos, ((stopPos - startPos)+(startPos)));
		}
		strMnemonic = strMnemonic.trim();
		
	
		// Special case: If mnemonic is VERS then we may encounter 2.0 / 3.0 and no need to
		// check for UNITS
	
		if((strMnemonic != "VERS") && (strMnemonic != "WRAP") && (strMnemonic != "DLM")) {
			//Get the UNIT - String between the First DOT and subsequent occurence of either
			// DOT (.) or SPACE (32) or COLON (:)
		
			startPos = stopPos;
	
			intFirstIndex = strLineData.indexOf(" ", startPos + 1);
			intSecondIndex = strLineData.indexOf(".", startPos + 1);
			intThirdIndex = strLineData.indexOf(":", startPos + 1);
				
			// Since we are looking for MIN value, if any one of the index is
			//set to -1, there will be a problem, hence set the index for which
			//value is -1 to Integer's Max Value
			if(intFirstIndex == -1) 	{
				intFirstIndex = Integer.MAX_VALUE;
			}
			if(intSecondIndex == -1)  {
				intSecondIndex = Integer.MAX_VALUE;
			}
	
			if(intThirdIndex == -1)  {
				intThirdIndex = Integer.MAX_VALUE;
			}
			stopPos = Math.min(intFirstIndex, intSecondIndex);
			stopPos = Math.min(stopPos, intThirdIndex);
			
			if ((startPos != -1) && (stopPos != -1) && (stopPos != Integer.MAX_VALUE))   {
				strUnits = strLineData.substring((startPos + 1), ((stopPos - startPos - 1)+(startPos + 1)));
			}
				strUnits = strUnits.trim();
		}//end if
		
		
		// Find the VALUE - After the Units is complete and till first Colon
		startPos = stopPos;
		stopPos = strLineData.indexOf(":", startPos + 1);
	
		if ((startPos != -1) && (stopPos != -1))  {
			strValue = strLineData.substring(startPos + 1, ((stopPos - startPos - 1)+(startPos + 1)));
		}
		strValue = strValue.trim();
	
		//Find the DESCRIPTION - String after the LAST COLON till either First { 
		//or Last |(Pipe) or End of Line
		startPos = strLineData.lastIndexOf(":") + 1;
	
		intFirstIndex = strLineData.indexOf("{", startPos);
		intSecondIndex = strLineData.lastIndexOf("|");
		intThirdIndex = strLineData.length(); //'End of line
	
		//Since we are looking for MIN value, if any one of the index is
		//set to -1, there will be a problem, hence set the index for which
		//value is -1 to Integer's Max Value
		if (intFirstIndex == -1) 	{
			intFirstIndex = Integer.MAX_VALUE;
		}
				
		if (intSecondIndex == -1) {
			intSecondIndex = Integer.MAX_VALUE;
		}
				
		if (intThirdIndex == -1) {        
			intThirdIndex = Integer.MAX_VALUE;
		}
				
		//Since we are looking for MAX value, if any one of the index is
		//set to -1, there is no problem
		stopPos = Math.min(intFirstIndex, intSecondIndex);
		stopPos = Math.min(stopPos, intThirdIndex);
		//get description
		if ((startPos != -1) && (stopPos != -1)) {
			if (stopPos == intThirdIndex){
				// Till End Of Line
				strDescription = strLineData.substring(startPos);
			} else{
				strDescription = strLineData.substring(startPos, ((stopPos - startPos - 1)+startPos));
			}
		}
		strDescription = strDescription.trim();
		
		int intDescStopPos = stopPos;
		//get format
		if (stopPos < strLineData.length()) {
			// Format and Associations are Optional
			// Format - This will be after the description is complete and enclosed in { and }
			startPos = strLineData.indexOf("{", stopPos);
			stopPos = strLineData.indexOf("}", startPos + 1);
			if ((startPos != -1) && (stopPos != -1)) {
				strFormat = strLineData.substring((startPos + 1), ((stopPos - startPos - 1)+(startPos+1)));
			}
			strFormat = strFormat.trim();
		 // ' Associations - After format is over and till EOL
			if (stopPos != -1) {
				startPos = stopPos + 1;
				strArrayAsocs = strLineData.substring(startPos);
			}
			
			if ((stopPos == -1) && (intDescStopPos != -1)) {
				startPos = intDescStopPos+1;
				strArrayAsocs = strLineData.substring(startPos);
			}
			strArrayAsocs = strArrayAsocs.trim();
		}
		//set the data in global variables
		this.strLineData = strLineData;
		this.strMnemonic = strMnemonic;
		this.strUnits = strUnits;
		this.strValue = strValue;
		this.strDescription = strDescription;
		this.strFormat = strFormat;
		this.strArrayAsocs = strArrayAsocs;
	}
	
	/**
	* Method processLogSection
	*Description: This method processes the well log data by reading the block of data from the LAS file
	* @param strSrcFileFullName input LAS file name
	* @param objLasData Data of the input LAS file
	*
	*/  
	public void processLogSection(String strSrcFileFullName, LasData objLasData) {
		//System.out.println(" in processLogSection()");
		pwPrintWriter.println(" in processLogSection()");
		int intLoopVar = 0;
		String strSearchString = "";
		String strDestFolder = "";
		String strSrcFileContents = "";
		try {
			//get the destination folder
			strDestFolder = objLasData.getDestFile();
			//Get ~A block for LAS 2.0
			String strPattern  = "~A";
			//strSrcFileContents = getSrcFileDataBlock(strSrcFileFullName, "");
			strSrcFileContents = getSrcFileDataBlock(strSrcFileFullName, "");
			//strSrcFileContents = getSrcFileDataBlock(strSrcFileFullName, strPattern);//replaced empty string with strPattern by ravi
			//System.out.println(" after getSrcFileDataBlock()");
			pwPrintWriter.println(" after getSrcFileDataBlock()");
			
			Pattern pattern = Pattern.compile(strPattern,Pattern.CASE_INSENSITIVE);
			/* Added below code for generating multiple well log xml files as per the runs performed STARTS here*/
			String[] arStrResult = pattern.split(strSrcFileContents);
			//System.out.println("length ..."+arStrResult.length);
			
			for (int i=1; i<arStrResult.length; i++) {
				arStrResult[i] = strPattern + arStrResult[i];
				pwPrintWriter.println("result after ="+i+"^^^^^^^^^^"+arStrResult[i]);
			}
			/* Added below code for generating multiple well log xml files as per the runs performed ENDS here*/
			
			Matcher matcher = pattern.matcher(strSrcFileContents);
			
			// Find all matches
			while (matcher.find()) {
				// Get the matching string
				strSearchString = matcher.group();
				//System.out.println(" in while Search string .."+strSearchString);
				//processCustomLogSection(strSrcFileFullName,strSearchString,objLasData, 0, strDestFolder);
				//commented below line for multiple well log files
				//processCustomLogSection(strSrcFileFullName,strSearchString,objLasData, 0, strDestFolder,strSrcFileContents);
				
				processCustomLogSection(strSrcFileFullName,strSearchString,objLasData, (intLoopVar+1), strDestFolder,arStrResult[intLoopVar+1]);
				//System.out.println("$$$$$$$$$$ processed ");
				pwPrintWriter.println("$$$$$$$$$$ processed ");
				intLoopVar++;
			}
		}catch (Exception e) {
				System.out.println("Exception is "+e.getMessage());
				pwPrintWriter.println("Exception is "+e.getMessage());
		}
	}
	
	/**
	* Method processLogSection
	*Description: This method reads the run parameter data and generates the WITMSL output files
	* @param strSrcFileFullName LAS file name to be processed
	* @param strSearchString SearchString from the LAS file
	* @param objLasData Data from the input LAS file
	* @param intLoopVar variable for section count
	* @param strDestFolder Destination folder to place the generated WITSML files
	* @param strSrcFileContents FileContents of the LAS File.
	*
	*/  
	public void processCustomLogSection(String strSrcFileFullName,String strSearchString,LasData objLasData,int intLoopVar, String strDestFolder, String strSrcFileContents) {
		try{
			//read the run parameter data
			//readRunParametersSectionData(strSrcFileFullName, objLasData, strSearchString); 22/2/06
			//get the mapping file
			MappingReader objMappingReader = new MappingReader();
			//System.out.println("The mapping file is ??????????????????? " + getMappingFile());
			pwPrintWriter.println("The mapping file is ??????????????????? " + getMappingFile());
			//get the DOM tree for well log section
			org.w3c.dom.Document logDoc = objMappingReader.createDocument("wellLogs",getMappingFile());  
			pwPrintWriter.println(" ********** elements for wellLogs section in processCustomLogSection is "+objMappingReader.getMappings("wellLogs"));
			//System.out.println(" document created .."+logDoc);
			pwPrintWriter.println(" document created .."+logDoc);
		
			int intIndex  = strSearchString.indexOf("_");
			String strDocName = "'";
			//set the document name
			if(intIndex != -1) {
				strDocName = strSearchString.substring(0, intIndex);
				if(strDocName.indexOf("~") != -1) {
					strDocName = strDocName.replaceAll("~", "");
					strDocName = strDocName.trim();
				}
			}
			//write the default header content
			//writeDefaultHeaderContent(logDoc , strDocName);
			
			//write the log section data
			writeLogSectionData(strSrcFileFullName, strSearchString, logDoc, objLasData,objMappingReader, strSrcFileContents,intLoopVar);
			//generate the WITSML files
			generateWITSML(logDoc,"wellLog",strDestFolder,intLoopVar) ;
		}catch(Exception e) {
			System.out.println("error " + e.getMessage());
			pwPrintWriter.println("error " + e.getMessage());
		}
	 }
	 
	/**
	* Method writeLogSectionData
	*Description: This method write the log section data by reading the DOM tree
	* @param strSrcFileFullName LAS file name
	* @param strSearchString SearchString from the LAS file
	* @param logDoc DOM representation of the XML file
	* @param objLasData Data of the LAS file
	* @param objMappingReader Mapping class to get  the LAS-WITSML mappings
	* @param strSrcFileContents Mapping class to get  the LAS-WITSML mappings
	*/   
	 public void writeLogSectionData(String strSrcFileFullName,String strSearchString , org.w3c.dom.Document logDoc,LasData objLasData,MappingReader objMappingReader, String strSrcFileContents,int intRunNumber){
	try{	
		//write the well name
		findElementAndWriteData(logDoc, "nameWell", objLasData.getWell(),"wellLog");
		
		//write the well bore name, commented the below code assuming the well is a vertical well. Uncommented as per UAT Testing
		findElementAndWriteData(logDoc, "nameWellbore", objLasData.getWell(),"wellLog");
		findElementAndWriteData(logDoc, "name", objLasData.getWell(),"wellLog"); 
		
		/*findElementAndWriteData(logDoc, "nameWellbore", "--","wellLog");
		findElementAndWriteData(logDoc, "name", "--","wellLog");*/
		
		//write the well log identifier
		findElementAndWriteAttributeData(logDoc, "wellLog", "uidWell", objLasData.getUWI());
		//set the well bore and well log identifier to 1 assuming the well is a vertical well
		findElementAndWriteAttributeData(logDoc, "wellLog", "uidWellbore", "1");
		findElementAndWriteAttributeData(logDoc, "wellLog", "uid", "1");
		
		//search for logheader in DOM tree
		//NodeList nlLogHeader  = logDoc.getElementsByTagName("logHeader"); //commented as per new stylesheet
		NodeList nlLogHeader  = logDoc.getElementsByTagName("wellLog");

		//if there are child nodes, write the well section data as per the mappings
		if ((nlLogHeader != null) && (nlLogHeader.getLength() > 0)) {
			//get the mappings for well log section
			Hashtable htMapping = objMappingReader.getMappings("wellLogs");
			
			//System.out.println(" ********** number of elements for wellLogs section is "+htMapping.size());
			pwPrintWriter.println(" ********** number of elements for wellLogs section is "+htMapping.size());
			//System.out.println(" ********** elements for wellLogs section is "+htMapping);
			pwPrintWriter.println(" ********** elements for wellLogs section is "+htMapping);
			//get the well section data of the LAS file
			ArrayList alWellSectionData = objLasData.getWellSectionData();
			//System.out.println(" Well section LAS data size "+alWellSectionData.size());
			pwPrintWriter.println(" Well section LAS data size "+alWellSectionData.size());
			//set the start depth, stop depth, depth increment , index type	
			if ((alWellSectionData != null) && (alWellSectionData.size() > 0)) {
				for(int intSectionIndex = 0; intSectionIndex < alWellSectionData.size(); intSectionIndex++) {
					LasSectionInfo objLasSectionInfo = (LasSectionInfo) alWellSectionData.get(intSectionIndex);
					String strLas = objLasSectionInfo.getMnemonic();
					//System.out.println("strLas "+strLas);
					if (htMapping.containsKey(strLas))	{ //added for null pointer exception
						if(strLas.equalsIgnoreCase("SRVC") || strLas.equalsIgnoreCase("STRT") || strLas.equalsIgnoreCase("STOP") || strLas.equalsIgnoreCase("NULL") || strLas.equalsIgnoreCase("STEP") || strLas.equalsIgnoreCase("DATE")){
							String strWitsml = (String) htMapping.get(strLas); 
							//set the start depth
							if (strLas.equalsIgnoreCase("STRT")) {
								objLasData.setStart(new Double(objLasSectionInfo.getValue()));
								/*//set the index type
								if(objLasData.getUnit() != "S") {
									findElementAndWriteData(logDoc, "indexType", "MeasuredDepth","serviceCompany");
								}*/
							}
							//set the stop depth
							if (strLas.equalsIgnoreCase("STOP")) {
								objLasData.setStop(new Double(objLasSectionInfo.getValue()));
							}
							//set the depth increment 
							String strValue = objLasSectionInfo.getValue();
							
							
							//added the below if block on 29-05-2006 for formatting date as per Nagi's suggestion STARTS here
							if (strLas.equalsIgnoreCase("DATE")) {
								SimpleDateFormat dfDateFormat = new SimpleDateFormat("dd-MMM-yyyy");
								//System.out.println("DDate ..."+strValue+"::::");
								Date dtDate = dfDateFormat.parse(strValue);
								dfDateFormat.applyPattern("yyyy-MM-dd'T'hh:mm:ss.sss");
								strValue = dfDateFormat.format(dtDate);
							}
							//added the below if block on 29-05-2006 for formatting date as per Nagi's suggestion ENDS here
							
							//write the data to output file
							findElementAndWriteData(logDoc, strWitsml, strValue,"");
							
							//written code to remove step and null under welllogs  on 11/May/2006
							if (strLas.equalsIgnoreCase("STEP") || strLas.equalsIgnoreCase("NULL")) {
								NodeList nlStep  = logDoc.getElementsByTagName(strWitsml);
								Node ndStep = (Element)nlStep.item(0);
								//System.out.println("The node name is " + ndStep.getParentNode().getNodeName());
								 Node ndroot = ndStep.getParentNode().removeChild(ndStep);
							}
							
							//written code to remove step and null under welllogs  on 11/May/2006
							if (strLas.equalsIgnoreCase("STRT")) {
								NodeList nlStart  = logDoc.getElementsByTagName(strWitsml);
								Node ndStart = (Element)nlStart.item(0);
								//System.out.println("The node name is " +ndStart.getNodeName());
								String strParentNode = (String)ndStart.getParentNode().getNodeName();
								//System.out.println("The parent node name is " + ndStart.getParentNode().getNodeName());
								if (strParentNode.equalsIgnoreCase("wellLog"))
								{
									Node ndNewNode = logDoc.createElement("indexType");
									//System.out.println("after creating indextype");
									Node ndNew = ndStart.getParentNode().insertBefore(ndNewNode,ndStart);
									//set the index type
									if(objLasData.getUnit() != "S") {
										//findElementAndWriteData(logDoc, "indexType", "MeasuredDepth","serviceCompany");
									}
								}
								
									NodeList nlIndexType  = logDoc.getElementsByTagName("indexType");
									for(int intNodeIndex=0; intNodeIndex<nlIndexType.getLength(); intNodeIndex++) {
										Node ndIndexType = nlIndexType.item(intNodeIndex);
										if (intNodeIndex == 1)
										{
										//System.out.println("The node is >>>> " + ndIndexType);
										Node ndIndexTypeRemove = ndIndexType.getParentNode().removeChild(ndIndexType);
										//System.out.println("Index Type removed");
										}
									}
									//set the mnemonic
									/*if (ndChildNode.getNodeName().equals("mnemonic")) 
									
									Node ndIndexType = ndStart.getParentNode().removeChild(ndStep);*/
								
							}
							//write the data to output file
							//findElementAndWriteData(logDoc, strWitsml, strValue,"");
						}
					} //end htMapping.containsKey
				} //end for
				
				//write index units to output file
				findElementAndWriteData(logDoc, "indexUnits", objLasData.getUnit(),"");
				
				
				
				
			} //end alWellSectionData.size() > 0
			
			//write log parameter elements
			createLogParamElements(objLasData, logDoc, nlLogHeader.item(0));
			
			logDoc = createDataElements("logCurveElements",logDoc,nlLogHeader.item(0));
			
			//System.out.println("before log defination");
			//Create the logCurveInfo elements and add to logHeader
			createLogDefnElements(objLasData, logDoc, nlLogHeader.item(0));
			
			logDoc = createDataElements("blockInfoElements",logDoc,nlLogHeader.item(0));
			
			//write the block info
			NodeList nlBlockInfo  = logDoc.getElementsByTagName("blockInfo");
			//System.out.println("before Node data1111");
			if ((nlBlockInfo != null) && (nlBlockInfo.getLength() > 0)) {
				//set the block id to 0	 as block processing is not being used now
				
				//System.out.println("before Node data22222");
				//findElementAndWriteAttributeData(logDoc, "blockInfo", "blockId", "0");
				findElementAndWriteAttributeData(logDoc, "blockInfo", "blockId", "0");
				//System.out.println("before Node data 3333");
				//Create the blockInfo elements and add to logHeader
				createBlockDefnElements(objLasData, logDoc, nlBlockInfo.item(0),intRunNumber);
				//System.out.println("before Node data 4444");
			}
			
			//System.out.println("after Node data");
			
			logDoc = createDataElements("logDataElements",logDoc,nlLogHeader.item(0));
			
			
			
		}
					
			//get the logdata node and write the log data
			NodeList nlLogData = logDoc.getElementsByTagName("logData");
			//process/write the log data
			if ((nlLogData != null) && (nlLogData.getLength() > 0)) {
				processLogData(strSrcFileFullName, new ArrayList(), strSearchString, 0, logDoc, strSrcFileContents, objLasData);
				
			}
			
			//added below 2 lines on 29-05-2006 as per Nagi's suggestion
			DateFormat dfDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss.sss");
			Date dtDate = new Date();
						
			//to print sysdate in dTimCreation under common Data
			findElementAndWriteData(logDoc, "dTimCreation", dfDateFormat.format(dtDate).toString(),"");
			//findElementAndWriteData(logDoc, "dTimCreation", new java.util.Date().toString(),"");
			
			//added below 2 lines on 29-05-2006 as per Nagi's suggestion to remove validation errors
			//to print sourceName under common Data
			findElementAndWriteData(logDoc, "sourceName", "--","");
		
	}
	catch(Exception e)
	 {
		System.out.println("The error in log data " + e);
	 } 
	}
	
	/**
	* Method createLogParamElements
	*Description: This method writes the log param information
	* @param objLasData Data from the LAS file
	* @param logDoc DOM representation of the documetn
	* @param parentNode parentNode for which the log parameters are to be appended
	*
	*/
	public void createLogParamElements(LasData objLasData,org.w3c.dom.Document logDoc, Node parentNode){
		LasSectionInfo objLasSectionInfo=null;
		Element elemParam = null;
		//get the log parameters
		ArrayList alLogParam=objLasData.getLogParameters();
		//System.out.println("Writing Log parameters .."+alLogParam.size());
		int intParamIndex = 1;
		//get the log parameter information and write the information to a DOM tree
		for(int intLoopVar = 0; intLoopVar < alLogParam.size(); intLoopVar++){
			//get the log parameter information 
			objLasSectionInfo = (LasSectionInfo)alLogParam.get(intLoopVar);
			boolean blnParamSelected = objLasSectionInfo.getIsSelected().booleanValue();
			if (blnParamSelected) {
				//set the log parameter information
				elemParam = logDoc.createElement("logParam");
				
				Text txtNode = logDoc.createTextNode(objLasSectionInfo.getValue());
				elemParam.appendChild(txtNode);
				elemParam.setAttribute("index",new String((intParamIndex++)+""));
				//elemParam.setAttribute("index",new String((intLoopVar+1)+""));
				elemParam.setAttribute("description",objLasSectionInfo.getDescription());
				elemParam.setAttribute("name",objLasSectionInfo.getMnemonic());
				
				if ((objLasSectionInfo.getUnit() == null) || objLasSectionInfo.getUnit().equals("")) {
					elemParam.setAttribute("uom","N/A");
				} else {
					//commented the below line on 22/6/2006 for uom mapping suggested by POSC
					//elemParam.setAttribute("uom",objLasSectionInfo.getUnit());
					
					//added the below line on 22/6/2006 for uom mapping suggested by POSC STARTS here
					if ((htUnitMapping != null)  && (htUnitMapping.size() > 0)){
						if (htUnitMapping.containsKey(objLasSectionInfo.getUnit())) {
							//System.out.println("LAS UNIT .. "+objLasSectionInfo.getUnit()+" >>>> WITSML UNIT .. "+(String) htUnitMapping.get(objLasSectionInfo.getUnit()));
							elemParam.setAttribute("uom",(String) htUnitMapping.get(objLasSectionInfo.getUnit()));
						
						} else {
							//System.out.println(" no mapping for unit ..");
							elemParam.setAttribute("uom",objLasSectionInfo.getUnit());
						}
					}
					//added the below line on 22/6/2006 for uom mapping suggested by POSC ENDS here
				}
				
				//elemParam.setAttribute("uom",objLasSectionInfo.getUnit());
								
				//append the above information to parent node
				parentNode.appendChild(elemParam);
			}
		}
	}
	
	/**
	* Method createLogDefnElements
	*Description: This method writes the log curve information
	* @param objLasData Data from the LAS file
	* @param logDoc DOM representation of the object
	* @param ndParentNode parentNode to append the curve definitions of the LAS file
	*
	*/
	public void createLogDefnElements(LasData objLasData,org.w3c.dom.Document logDoc, Node ndParentNode){
	
		LasSectionInfo objLasSectionInfo = null;
		Element elemParam, childElem;
		Text txtNode = null;
		//arrray list to hold selected curve definitions
		ArrayList alSelectedDefn = new ArrayList();
		//get the curve definitons of the LAS file
		ArrayList alLogDefn = objLasData.getLogDefns();
		
		//if there are curve definitons
		if (alLogDefn != null) {
			alSelectedCurves = new ArrayList();	
			//get the curve definitions size
			int intLogDefnSize = alLogDefn.size();
			//System.out.println(" createLogDefnElements log defn size is "+intLogDefnSize);
			for (int intLogDefnIndex = 0; intLogDefnIndex < intLogDefnSize; intLogDefnIndex++) {
				objLasSectionInfo = (LasSectionInfo)alLogDefn.get(intLogDefnIndex);
				boolean blnSelected = objLasSectionInfo.getIsSelected().booleanValue();
				//check if the curve is selected, if the curve is selected add it to selected curves definitions
				if (blnSelected) { 
					alSelectedDefn.add(objLasSectionInfo);
					alSelectedCurves.add(objLasSectionInfo.getMnemonic());
				}
			}
		} else {
			//System.out.println(" createLogDefnElements() , No curve definitions are available");
		}
		//System.out.println(" >>>> No. of Curves selected .."+alSelectedDefn.size());
		pwPrintWriter.println(" >>>> No. of Curves selected .."+alSelectedDefn.size());
		//get the selected curves
		if (alSelectedDefn != null) {
			//get the selected definitions size
			int intSelectedLogDefnSize = alSelectedDefn.size();
			//for each of the above curves, write the log curve info the DOC tree
			for(int intCurveVar = 0; intCurveVar < intSelectedLogDefnSize; intCurveVar++){
					NodeList nlLogCurveInfo = logDoc.getElementsByTagName("logCurveInfo");
					Node ndLogCurveInfo, ndDupLogCurve;
					if ((nlLogCurveInfo != null) && (nlLogCurveInfo.getLength() > 0)) {
						ndLogCurveInfo = nlLogCurveInfo.item(0);
						//get the curve defintion
						objLasSectionInfo = (LasSectionInfo)alSelectedDefn.get(intCurveVar);
						//get the node from DOM tree and write the curve data
						if (intCurveVar == 0) {
							elemParam = (Element) nlLogCurveInfo.item(0);
							//set the curve id
							//elemParam.setAttribute("curveId", new Integer(intCurveVar).toString());changed due to new changes
							elemParam.setAttribute("uid", new Integer(intCurveVar).toString());
							
							if (ndLogCurveInfo.hasChildNodes()) {
								NodeList nlChildLogCurveInfo = ndLogCurveInfo.getChildNodes() ;
								for(int intNodeIndex=0; intNodeIndex<nlChildLogCurveInfo.getLength(); intNodeIndex++) {
									Node ndChildNode = nlChildLogCurveInfo.item(intNodeIndex);
									//set the mnemonic
									if (ndChildNode.getNodeName().equals("mnemonic")) {
										txtNode = logDoc.createTextNode(objLasSectionInfo.getMnemonic());
										ndChildNode.appendChild(txtNode); 
									} else if (ndChildNode.getNodeName().equals("classWitsml")) {
										//set the classWitsml
										txtNode = logDoc.createTextNode(objLasSectionInfo.getMnemonic().toLowerCase());
										ndChildNode.appendChild(txtNode);
									} else if (ndChildNode.getNodeName().equals("unit")) {
										//set the unit
										//commented the below line on 22/6/2006 for uom mapping suggested by POSC
										//txtNode = logDoc.createTextNode(objLasSectionInfo.getUnit());
										
										//added the below line on 22/6/2006 for uom mapping suggested by POSC STARTS here
										if ((htUnitMapping != null)  && (htUnitMapping.size() > 0)){
											if (htUnitMapping.containsKey(objLasSectionInfo.getUnit())) {
												//System.out.println("LAS UNIT .. "+objLasSectionInfo.getUnit()+" >>>> WITSML UNIT .. "+(String) htUnitMapping.get(objLasSectionInfo.getUnit()));
												txtNode = logDoc.createTextNode((String) htUnitMapping.get(objLasSectionInfo.getUnit()));
											} else {
													//System.out.println(" no mapping for unit ..");
													txtNode = logDoc.createTextNode(objLasSectionInfo.getUnit());
											}
										}
										//added the below line on 22/6/2006 for uom mapping suggested by POSC ENDS here
																				
										ndChildNode.appendChild(txtNode);				
									}else if (ndChildNode.getNodeName().equals("nullValue")) {
										
										//set the null value
										
											if (ndChildNode.getFirstChild() instanceof Text) {
										
											ndChildNode.removeChild(ndChildNode.getFirstChild());
													//((Text) ndChildNode.getFirstChild()).setData("");
											} 
											txtNode = logDoc.createTextNode((new Double(objLasData.getNullValue())).toString());
											ndChildNode.appendChild(txtNode);
																				
									}else if (ndChildNode.getNodeName().equals("curveDescription")) {
										//set the curve description
										txtNode = logDoc.createTextNode(objLasSectionInfo.getDescription());
										ndChildNode.appendChild(txtNode);
									}else if (ndChildNode.getNodeName().equals("typeLogData")) {
										//set the log data data type
										//txtNode = logDoc.createTextNode("Float");
										txtNode = logDoc.createTextNode("double");
										ndChildNode.appendChild(txtNode);
									}
								} //end for
							} //end ndLogCurveInfo.hasChildNodes()
						} else {
								//for other curves, create the log curve info nodes by cloning the node from the mapping file and write the curve data
								ndDupLogCurve = ndLogCurveInfo.cloneNode(true);
								if (ndDupLogCurve.hasChildNodes()) {
									NodeList nlDupChildLogCurveInfo = ndDupLogCurve.getChildNodes() ;
									if (nlDupChildLogCurveInfo != null) {
										for(int intNodeIndex = 0; intNodeIndex < nlDupChildLogCurveInfo.getLength(); intNodeIndex++) {
											Node ndChildNode = nlDupChildLogCurveInfo.item(intNodeIndex);
											//set the curve mnemonic	
											if (ndChildNode.getNodeName().equals("mnemonic")) {
												//as the node is created by cloning, remove the existing curve mnemonic and set the mnemonic
												if (ndChildNode.getFirstChild() instanceof Text) {
													ndChildNode.removeChild(ndChildNode.getFirstChild());
													((Text) ndChildNode.getFirstChild()).setData("");
												} 
												txtNode = logDoc.createTextNode(objLasSectionInfo.getMnemonic());
												ndChildNode.appendChild(txtNode);
											} 
											//set the classWitsml
											if (ndChildNode.getNodeName().equals("classWitsml")) {
											//as the node is created by cloning, remove the existing curve classWitsml and set the classWitsml
												if (ndChildNode.getFirstChild() instanceof Text) {
													ndChildNode.removeChild(ndChildNode.getFirstChild());
													((Text) ndChildNode.getFirstChild()).setData("");
												} 
												txtNode = logDoc.createTextNode(objLasSectionInfo.getMnemonic().toLowerCase());
												ndChildNode.appendChild(txtNode);
											}
											//set the unit
											if (ndChildNode.getNodeName().equals("unit")) {
											//as the node is created by cloning, remove the existing curve unit and set the unit
												if (ndChildNode.getFirstChild() instanceof Text) {
													ndChildNode.removeChild(ndChildNode.getFirstChild());
													((Text) ndChildNode.getFirstChild()).setData("");
												} 
												
												//set the unit
												//commented the below line on 22/6/2006 for uom mapping suggested by POSC
												//txtNode = logDoc.createTextNode(objLasSectionInfo.getUnit());
												
												//added the below line on 22/6/2006 for uom mapping suggested by POSC STARTS here
												if ((htUnitMapping != null)  && (htUnitMapping.size() > 0)){
													if (htUnitMapping.containsKey(objLasSectionInfo.getUnit())) {
														//System.out.println("LAS UNIT .. "+objLasSectionInfo.getUnit()+" >>>> WITSML UNIT .. "+(String) htUnitMapping.get(objLasSectionInfo.getUnit()));
														txtNode = logDoc.createTextNode((String) htUnitMapping.get(objLasSectionInfo.getUnit()));
													} else {
															//System.out.println(" no mapping for unit ..");
															txtNode = logDoc.createTextNode(objLasSectionInfo.getUnit());
													}
												}
												//added the below line on 22/6/2006 for uom mapping suggested by POSC ENDS here
												
												ndChildNode.appendChild(txtNode);
											}
											
											
											//commented the block as the null value will the same as the node that is cloned
											/*if (ndChildNode.getNodeName().equals("nullValue")) {
												
												if (ndChildNode.getFirstChild() instanceof Text) {
													ndChildNode.removeChild(ndChildNode.getFirstChild());
												} 
												txtNode = logDoc.createTextNode((new Double(objLasData.getNullValue())).toString());
												ndChildNode.appendChild(txtNode);
											} */
											
											//set curve description
											if (ndChildNode.getNodeName().equals("curveDescription")) {
											//as the node is created by cloning, remove the existing curve description and set the curve description
											if (ndChildNode.getFirstChild() instanceof Text) {
													ndChildNode.removeChild(ndChildNode.getFirstChild());
													((Text) ndChildNode.getFirstChild()).setData("");
												} 
												txtNode = logDoc.createTextNode(objLasSectionInfo.getDescription());
												ndChildNode.appendChild(txtNode);
											}
											
											//set log data datatype 
											if (ndChildNode.getNodeName().equals("typeLogData")) {
											//as the node is created by cloning, remove the existing datatype of log data and set the log data datatype
												if (ndChildNode.getFirstChild() instanceof Text) {
													ndChildNode.removeChild(ndChildNode.getFirstChild());
													((Text) ndChildNode.getFirstChild()).setData("");
												} 
												//txtNode = logDoc.createTextNode("Float");//commented for validating the xml and float is not a valid value
												txtNode = logDoc.createTextNode("double");
												ndChildNode.appendChild(txtNode);
											}
										} //end for 
									} //end nlDupChildLogCurveInfo != null
								} //ndDupLogCurve.hasChildNodes()
							//append this new log curve info to the parent node
							ndLogCurveInfo.getParentNode().appendChild(ndDupLogCurve);
							//set the curve id
							elemParam = (Element) nlLogCurveInfo.item(intCurveVar);
							//elemParam.setAttribute("curveId", new Integer(intCurveVar).toString());changed due to new changes
							elemParam.setAttribute("uid", new Integer(intCurveVar).toString());
						} //end else
					} //end nlBlockCurveInfo != null
					//System.out.println(" <<<<<< written curve info for curve "+intCurveVar);
				} //end SelectedLogDefn for loop
			} //end alSelectedDefn != null
	}
	
	/**
	* Method createBlockDefnElements
	*Description: This method writes the block information
	* @param objLasData Data from the LAS file
	* @param logDoc DOM representation of the document
	* @param parentNode to append the block information of the LAS file
	*
	*/
	public void createBlockDefnElements(LasData objLasData,org.w3c.dom.Document logDoc, Node parentNode,int intRunNumber){
	
		LasSectionInfo objLasSectionInfo = null;
		ArrayList alSelectedDefn = new ArrayList();
		Element elemParam, childElem;
		Text txtNode;
		//write block information
		if (parentNode != null) {
			//set the block information, like pass, description, index units, start index, end index and index type
			//commented below line for displaying multiple runs. 22/6/06
			//findElementAndWriteData(logDoc, "pass", new Integer(1).toString(), ""); 
			//added below line for displaying multiple runs.
			findElementAndWriteData(logDoc, "pass", new Integer(intRunNumber).toString(), "");
			findElementAndWriteData(logDoc, "description", new String("main log section"), "");
			
			findElementAndWriteData(logDoc, "indexUnits", objLasData.getUnit(),"");
			findElementAndWriteData(logDoc, "startIndex", objLasData.getStart().toString(),"");
			findElementAndWriteData(logDoc, "endIndex", objLasData.getStop().toString(),"");
			//stepincrement
			findElementAndWriteData(logDoc, "stepIncrement", objLasData.getStepValue().toString(),"direction");
			findElementAndWriteAttributeData(logDoc, "stepIncrement", "uom", objLasData.getUnit());
			
			if(objLasData.getUnit() != "S") {
				findElementAndWriteData(logDoc, "indexType", "measured depth","startIndex");
			}
				
			if (objLasData != null) {
				if ((objLasData.getStart() != null) && (objLasData.getStop() != null)) {
					double dbStartValue = objLasData.getStart().doubleValue();
					double dbStopValue = objLasData.getStop().doubleValue();
					//set the direction
					if (dbStartValue > dbStopValue) {
						findElementAndWriteData(logDoc, "direction", "decreasing","");
					} else {
						findElementAndWriteData(logDoc, "direction", "increasing","");
					} 
				}
			}
			
			findElementAndWriteData(logDoc,"nullValue",(new Double(objLasData.getNullValue())).toString(),"direction");
												
			
						
			//get curve defintions
			ArrayList alLogDefn = objLasData.getLogDefns();
			alSelectedCurves = new ArrayList();
			//get only selected curve definitions
			if (alLogDefn != null) {
				int intLogDefnSize = alLogDefn.size();
				//System.out.println(" createBlockDefnElements log defn size is "+intLogDefnSize);
				for (int intLogDefnIndex =0; intLogDefnIndex < intLogDefnSize; intLogDefnIndex++) {
					objLasSectionInfo = (LasSectionInfo)alLogDefn.get(intLogDefnIndex);
					//add the selected curve information to array list
					boolean blnSelected = objLasSectionInfo.getIsSelected().booleanValue();
					if (blnSelected) { 
						alSelectedDefn.add(objLasSectionInfo);
						alSelectedCurves.add(objLasSectionInfo.getMnemonic());
					}
				}
				//System.out.println(" %%%%%%%% createBlockDefnElements selected log defn size is "+alSelectedDefn.size());
				pwPrintWriter.println(" %%%%%%%% createBlockDefnElements selected log defn size is "+alSelectedDefn.size());
			} else {
				//System.out.println(" No curve definitons exist ");
				pwPrintWriter.println(" No curve definitons exist ");
			}
			//display blockcurveinfo from alSelectedDefn array
			if (alSelectedDefn != null) {
				int intSelectedLogDefnSize = alSelectedDefn.size();
				//create the block info for each of these selected curves
				for(int intLoopVar = 0; intLoopVar < intSelectedLogDefnSize; intLoopVar++){
				
					//System.out.println("After for loop");
					NodeList nlBlockCurveInfo = logDoc.getElementsByTagName("blockCurveInfo");
					Node ndBlockCurveInfo, ndDupBlockCurve;
					
					if ((nlBlockCurveInfo != null) && (nlBlockCurveInfo.getLength() > 0)) {
						ndBlockCurveInfo = nlBlockCurveInfo.item(0);
						//get the block info node from the mapping file
						if (intLoopVar == 0) {
						//System.out.println("The node is elRoot " + logDoc.getDocumentElement());
							if (ndBlockCurveInfo.hasChildNodes()) {
								NodeList nlChildBlockCurveInfo = ndBlockCurveInfo.getChildNodes() ;
								//System.out.println("<<<<<< no. of. elements in BlockCurveInfo >>> "+nlChildBlockCurveInfo.getLength());
								for(int intNodeIndex = 0; intNodeIndex < nlChildBlockCurveInfo.getLength(); intNodeIndex++) {
									Node ndChildNode = nlChildBlockCurveInfo.item(intNodeIndex);
									//System.out.println("The block nodes are " + ndChildNode.getNodeName());
									//set the curve id 								
									if (ndChildNode.getNodeName().trim().equalsIgnoreCase("curveId")) {
									        txtNode = logDoc.createTextNode(new Integer(intLoopVar).toString());
										ndChildNode.appendChild(txtNode); 
									} else if (ndChildNode.getNodeName().trim().equalsIgnoreCase("columnIndex")) {
										//set the column index
										txtNode = logDoc.createTextNode(new Integer((intLoopVar+1)).toString());
										ndChildNode.appendChild(txtNode);
									}else if (ndChildNode.getNodeName().trim().equalsIgnoreCase("minIndex")) {
										//set the minindex
										if (ndChildNode.getNodeValue() != null) {
											txtNode = logDoc.createTextNode(objLasData.getStart().toString());
											ndChildNode.appendChild(txtNode);
										}else{
											txtNode = logDoc.createTextNode(objLasData.getStart().toString());
											ndChildNode.appendChild(txtNode);
										}
									}else if (ndChildNode.getNodeName().trim().equalsIgnoreCase("maxIndex")) {
										//set the maxindex
										if (ndChildNode.getNodeValue() != null) {
											txtNode = logDoc.createTextNode(objLasData.getStop().toString());
											ndChildNode.appendChild(txtNode);
										}else{
											txtNode = logDoc.createTextNode(objLasData.getStop().toString());
											ndChildNode.appendChild(txtNode);
										}
									}
								}
							}
						} else {
							//System.out.println("IN else loop" + intLoopVar);
							objLasSectionInfo = (LasSectionInfo)alSelectedDefn.get(intLoopVar);
							//clone blockinfo node from the mapping file										
							ndDupBlockCurve = ndBlockCurveInfo.cloneNode(true);
							//System.out.println("The node is ndDupBlockCurve " + ndDupBlockCurve);
							if (ndDupBlockCurve.hasChildNodes()) {
								NodeList nlDupChildBlockCurveInfo = ndDupBlockCurve.getChildNodes() ;
								//System.out.println("<<<<<< no. of. elements in BlockCurveInfo >>> "+nlDupChildBlockCurveInfo.getLength());
								if (nlDupChildBlockCurveInfo != null) {
								
									for(int intNodeIndex = 0; intNodeIndex < nlDupChildBlockCurveInfo.getLength(); intNodeIndex++) {
										Node ndChildNode = nlDupChildBlockCurveInfo.item(intNodeIndex);
										//set curveId
										 if (ndChildNode.getNodeName().trim().equalsIgnoreCase("curveId")) {
											 if (ndChildNode.getFirstChild() instanceof Text) {
												//clear the curve id of the cloned node and set the curve id
												ndChildNode.removeChild(ndChildNode.getFirstChild());
												((Text) ndChildNode.getFirstChild()).setData("");
											} 
												txtNode = logDoc.createTextNode(new Integer(intLoopVar).toString());
												ndChildNode.appendChild(txtNode);
										 }
										 //set columnIndex
										else if (ndChildNode.getNodeName().trim().equalsIgnoreCase("columnIndex")) {
											if (ndChildNode.getFirstChild() instanceof Text) {
												//clear the curve id of the cloned node and set the curve id
												ndChildNode.removeChild(ndChildNode.getFirstChild());
												((Text) ndChildNode.getFirstChild()).setData("");
											} 
											txtNode = logDoc.createTextNode(new Integer((intLoopVar+1)).toString());
											ndChildNode.appendChild(txtNode);
											 
										}
									}
								} //end nlDupChildBlockCurveInfo != null
							} //end ndDupBlockCurve.hasChildNodes()
							//append the block curve info to the parent node
							ndBlockCurveInfo.getParentNode().appendChild(ndDupBlockCurve);
							//System.out.println("out of else loop");
						} //end else
					} //end nlBlockCurveInfo != null
					//System.out.println("out of for loop");
				} //end SelectedLogDefn for loop
			} //alSelectedDefn
		}
		//System.out.println("Coming out of method");
	}    
	
	/**
	* Method readLogDefinitionsData
	*Description: This method will read details of CURVE INFORMATION block in the input LAS File.
	* @param strSrcFileFullName Input LAS file name
	* @param alArrayList Log definitions array list
	* @param strSearchString Search string from the LAS file
	* @param objLasData Data from the LAS file
	*
	*/
	public void readLogDefinitionsData(String strSrcFileFullName, ArrayList alArrayList, String strSearchString, LasData objLasData) {
		
		String strLogDefnBlock = "";
		String strLineData = "";
		int intNextIndex  = -1;
		ArrayList   alCurveDataArray = new ArrayList();
		//Get Curve Information block starting from~Curve till next occurence of ~ character
		// This will work for LAS2.0
		strLogDefnBlock = getSrcFileDataBlock(strSrcFileFullName, "~C");
		//if no curve information block is fund, search if it is LAS 3.0
		if (strLogDefnBlock.length() == 0) {
			//File is Version 3.0, check for Log_Definition
			strSearchString = strSearchString.replaceFirst("PARAMETER", "DEFINITION");
			strLogDefnBlock = getSrcFileDataBlock(strSrcFileFullName, strSearchString);
			if (strLogDefnBlock.length() == 0) {
				//System.out.println(" readLogDefinitionsData() No curve definitions are found ..");
				return;
			}
		}
		//get a line of data from the above data blcok that is read
		intNextIndex = getLineFromBlock(strLogDefnBlock, strLineData, (intNextIndex + 1));
		strLineData = this.strLineData;
		
		// Skip through the block till you reach the first curve definition
		while ((strLineData.indexOf("#") != -1) || (strLineData.indexOf("~") != -1)) {
			intNextIndex = getLineFromBlock(strLogDefnBlock, strLineData, (intNextIndex + 1));
			strLineData = this.strLineData;
		}
		//set the curve data
		while (intNextIndex != -1) {
			//get the blocks from the line of data read
			getLineData(strLineData, strMnemonic, strUnits, strValue, strDescription, strFormat, strArrayAsocs);
			
			//ignore the line as this is a comment in LAS file
			if (!(strLineData.startsWith("#"))) {
				if (objLasData != null) {
					//set the curve data
					objLasSectionInfo = new LasSectionInfo();
					objLasSectionInfo.setIsSelected(new Boolean(true));
					objLasSectionInfo.setMnemonic(strMnemonic);
					objLasSectionInfo.setUnit(strUnits);
					objLasSectionInfo.setValue(strValue);
					objLasSectionInfo.setDescription(strDescription);
					objLasSectionInfo.setFormat(strFormat);
					//add the curve data to the array list
					objLasData.getParameterData().setCurveSectionData(objLasSectionInfo);
				}
			
				if (strMnemonic != "") {
					if (alArrayList != null) {
						alArrayList.add(strMnemonic);
						alArrayList.add(strUnits);
					}
					if  (objLasData != null) {
						alCurveDataArray.add(objLasSectionInfo);
					}
				}
			}
			//get the remaining lines and set the curve data
			intNextIndex = getLineFromBlock(strLogDefnBlock, strLineData, intNextIndex + 1);
			strLineData = this.strLineData;
		}
		//set the curve definitions
		objLasData.setLogDefns(alCurveDataArray);
		//set the LasData
		setLasData(objLasData);
	}
	
	/**
	* Method processLogData
	*Description: This method will read details of ASCII or LOG Data block in the input LAS File and write the log data to DOM tree.
	* @param strSrcFileFullName input LAS file name
	* @param saDataArray Array for holding log data
	* @param strSearchString String to be searching from the LAS file
	* @param intRecLimit Record Limit 
	* @param logDoc DOM representation of the document
	* @param strSrcFileContents FileContents of the File.
	* @param objLasData LASData object
	*/
	public void processLogData(String strSrcFileFullName, ArrayList saDataArray, String strSearchString, int intRecLimit, org.w3c.dom.Document logDoc, String strSrcFileContents, LasData objLasData) {
		String strVers = "";
		String strWrap = "";
		int intDelimiter = 32; // 44--COMMA, 32 -- SPACE
		String strLineData = "";
		int intNextIndex = -1;
		LasSectionInfo objLasSectionInfo;
		try {
			//read version information
			readVersionInformation(strSrcFileFullName, strVers, strWrap, intDelimiter);
			
			//get the Delimiter
			intDelimiter = getDelimiter();
				
			String strLogDataBlock = "";
			// Get ASCII block starting from ~A till next occurence of ~ character
			//strLogDataBlock = getSrcFileDataBlock(strSrcFileFullName, "~A");
			
			int intLogIndex = strSrcFileContents.indexOf("~A");
			//System.out.println("The index of ~A is " + intLogIndex);
			if (intLogIndex != -1) {
				strLogDataBlock = strSrcFileContents.substring(intLogIndex);
			}
			//System.out.println("The strSRc File full name is " + strLogDataBlock);
			if (strLogDataBlock.length() == 0) {
				//File is Version 3.0, check for Log_Definition
				strSearchString = strSearchString.replaceFirst("PARAMETER", "DATA");
				strLogDataBlock = getSrcFileDataBlock(strSrcFileFullName, strSearchString);
				
				if (strLogDataBlock.length() == 0) {
					//System.out.println(" processLogData() No log data is found ..");
					return;
				}
			}
			//get line of data from the block that is read above
			intNextIndex = getLineFromBlock(strLogDataBlock, strLineData, intNextIndex + 1);
			strLineData = this.strLineData;
		
			// Skip through the block till you reach the first log data
			while ((strLineData.indexOf("#") != -1) || (strLineData.indexOf("~") != -1)) {
				intNextIndex = getLineFromBlock(strLogDataBlock, strLineData, intNextIndex + 1);
				strLineData = this.strLineData;
			}
			//get the log data
			while (intNextIndex != -1) {
				if  (saDataArray != null) {
					//get blocks of data from the line
					getDataFromLine(strLineData, intDelimiter, saDataArray);
					strLineData = this.strLineData;
					saDataArray = this.alDataArray;
				}
				//get the remaining lines and add to data array			
				intNextIndex = getLineFromBlock(strLogDataBlock, strLineData, intNextIndex + 1);
				strLineData = this.strLineData;
				//pwPrintWriter.println(" strLineData >>>>> "+strLineData);	
			}
			
			//System.out.println(" <<<<<<<<<< processLogData saDataArray >>>>> "+this.alDataArray.size());	
			pwPrintWriter.println(" <<<<<<<<<< processLogData saDataArray >>>>> "+this.alDataArray.size());	
			//System.out.println(" %%%%%%% SELECTED CURVES SIZE is "+alSelectedCurves.size());
			pwPrintWriter.println(" %%%%%%% SELECTED CURVES SIZE is "+alSelectedCurves.size());
			//if there are selected curves, get the selected curve log data from the LAS file and then write the data to the DOM tree
			if (alSelectedCurves.size() > 0) {
				char chDelimiter = (char) intDelimiter;
				char data[] = {chDelimiter};
				String strDelimiter = new String(data);
				StringBuffer strCurveLineData = new StringBuffer();
				//get curve names
				ArrayList alLogDefn = objLasData.getLogDefns();
				//System.out.println("Curve defns from selection ..."+alLogDefn.size());
				if (alLogDefn != null) {
					for (int intDefnSize = 0; intDefnSize < alLogDefn.size(); intDefnSize++) {
						objLasSectionInfo = (LasSectionInfo) alLogDefn.get(intDefnSize); 
						//strCurveLineData = strCurveLineData + objLasSectionInfo.getMnemonic() + strDelimiter;
						strCurveLineData.append(objLasSectionInfo.getMnemonic());
						strCurveLineData.append(strDelimiter);
					}
				}
				//System.out.println("strCurveLineData before removing last delimiter" +strCurveLineData+":::::");
				//System.out.println("strDelimiter"+strDelimiter);
				//remove the last delimiter and get the curve names
				int intStopIndex = -1;
				intStopIndex = strCurveLineData.toString().lastIndexOf(strDelimiter);
				if (intStopIndex != -1) {
					strCurveLineData = new StringBuffer((strCurveLineData.toString()).substring(0,intStopIndex));
				}
				
				pwPrintWriter.println("strCurveLineData after removing last delimiter" +strCurveLineData+"::::");
				//System.out.println("strCurveLineData after removing last delimiter" +strCurveLineData+"::::");
				
				StringTokenizer strLogDataTokenizer  = new StringTokenizer(strCurveLineData.toString(),strDelimiter);
				//get the number of curves from the header
				int intNumCurves = strLogDataTokenizer.countTokens();
				//get the number of rows in the LAS file
				int intNumRows = this.alDataArray.size()/intNumCurves;
				//System.out.println(" number of intNumRows is "+intNumRows);
				pwPrintWriter.println(" number of intNumRows is "+intNumRows);
				//array for holding log data
				String[][] arCurveList = new String[intNumRows][intNumCurves];
				//System.out.println(" arCurveList size is .."+arCurveList.length); 
				pwPrintWriter.println(" arCurveList size is .."+arCurveList.length); 
				
				int intRowNum = 0;
				int intColNum  =0;
				//copy the log data to an array
				for (int intArrayIndex = 0; intArrayIndex < alDataArray.size(); intArrayIndex++) {
					if (intRowNum < intNumRows) {
						//System.out.println(" intArrayIndex is "+intArrayIndex +"<<<< intRowNum"+intRowNum+">>>>intColNum "+intColNum );
						//System.out.println(" array list element is "+(String) alDataArray.get(intArrayIndex));
						if (intColNum == 0) {
							arCurveList[intRowNum] = new String[intNumCurves];	
						}
						//copy the data to array
						arCurveList[intRowNum][intColNum]=(String)alDataArray.get(intArrayIndex);
						intColNum++;
						
						if(intColNum == intNumCurves) {
							//System.out.println("copied details for rownum"+intRowNum);
							//create a new row
							intColNum = 0;
							intRowNum++;
						}
					}
				}
				/*for (int intArrayIndex=0; intArrayIndex<arCurveList.length; intArrayIndex++) {
					System.out.println("The data of first column  is " + arCurveList[intArrayIndex][0]);
				}*/
				//search for log data node in DOM tree	
				NodeList nlLogData = null;
				Element elemlogData = null;
				if (logDoc != null) {
					nlLogData = logDoc.getElementsByTagName("logData");
					//elemlogData = logDoc.createElement("logData");
				}
				//array for holding selected curves log data
				String[] strSelectedLogData = new String[intNumRows*alSelectedCurves.size()];
				int intTokenNum = 0;
				int intRowIndex = 0;
				while (strLogDataTokenizer.hasMoreTokens()) {
					//get the curve name
					String strCurveName = strLogDataTokenizer.nextToken();
					//check if the curve is selectd, if selected copy the log data into the array
					if (alSelectedCurves.contains(strCurveName)) {
						//System.out.println(" SELECTED curve name .."+strCurveName);
						for (int intCurveIndex = 0; intCurveIndex < arCurveList.length; intCurveIndex++) {
							//copy the data
							strSelectedLogData[intRowIndex] = arCurveList[intCurveIndex][intTokenNum];
							//System.out.println(" data for "+strCurveName+ ">> intRowIndex = .."+intRowIndex+" >>"+strSelectedLogData[intRowIndex]);
							intRowIndex++;
						}
					}
					intTokenNum++;
				}
				//System.out.println("strSelectedLogData"+strSelectedLogData.length+">>>> intNumRows"+intNumRows);
				
				//copy the data to DOM tree
				for(int intIndex = 0; intIndex < intNumRows; intIndex++) {
					//System.out.println("intIndex"+intIndex);
					if ((intNumRows+intIndex) < strSelectedLogData.length) {
					
						//get the log data
						StringBuffer strLogData = new StringBuffer();
						strLogData.append(strSelectedLogData[intIndex]);
						
						StringBuffer strAllCurveData = new StringBuffer();
						int intNextRowNum = 0;
						for (int intSelectedCurveIndex = 0; intSelectedCurveIndex < alSelectedCurves.size(); intSelectedCurveIndex++) {
							
							if (intSelectedCurveIndex == 0) {
								intNextRowNum = intNumRows;
							} else {
								intNextRowNum = (intNextRowNum)+intNumRows;
							}
							
							if (intNextRowNum < strSelectedLogData.length) {
								//concatenate the data with delimiter "," as per the stylesheet
								strAllCurveData.append(strSelectedLogData[((intNextRowNum)+intIndex)] + ",");
						
							}
						
						}
						//remove the last delimiter
						int intCommaIndex = strAllCurveData.toString().lastIndexOf(",");
						if (intCommaIndex != -1) {
							strAllCurveData = new StringBuffer(strAllCurveData.toString().substring(0,intCommaIndex));
						}
						//System.out.println("strAllCurveData"+strAllCurveData);
						//append comma to log data
						strLogData.append("," + strAllCurveData.toString());
						//System.out.println("after , strLogData"+strLogData);
						Element elBlockElem, elDataElem;
						//create log data
						if  (nlLogData != null) {
						//if  (elemlogData != null) {
							//if (nlLogData.getLength() > 0) {		
								/* commented below code as per old stylesheet
								//create block
								elBlockElem = logDoc.createElement("block");
								//set block id as 0(as per xsd), as blocks are not considered now
								elBlockElem.setAttribute("id",new Integer(0).toString());
								//copy the actual log data
								elDataElem = logDoc.createElement("data");
								Text txtNode = logDoc.createTextNode(strLogData);
								elDataElem.appendChild(txtNode);
								//append this data node to block
								elBlockElem.appendChild(elDataElem);
								nlLogData.item(0).appendChild(elBlockElem);*/
								
								//added below code as per new stylesheet
								//create data block
								elDataElem = logDoc.createElement("data");
								//set the id attribute
								elDataElem.setAttribute("id",new Integer(0).toString());
								//get the actual data
								Text txtNode = logDoc.createTextNode(strLogData.toString());
								//copy the actual log data
								elDataElem.appendChild(txtNode);
								//append this data node to logData
								nlLogData.item(0).appendChild(elDataElem);
								//elemlogData.appendChild(elDataElem);
							//}
						} 
					}
				}
					//parentNode.appendChild(elemlogData);//added this line for fixing new element
			} //end selected curves 
		}catch(Exception e ) {
			System.out.println("Exception in processLogData "+e.getMessage());
		}
	}
	
	/**
	* Method processWellSection
	*Description: This writes the well data to DOM tree and generates the WITSML Files.
	* @param strSrcFileFullName The input LAS file name
	* @param objLasData Data from the LAS file
	*
	*/
	public void processWellSection(String strSrcFileFullName, LasData objLasData) {
		//System.out.println(" in processWellSection()");
		try {
			//get the destination folder
			String strDestFolder = objLasData.getDestFile();
			//get the mappings for well section from mapping file
			MappingReader objMappingReader = new MappingReader();
			org.w3c.dom.Document wellDoc = objMappingReader.createDocument("wells",getMappingFile()); 
			//System.out.println(" document created .."+wellDoc);
			
			//write the default header content
			writeDefaultHeaderContent(wellDoc , "wellDocument");
			
			//write well data  
			writeWellSectionData(wellDoc,objLasData,objMappingReader);  
			//generate the well WITSML file
			generateWITSML(wellDoc,"well",strDestFolder,0) ;
		}catch (Exception e) {
			System.out.println("Exception is "+e.getMessage());
		}
	}
	
	/**
	* Method writeDefaultHeaderContent
	*Description: This writes the default header data to DOM tree
	* @param xmlDoc DOM represenation of the Document
	* @param strDocName Document Name
	*
	*/
	public void writeDefaultHeaderContent(org.w3c.dom.Document xmlDoc,String strDocName) {
		//disclaimer	
		String strDisclaimer = "SATYAM is NOT responsible for the quality or correctness of the data contained in this file.";
		
		//file creation date 
		//commented below line on 29-05-2006 as per Nagi's suggestion for formatting date
		//java.util.Date dtCreationDate = new java.util.Date();
		
		//added below 2 lines on 29-05-2006 as per Nagi's suggestion for formatting date
		DateFormat dfDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss.sss");
		Date dtCreationDate = new Date();
				
		//application name
		String strAppName = "LAS-WITSML Data Format Converter";
		
		
		//pwPrintWriter.println("xmlDoc    "+xmlDoc);
		
		// Write document name
		findElementAndWriteData(xmlDoc, "DocumentName", ("well" + " - " +strDocName),"FileCreationInformation");
		
		//FileCreationDate
		//findElementAndWriteData(xmlDoc, "FileCreationDate", dtCreationDate.toString(),"");
		findElementAndWriteData(xmlDoc, "dTimCreation", dfDateFormat.format(dtCreationDate).toString(),"");
		findElementAndWriteData(xmlDoc, "FileCreationDate", dfDateFormat.format(dtCreationDate).toString(),"SoftwareName");
		
		//SoftwareName
		findElementAndWriteData(xmlDoc, "SoftwareName", strAppName,"FileCreationDate");
		
		//Disclaimer
		findElementAndWriteData(xmlDoc, "Disclaimer", strDisclaimer,"FileCreationInformation");
		
		
		//Get AuditTrail segment
		//commented below line on 29-05-2006 as per Nagi's suggestion for formatting date
		//writeAuditTrailData(xmlDoc, dtCreationDate.toString(), "UserName", "Created the file");
		
		writeAuditTrailData(xmlDoc, dfDateFormat.format(dtCreationDate).toString(), "UserName", "Created the file");
		
		//System.out.println("after write Audit Trail Data");
	}
	
	/**
	* Method writeAuditTrailData
	*Description: This method writes the audit trail data to DOM tree
	* @param xmlDoc DOM representation of theXML document
	* @param dtDate Date of Log data
	* @param strRespParty Responsible party
	* @param strComment Comment
	*
	*/
	public void writeAuditTrailData(org.w3c.dom.Document xmlDoc, String dtDate, String strRespParty, String strComment) {
	
		//Find the Event in AuditTrail element
		NodeList nodeList = xmlDoc.getElementsByTagName("Event");
		
	
		if (nodeList.getLength() > 0) {
			Element elem = (Element) nodeList.item(0); 
			//System.out.println("elem " + elem);
			/*Element elEventDate = (Element) elem.getFirstChild(); 
			System.out.println("elEventDate " + elEventDate);
			String strElemName = elEventDate.getTagName(); 
			
			System.out.println("The element in audit trail is " + strElemName);*/
			//write EventDate
			//findElementAndWriteData(xmlDoc, strElemName, dtDate, "");
			findElementAndWriteData(xmlDoc, "EventDate", dtDate, "");
			//write ResponsibleParty
			//findElementAndWriteData((Element)nodeList.item(0), "ResponsibleParty", strRespParty);
	
			// write Comment
			//findElementAndWriteData((Element)nodeList.item(0), "Comment", strComment);
		}
	}
			
	/**
	* Method findElementAndWriteData
	*Description: This method searches for an element in a DOM tree and writes the  data to DOM tree
	* @param xmlDoc DOM representation of the XML document
	* @param strElemName Element that is to be searched/created(if it does not exist)
	* @param objData The actual data to be written for that element
	* @param strPrevSibling Sibling where this new element is to be added in DOM object
	*
	*/
	public void findElementAndWriteData(org.w3c.dom.Document xmlDoc, String strElemName, Object objData, String strPrevSibling ) {
	
		Element elem=null;
		Node node=null;
		NodeList nodeList;
		try {
			//search for element
			nodeList = xmlDoc.getElementsByTagName(strElemName);	
		
			//System.out.println("findElementAndWriteData >>>> nodeList.getLength() for element "+strElemName+"is .."+nodeList.getLength());
			//if the element is present, write the data
			if (nodeList.getLength() > 0) {
			
				for(int intListIndex = 0; intListIndex < nodeList.getLength(); intListIndex++) {
				
					elem = (Element) nodeList.item(intListIndex);
					//remove the existing data, if any
					if (elem.getFirstChild() instanceof Text) {
						elem.removeChild(elem.getFirstChild());
					}
					//write the data
					Text newNodeValue = xmlDoc.createTextNode((String)objData);
					elem.appendChild(newNodeValue); 
				}
			} else {
				//Element not available. Create it
				elem = xmlDoc.createElement(strElemName);
				elem.setNodeValue((String)objData);
				//search for sibling and write the data
				Element siblingElem;
				nodeList = xmlDoc.getElementsByTagName(strPrevSibling);
				
				if (nodeList.getLength() > 0) {
					for(int intListIndex = 0; intListIndex < nodeList.getLength(); intListIndex++) {
						siblingElem = (Element)nodeList.item(intListIndex);
						
						Node parentNode;
						parentNode = siblingElem.getParentNode();
						if (parentNode != null) {
							parentNode.appendChild(elem);
						}
					}
				}		
			}
			
			newDoc = xmlDoc;
		} catch (Exception e) {
			System.out.println("Exception in findElementAndWriteData() "+e);
		}
	}
	
	/**
	* Method writeWellSectionData
	*Description: This method write the well section data to a  DOM tree
	* @param xmlDoc DOM representation of the document
	* @param objLasData Data from the LAS file
	* @param objMappingReader Mapping Class that has LAS-WITSML mappings
	*
	*/
	public void writeWellSectionData(org.w3c.dom.Document xmlDoc, LasData objLasData, MappingReader objMappingReader) {
		
		//Write the Well UID AS W1 by default
		findElementAndWriteAttributeData(xmlDoc, "well", "uid", "W1");
		
		findElementAndWriteData(xmlDoc, "name", objLasData.getWell(),"well"); 
		//get mappings for well section from mapping file
		Hashtable htMapping = objMappingReader.getMappings("wells");
		//get well data from the LAS file
		Hashtable htWellData = objLasData.getParameterData().getWellSectionData();
		//System.out.println(" elements for wells section htWellData is "+htWellData);
		
		//get well section data
		ArrayList alWellSectionData = objLasData.getWellSectionData();
		//System.out.println(" Well section LAS data size "+alWellSectionData.size());
		//write the well data to a DOM tree	
		if ((alWellSectionData != null) && (alWellSectionData.size() > 0)) {
			for (int intDataIndex = 0; intDataIndex < alWellSectionData.size(); intDataIndex++)	{		
				//get the LasSectionInfo object for getting the value
				LasSectionInfo objLasSectionInfo = (LasSectionInfo) alWellSectionData.get(intDataIndex);
				String strLas = objLasSectionInfo.getMnemonic(); 
						
				if (strLas.equalsIgnoreCase("LATI") || strLas.equalsIgnoreCase("LONG") || strLas.equalsIgnoreCase("X") || strLas.equalsIgnoreCase("Y") || strLas.equalsIgnoreCase("GDAT") || strLas.equalsIgnoreCase("HZCS")) {
					
					String[] strLasDeletion = new String[4];	
					
					if (htWellData.containsKey("LATI") && htWellData.containsKey("LONG"))	{ 
						strLasDeletion[0] = "X";
						strLasDeletion[1] = "Y";
						strLasDeletion[2] = "GDAT";
						strLasDeletion[3] = "HZCS";
					} else if (htWellData.containsKey("X") && htWellData.containsKey("Y"))	{ 
						strLasDeletion[0] = "LATI";
						strLasDeletion[1] = "LONG";
						strLasDeletion[2] = "GDAT";
						strLasDeletion[3] = "HZCS";
					} else if (htWellData.containsKey("GDAT") && htWellData.containsKey("HZCS"))	{ 
						strLasDeletion[0] = "LATI";
						strLasDeletion[1] = "LONG";
						strLasDeletion[2] = "X";
						strLasDeletion[3] = "Y";
					}
					//get the WITSML elements
					NodeList nlWellLocation  = xmlDoc.getElementsByTagName("wellLocation");
					Element elWellLocation= (Element)nlWellLocation.item(0);
					
					for (int intLas = 0; intLas < strLasDeletion.length; intLas++) {
						//System.out.println("Witsml element >>>>"+(String)htMapping.get(strLasDeletion[intLas]));
						NodeList nlLocalX = elWellLocation.getElementsByTagName((String)htMapping.get(strLasDeletion[intLas])); 
						if (nlLocalX.getLength() > 0) {
							elWellLocation.removeChild((Node)nlLocalX.item(0));
						}				
					}
					
					
				} else if (!(htWellData.containsKey("LATI")) && (!(htWellData.containsKey("LONG"))) &&
						!(htWellData.containsKey("X")) && (!(htWellData.containsKey("Y"))) && 
						!(htWellData.containsKey("GDAT")) && (!(htWellData.containsKey("HZCS")))
						)	{ 
					//System.out.println(" >>>> DOES NOT EXIST");
					NodeList nlWellLocation  = xmlDoc.getElementsByTagName("wellLocation");
					Element elWellLocation= (Element)nlWellLocation.item(0);
					String[] strLasDeletion = {"LATI","LONG","X","Y","GDAT","HZCS"};
					for (int intLas = 0; intLas < strLasDeletion.length; intLas++) {
						//System.out.println("Witsml element >>>>"+(String)htMapping.get(strLasDeletion[intLas]));
						NodeList nlLocalX = elWellLocation.getElementsByTagName((String)htMapping.get(strLasDeletion[intLas])); 
						if (nlLocalX.getLength() > 0) {
							elWellLocation.removeChild((Node)nlLocalX.item(0));
						}				
					}
				}		
				
				
				if (htMapping.containsKey(strLas))	{ 
					String strWitsml = (String) htMapping.get(strLas); 
					String strValue = objLasSectionInfo.getValue();
					
					//System.out.println("The strValue is " + strValue);
					if ( strValue.equals("")){
						strValue = "--";
					}
					//System.out.println("The strValue after assiging -- is  " + strValue);
					
							
					//System.out.println(" LAS ...."+ strLas + ".WITSML ...."+strWitsml+"<<<strValue "+strValue);
					//write the data 
					findElementAndWriteData(xmlDoc, strWitsml, strValue,"");
					
					if (strLas.equalsIgnoreCase("LATI") || strLas.equalsIgnoreCase("LONG") || strLas.equalsIgnoreCase("X") || strLas.equalsIgnoreCase("Y") || strLas.equalsIgnoreCase("GDAT") || strLas.equalsIgnoreCase("HZCS")) {
						//System.out.println("value is .."+strValue);
						//System.out.println("unit is .."+objLasSectionInfo.getUnit());
						findElementAndWriteAttributeData(xmlDoc, strWitsml, "uom", objLasSectionInfo.getUnit());
					}
				} 
			}
		} 
	}
	
	/**
	* Method findElementAndWriteAttributeData
	*Description: This method searches for an element in a DOM tree and writes the attrbute data for that element
	* @param xmlDoc DOM representation of the XML document
	* @param strElemName Element that is to be searched
	* @param strAttribName Attribute to be created/set
	* @param objData The actual data to be written for that attribute
	*
	*/
	public void findElementAndWriteAttributeData(org.w3c.dom.Document xmlDoc, String strElemName, String strAttribName, Object objData) {
		Element elem;
		NodeList nodeList;
		//search for the element
		nodeList = xmlDoc.getElementsByTagName(strElemName);
		
		if (nodeList.getLength() > 0) {
			elem = (Element) nodeList.item(0);
			//get the attributes for the element
			Attr objAttr  = elem.getAttributeNode(strAttribName);
			//attribute exists, set the value	
			if (objAttr != null) {
				objAttr.setValue((String)objData);
			} else {
				//attribute does not exists, create a new attribute
				elem.setAttribute(strAttribName,(String)objData);
			}
			
		}
		this.newDoc = xmlDoc;
	}
	
	/**
	* Method generateWITSML
	*Description: This method creates the WITSML output file depending on the section in the destination folder as per the DOM Tree
	* @param domSrc DOM representation of the XML document
	* @param strWriteSection The section of the LAS file for which WITMSL file to be generated
	* @param strDestFolder Destination folder for placing the generated files
	*
	*/
	public void generateWITSML(org.w3c.dom.Document domSrc, String strWriteSection, String strDestFolder, int intRunNumber) {
		try{
			// Get an instance of a TransformerFactory
			TransformerFactory transFactory =  TransformerFactory.newInstance();
			
			// Get a transformer using the factory
			Transformer transformer = transFactory.newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "5");
			
			// Define the source, which is a DOM document
			DOMSource objSource = new DOMSource(domSrc);
			
			//System.out.println("The destination folder is " + strDestFolder);
			//commented the below line for multiple well log files
			//String strSourceId = strDestFolder+"_"+strWriteSection+".xml";
			//set	the source 	
			String strSourceId = "";	
			if (strWriteSection.equals("well"))  {
				strSourceId = strDestFolder+"_"+strWriteSection+".xml";
			} else if (strWriteSection.equals("wellLog"))  {
				strSourceId = strDestFolder+"_"+strWriteSection+"_"+intRunNumber+".xml";
			}
			//System.out.println("The source Id is >>>>>>>>>> " + strSourceId);
			// Define the result, which is a StreamResult 
			// encapsulating the file
			//StreamResult objResult = new  StreamResult(new File(strSourceId));
			StreamResult objResult = new  StreamResult(strSourceId);
			//System.out.println("result ..."+objResult);
				
			// Use the identity transformation which just echoes 
			// the input tree
			transformer.transform(objSource,objResult);
			
			//System.out.println(" >>>>>>> vecGeneratedFiles"+vecGeneratedFiles);
			//add the generated output files to a vector
			vecGeneratedFiles.add(strSourceId);
			
			//System.out.println("The size of the generated files vector is " + vecGeneratedFiles.size());
		}catch (javax.xml.transform.TransformerConfigurationException tce) {
			System.out.println("generateWITSML() :::::::::TransformerConfigurationException .." +tce.getMessage());
		}catch (javax.xml.transform.TransformerException te) {
			System.out.println("generateWITSML() :::::::::TransformerException .."+te.getMessage());
		}catch (Exception e) {
			System.out.println("generateWITSML() :::::::::Exception .."+e);
		}
	}
	
	/**
	* Method generateWITSML
	*Description: This method creates the WITSML output file depending on the section in the destination folder as per the DOM Tree
	* @param domSrc DOM representation of the XML document
	* @param strWriteSection The section of the LAS file for which WITMSL file to be generated
	* @param strDestFolder Destination folder for placing the generated files
	*
	*/
	public org.w3c.dom.Document  createDataElements(String strSectionName,Document logDoc,Node ndLogHeader) {
		try{
			//To create LogData Tag , Cummon Tag and custom Tag added below code on 10/May/2006
			//Obtain a new instance of a DocumentBuilderFactory
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			//Creates a new instance of a DocumentBuilder using the currently configured parameters.
			DocumentBuilder builder = factory.newDocumentBuilder();
			//Parse the content of the given URI as an XML document and return a new DOM Document object
			org.w3c.dom.Document mappingDocument = builder.parse(getMappingFile());
			// Get attibutes list
			Element elRoot = logDoc.getDocumentElement();
			//search for mandatoryelements in mapping
			String strLogDataPath = "//las2witsml/mappings/section[@name=\"wellLogs\"]/"+strSectionName;
			//get mapping elements
			NodeList nlLogDataElements = XPathAPI.selectNodeList(mappingDocument, strLogDataPath, elRoot);
			//System.out.println("The length is " + nlLogDataElements.getLength());
			//create mandatory elements
			if (nlLogDataElements.getLength() > 0) {
				// Process the elements in the mandatory elements list
				for (int intMandatory = 0; intMandatory < nlLogDataElements.getLength(); intMandatory++) {
				
					Element elDataNode = (Element) nlLogDataElements.item(intMandatory);
					
					//get the WITSML elements
					NodeList nlWitsmlName  = elDataNode.getElementsByTagName("WITSML");
					
					for (int m=0; m<nlWitsmlName.getLength(); m++) {
						Element elWitsml = (Element)nlWitsmlName.item(m);
						//get the WITSML element name 					
						String strWitsmlName = elWitsml.getAttributeNode("elemname").getValue();
						//get the WITSML element parent name 	
						String strWitsmlParent = elWitsml.getAttributeNode("elemparent").getValue();
						
						//if (strWitsmlName.equals("logData")) {
							//create a new element 
							Node ndWitsml = logDoc.createElement(strWitsmlName);
							//get the elements under the parent node
							NodeList nlWitsmlParent  = logDoc.getElementsByTagName(strWitsmlParent);
							
							//create nodes
							if (nlWitsmlParent.getLength() > 0) {
								Node ndParentNode = (Element)nlWitsmlParent.item(0);
								//create a text node and append to the parent node
								Text text = logDoc.createTextNode("");
								ndWitsml.appendChild(text);
								ndParentNode.appendChild(ndWitsml);
							} else {
								//append the node to document root
								ndLogHeader.appendChild(ndWitsml);
							}
						//}
					}
				}
			}
		}
		catch (Exception e)
		{
			System.out.println("The exception in creating log data" + e);
		}	
		return logDoc;	
	}
	/**
	* Method getOutputFiles
	* This method returns the output files vector
	* @return Generated WITSML files 
	*
	*/
	public Vector getOutputFiles() {
		return vecGeneratedFiles;
	}
	
	/**
	* Method setOutputFiles
	* This method will set the output files vector
	* @param vecOutputFiles input WITSML File
	*
	*/
	public void setOutputFiles(Vector vecOutputFiles) {
		this.vecGeneratedFiles=vecOutputFiles;
	}
	
	/**
	* Method setMappingFile
	* This method will set the mapping file to be used for conversion
	* @param strMappingFile Mapping file Name
	*
	*/
	public void setMappingFile(String strMappingFile) {
		this.strMappingFile=strMappingFile;
	}
	
	/**
	* Method getMappingFile
	* This method returns the mapping file to be used for conversion
	* @return  Mapping file
	*
	*/
	public String getMappingFile() {
		return strMappingFile;
	}
	
	/**
	* Method setSelectionFile
	* This method sets the selection file
	* @param  strSelectionFile Selection file
	*
	*/
	public void setSelectionFile(String strSelectionFile) {
		this.strSelectionFile = strSelectionFile;
	}
	
	/**
	* Method getSelectionFile
	* This method returns the selection file name
	* @return Selection
	*
	 */
	public String getSelectionFile() {
		return this.strSelectionFile;
	}
	/**
	* Method setWellParamSize
	*This method sets the well information size
	* @param intParamSize Number of WELL parameters in LAS file
	*
	*/
	public void setWellParamSize(int intParamSize) {
		this.intWellParamSize = intParamSize;
	}
	
	/**
	* Method getWellParamSize
	* This method returns the number of well info parameters
	* @return Number of WELL parameters in LAS file
	*
	 */
	public int getWellParamSize() {
		return this.intWellParamSize;
	}	
	
	/**
	* Method setLogger
	*This method sets the logger information
	* @param pwPrintWriter PrintWriter
	*
	*/
	public void setLogger(PrintWriter pwPrintWriter) {
		this.pwPrintWriter = pwPrintWriter;
	}
	
	/**
	* Method getLogger
	* This method returns Logger information
	* @return PrintWriter
	*
	 */
	public PrintWriter getLogger() {
		return this.pwPrintWriter;
	}	
	
/**
	* Method setRunNumber
	*This method sets the RunNumber
	* @param intRunNumber Number of WELL parameters in LAS file
	*
	*/
	public void setRunNumber(int intRunNumber) {
		this.intRunNumber = intRunNumber;
	}
	
	/**
	* Method getRunNumber
	* This method returns the RunNumber
	* @return RunNumber in LAS file
	*
	 */
	public int getRunNumber() {
		return this.intRunNumber;
	}					
}
