package com.las2witsml;

//import classes

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.Container;

import java.awt.event.ActionListener;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;

import java.awt.Cursor;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;

//import swing classes
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JRadioButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JProgressBar;
import javax.swing.JPanel;
import javax.swing.JEditorPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.BorderFactory;
import javax.swing.ListSelectionModel;
import javax.swing .ImageIcon;

//table related classes
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import javax.swing.table.JTableHeader;

//event related classes
import javax.swing.event .TableModelListener;
import javax.swing.event .TableModelEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.Text;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException; 

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.OutputKeys;

/**
* Title: LasWitsml.java
* Description: This is the main class for converting LAS to WITSML file.
* Copyright: Copyright (c) 2005
* Company: Satyam Computer Services Ltd.
* @author Ravi Domakonda, Lalitha Yengaldas
* @version 1.0
*/
public class LasWitsml extends JFrame implements ActionListener, TableModelListener, KeyListener {
	//declare variables 
	//To store the row selected in Current Files grid
	public int ROW_SELECTED = -1;
	// To store Input LAS File folder
	public String INPUT_FILE_DIRECTORY;
	// To store Destination folder
	public String DEST_FILE_DIRECTORY;
	// To store the selected LAS file
	public String LAS_FILE_SELECTED;
	// To store the previous selected LAS file
	public String PREV_SELECTED_LAS_FILE;
	//To hold the mandatory section that is selected -- Parameter or Curve
	public String MAND_DATA_SELECTED;
	
	// Used to identify the windows platform.
	private static final String WIN_ID = "Windows";
	// The default system browser under windows.
	private static final String WIN_PATH = "rundll32";
	// The flag to display a url.
	private static final String WIN_FLAG = "url.dll,FileProtocolHandler";
	// The default browser under unix.
	private static final String UNIX_PATH = "netscape";
	// The flag to display a url.
	private static final String UNIX_FLAG = "-remote openURL";
	
	//declare panels for placing controls
	JPanel pnNavFileSelection = new JPanel();
	JPanel pnNavMandData = new JPanel();
	JPanel pnMainPanel = new JPanel();
	JPanel pnButtonPanel = new JPanel();
	JPanel pnConvertStatus=null;

	JPanel pnFileSelection = new JPanel();
	JPanel pnFileSelectHdr = new JPanel();
	JPanel pnInputFileSelectDtls = new JPanel();
	JPanel pnDestFileSelectDtls = new JPanel();
	JPanel pnFileSelectDtls = new JPanel();
	JPanel pnCurrentFile = new JPanel();
	JPanel pnCurrentFileHdr = new JPanel();
	JPanel pnCustomizeMapping=new JPanel();
	JPanel pnComboPanel =new JPanel();

	JPanel pnMandData = new JPanel();
	JPanel pnCurveSelection = new JPanel();
	JPanel pnOptionalSelection = new JPanel();
	JPanel pnOptionalHeader = null;
	
	//progressbar to display the status of conversion
	JProgressBar progressBar = new JProgressBar(0);
	
	//labels for displaying on the user interface
	JLabel lblFileSelHeader = new JLabel(" LAS to WITSML File Selection ",SwingConstants.LEFT);
	JLabel lblConversionStatus = new JLabel(" LAS to WITSML Conversion Status ",SwingConstants.LEFT);
	JLabel lblCurrentFile = new JLabel(" Current Files ",SwingConstants.LEFT);
	JLabel lblParameterSelection;
	JLabel lblCurveSelectAll = new JLabel(" Select All ",SwingConstants.LEFT);
	JLabel lblOptionalHdr = null;
	JLabel lblEmpty = new JLabel(" ");
	JLabel lblEmptyValue = new JLabel("                 ");
	JLabel lblInputFile = new JLabel("Input LAS File Folder:    " ,SwingConstants.LEFT);
	JLabel lblDestFile = new JLabel("  Destination WITSML Files Folder: ",SwingConstants.LEFT);
	JLabel lblFileProcessed = new JLabel("File being processed: " ,SwingConstants.LEFT);
	JLabel lblFileStatus = new JLabel("   File completion status:" ,SwingConstants.LEFT);
	JLabel lblMappingFile = new JLabel("Mapping Files");
	JLabel lblMappingEmpty = new JLabel(" ");
	
	//button for selection an operation on the interface
	JButton btnFileSelection = new JButton();
	//JButton btnMandData = new JButton("Mandatory Data Selection");
	JButton btnMandData = new JButton();
	JButton btnConvert = new JButton();
	JButton btnConvertSummary = new JButton();
	JButton btnInputFile = new JButton();
	JButton btnDestFile = new JButton();
	JButton btnHelp = new JButton();
	JButton btnParameterHelp = new JButton();
	JButton btnCurveHelp = new JButton();
	JButton btnProcessHelp = new JButton();
	JButton btnSummaryHelp = new JButton();
	JButton btnConvertHelp = new JButton();
	JButton btnParamPrev = new JButton();
	JButton btnCurvePrev = new JButton();
	JButton btnOptPrev = new JButton("<< Curve Section");
	JButton btnNext = new JButton();
	JButton btnParamNext = new JButton();
	JButton btnCurveNext = new JButton();
	JButton btnProcess = new JButton("Process");
	JButton btnExit = new JButton();
	JButton btnLoadSelection = new JButton();
	JButton btnSaveSelection = new JButton();
	JButton btnCustomizeMapping = new JButton();
	JButton btnProcessConversion = new JButton();
	JButton btnBackFileSelection = new JButton();
	JButton btnConverCustomizeMapping = new JButton();
	JButton btnConversionBack = new JButton();
		
	//declare text fields for allowing user input
	JTextField txtInputFile = new JTextField(30);
	JTextField txtDestFile = new JTextField(30);
	JTextField txtFileProcessed = new JTextField(20);
	JTextField txtFileStatus = new JTextField(20);	
	
	//check box for selecting all curves
	JCheckBox chCurveSelection = new JCheckBox("Select All");
	JCheckBox chParameterSelection = new JCheckBox("Select All");
	
	//combo box for displaying output and mapping files
	JComboBox cbnOutputFile = new JComboBox();
	JComboBox cbnMappingFiles = new JComboBox();
	
	//list for displaying mandatory data (Parameter, Curve) on the left hand menu
	JList jlMandData = null;
	ListSelectionModel jListSelectionModel;
	
	//scroll pane for adding components
	JScrollPane scrollPane;
	
	//tables for displaying data
	JTable jtCurrentTable;
	JTable jtMandatory;
	JTable jtSummary;
	
	//tables models for displaying data	
	DefaultTableModel dmCurrentData;
	DefaultTableModel dmMandatory;
	DefaultTableModel dmSummary;
	
	//objects for storing data
	LasData objLasData = null;
	LasDataFile objLasDataFile = null;
	CustomizeMapping jdCustomizeMapping = null;
	ConversionCustomizeMapping jdConvCustomizeMapping = null;
	
	//hashtable for saving changed mappings
	Hashtable htChangedMappings = null;
	
	//strings for displaying table headers
	String[] strCurrentFileHeader = {"S.No","Input File","Select"};
	String[] strParameterHeader = {" ","LAS Name","Description"};
	//String[] strCurveHeader= {" ","LAS Name","Description","API Code","Unit"}; commented for interchaning unit and api code
	String[] strCurveHeader = {" ","LAS Name","Description","Unit","API Code"};
	String[] strOptionalHeader = {"S.No","LAS Name","Description","Value"};
	//Commented to add selection name in summary screen
	//String[] strSummaryHeader = {"S.No","Input File","Generated File","Status","Mapping Template"};
	String[] strSummaryHeader = {"S.No","Input File","Generated File","Status","Mapping Template","Selection Template"};
	//strings to be displayed on the left menu
	String strMandParameter = "->Parameters";
	String strMandCurveData = "->Curve Data";
	String strSelectionFileName = null;
	String strSelectionName = null;
	
	
	//to display mandatory data menu
	String strMandData[] = {strMandParameter,strMandCurveData};
	//main container for placing components on the interface
	Container c ;
	//object for displaying progress bar status
	SwingWorker objSwingWorker;
	//vector for storing output files 
	Vector vecFiles;
	//vector for storing LAS files
	Vector vecFileDetails = null;
	
	//Font and color to be used on interface
	Font fnHdrFont;
	Font fnMenuFont;
	Color clrHeaderBg;
	Color clActiveButton = new Color(255,255,201);
	//ArrayList to hold parameter data
	ArrayList alParameterData = null;
	//ArrayList to hold curve data
	ArrayList alCurveData = null;
	
	//File writer and printWriter
	FileWriter fwFileWriter = null;
	PrintWriter pwPrintWriter = null;
	Hashtable htWellData = null;
	
	
	boolean bnErrorFlag = false;
	/**
	* Constructor LasWitsml 
	*/
	public LasWitsml() {
		try{
			
			//setting background for panels
			pnMainPanel.setBackground(Color.white);
			pnButtonPanel.setBackground(Color.white);
			pnNavFileSelection.setBackground(Color.white);
			pnNavMandData.setBackground(Color.white);
			//pnConvertStatus.setBackground(Color.white);
			pnFileSelection.setBackground(Color.white);
			pnFileSelectHdr.setBackground(Color.white);
			pnFileSelectDtls.setBackground(Color.white);
			pnDestFileSelectDtls.setBackground(Color.white);
			pnInputFileSelectDtls.setBackground(Color.white);
			pnCurrentFile.setBackground(Color.white);
			pnCurrentFileHdr.setBackground(Color.white);
			pnCustomizeMapping.setBackground(Color.white);
			pnComboPanel.setBackground(Color.white);
			pnMandData.setBackground(Color.white);
			pnCurveSelection.setBackground(Color.white);
			pnOptionalSelection.setBackground(Color.white);
			//pnOptionalHeader.setBackground(Color.white);
			
			
		
			//to display the file selection grid
			fnDisplayFileSection();
			//set background color for File Selection on left hand menu
			//btnFileSelection.setBackground(clActiveButton);
			//btnFileSelection.setBackground(new Color(255,245,219));
			btnFileSelection.setBackground(new Color(239,247,251));
			
			
			chParameterSelection.setBackground(Color.white);
			chCurveSelection.setBackground(Color.white);
			
			//registers listeners for buttons
			btnProcess.addActionListener(this);
			btnCurveNext.addActionListener(this);
			btnCurvePrev.addActionListener(this);
			btnLoadSelection.addActionListener(this);
			btnSaveSelection.addActionListener(this);
			btnCustomizeMapping.addActionListener(this);
			btnParamNext.addActionListener(this);
			btnParamPrev.addActionListener(this);
			btnConvert.addActionListener(this);
			btnConvertSummary.addActionListener(this);
			btnBackFileSelection.addActionListener(this);
			btnProcessConversion.addActionListener(this);
			btnProcessConversion.addKeyListener(this);
			btnConverCustomizeMapping.addActionListener(this);
			btnConversionBack.addActionListener(this);
			btnInputFile.addActionListener(this);
			btnDestFile.addActionListener(this);
			btnFileSelection.addActionListener(this);
			btnNext.addActionListener(this);
			btnHelp.addActionListener(this);
			btnParameterHelp.addActionListener(this);
			btnCurveHelp.addActionListener(this);
			btnProcessHelp.addActionListener(this);
			btnSummaryHelp.addActionListener(this);
			btnConvertHelp.addActionListener(this);
			//add listeners for input and destination files for allowing enter key selection
			txtInputFile.addActionListener(this);
			txtDestFile.addActionListener(this);
			
			chCurveSelection.addActionListener(this);
			chParameterSelection.addActionListener(this);
			
			cbnOutputFile.addActionListener(this);
			
			btnMandData.addActionListener(this);
			//enable file selection link and disable other links of the left hand menu
			btnFileSelection.setEnabled(true);
			btnMandData.setEnabled(false);
			btnConvert.setEnabled(false);
			btnConvertSummary.setEnabled(false);
			
						
			//instantiate LasDataFile object for storing all the LAS data
			objLasDataFile = new LasDataFile();
			
			//set string painted for progress bar
			//progressBar.setStringPainted(true);
			
			chParameterSelection.setSelected(true);
			//set shortcut keys for buttons	
			btnConverCustomizeMapping.setMnemonic('C');
			btnConversionBack.setMnemonic('B');
			alParameterData = new ArrayList();
			alCurveData = new ArrayList();
			
			//opening the FileWriter with currentpath
			File fCurrentPath = new File(".");
			String strPath  = new File(fCurrentPath.getCanonicalPath()).getParent();
			
			//adding button images
			String strImagePath=strPath+"/images/";
			
			btnFileSelection.setIcon(new ImageIcon(strImagePath+"fileselector_active.gif"));
			btnFileSelection.setDisabledIcon(new ImageIcon(strImagePath+"fileselector_active.gif"));
			
			btnMandData.setIcon(new ImageIcon(strImagePath+"mandatory_active.gif"));
			btnMandData.setDisabledIcon(new ImageIcon(strImagePath+"mandatory_active.gif"));
			
			btnConvert.setIcon(new ImageIcon(strImagePath+"convert_to_Las_active.gif"));
			btnConvert.setDisabledIcon(new ImageIcon(strImagePath+"convert_to_Las_active.gif"));
			
			btnConvertSummary.setIcon(new ImageIcon(strImagePath+"summary_active.gif"));
			btnConvertSummary.setDisabledIcon(new ImageIcon(strImagePath+"summary_active.gif"));
						
			btnInputFile.setIcon(new ImageIcon(strImagePath+"input_folder.gif"));
			btnInputFile.setToolTipText("Input LAS File Folder ");
			btnInputFile.setBackground(Color.white);
			btnInputFile.setBorder(null);
			
			btnDestFile.setIcon(new ImageIcon(strImagePath+"destination_folder.gif"));
			btnDestFile.setToolTipText("Destination WITSML Folder ");
			btnDestFile.setBackground(Color.white);
			btnDestFile.setBorder(null);
			
			btnCustomizeMapping.setIcon(new ImageIcon(strImagePath+"customize_mapping.gif"));
			btnCustomizeMapping.setToolTipText("Customize Mapping ");
			btnCustomizeMapping.setBorder(null);
			
			btnConverCustomizeMapping.setIcon(new ImageIcon(strImagePath+"customize_mapping.gif"));
			btnConverCustomizeMapping.setToolTipText("Customize Mapping ");
			btnConverCustomizeMapping.setBorder(null);
			
			btnHelp.setIcon(new ImageIcon(strImagePath+"help.gif"));
			btnHelp.setToolTipText(" Help ");
			btnHelp.setBorder(null);
			
			btnExit.setIcon(new ImageIcon(strImagePath+"exit.gif"));
			btnExit.setToolTipText(" Exit ");
			btnExit.setBorder(null);
			
			btnNext.setIcon(new ImageIcon(strImagePath+"next.gif"));
			btnNext.setToolTipText(">> Parameter Section ");
			btnNext.setBorder(null);
			
			btnParameterHelp.setIcon(new ImageIcon(strImagePath+"help.gif"));
			btnParameterHelp.setToolTipText(" Help ");
			btnParameterHelp.setBorder(null);
			
			btnParamPrev.setIcon(new ImageIcon(strImagePath+"file_selection.gif"));
			btnParamPrev.setToolTipText("<< File Selection ");
			btnParamPrev.setBorder(null);
			
			btnParamNext.setIcon(new ImageIcon(strImagePath+"next.gif"));
			btnParamNext.setToolTipText(">> Curve Section ");
			btnParamNext.setBorder(null);
			
			btnCurveHelp.setIcon(new ImageIcon(strImagePath+"help.gif"));
			btnCurveHelp.setBorder(null);
			
			btnLoadSelection.setIcon(new ImageIcon(strImagePath+"load_selection.gif"));
			btnLoadSelection.setToolTipText(" Load Selection ");
			btnLoadSelection.setBorder(null);
			
			btnSaveSelection.setIcon(new ImageIcon(strImagePath+"save_selection.gif"));
			btnSaveSelection.setToolTipText(" Save Selection ");
			btnSaveSelection.setBorder(null);
			
			btnCurvePrev.setIcon(new ImageIcon(strImagePath+"back.gif"));
			btnCurvePrev.setToolTipText(" << Parameter Section");
			btnCurvePrev.setBorder(null);
			
			btnCurveNext.setIcon(new ImageIcon(strImagePath+"next.gif"));
			btnCurveNext.setToolTipText(" << Convert LAS-WITSML");
			btnCurveNext.setBorder(null);
			
			btnConvertHelp.setIcon(new ImageIcon(strImagePath+"help.gif"));
			btnConvertHelp.setToolTipText(" Help ");
			btnConvertHelp.setBorder(null);
			
			btnProcessHelp.setIcon(new ImageIcon(strImagePath+"help.gif"));
			btnProcessHelp.setToolTipText(" Help ");
			btnProcessHelp.setBorder(null);
			
			btnSummaryHelp.setIcon(new ImageIcon(strImagePath+"help.gif"));
			btnSummaryHelp.setToolTipText(" Help ");
			btnSummaryHelp.setBorder(null);
			
			btnConversionBack.setIcon(new ImageIcon(strImagePath+"back.gif"));
			btnConversionBack.setToolTipText(" << Curve Section");
			btnConversionBack.setBorder(null);
			
			btnBackFileSelection.setIcon(new ImageIcon(strImagePath+"file_selection.gif"));
			btnBackFileSelection.setToolTipText("<< File Selection ");
			btnBackFileSelection.setBorder(null);
			
			btnProcessConversion.setIcon(new ImageIcon(strImagePath+"process.gif"));
			btnProcessConversion.setToolTipText(" Click here to process ");
			btnProcessConversion.setBorder(null);
			
			//creating looger file
			fwFileWriter = new FileWriter(strPath+"/log/LASWITSML.log",false);
			pwPrintWriter = new PrintWriter(fwFileWriter,true);
			pwPrintWriter.println("In Constructor");
			objLasDataFile.setLogger(pwPrintWriter);
			
			btnFileSelection.setFont(fnMenuFont);
			btnMandData.setFont(fnMenuFont);
			btnConvert.setFont(fnMenuFont);
			btnConvertSummary.setFont(fnMenuFont);
		}catch (ConverterException eConverterException) {
				System.out.println(" Exception  " + eConverterException);
		}catch (Exception eException) {
				System.out.println(" Exception in opening File  " +eException);
		} 	 	
	} //end constructor
	
	/**
	 * Method actionPerformed
	 * This is the main method controlling the flow of the conversion process
	 * @param ActionEvent
	 *
	 */
	public void actionPerformed(ActionEvent ae)  {
		//if Input File button  is clicked
		if (ae.getSource() == btnInputFile)
		{
			try {
				//instantiate JFileChooser for displaying the File Chooser dialog box
				JFileChooser fcFilechooser = new JFileChooser();
				// Create a File object containing the canonical path of the
				// desired directory
				File fCurrentFolder = new File(new File(new File(".").getCanonicalPath()).getParent());
	    	// Set the current directory
				fcFilechooser.setCurrentDirectory(fCurrentFolder);
				//set the directory selection mode
				fcFilechooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				//show the dialog box for choosing directory
				int intReturnVal = fcFilechooser.showOpenDialog(null);
							
				File fSelectedFolder = null;
				
				if (intReturnVal == JFileChooser.CANCEL_OPTION){
						//System.out.println("User clicked on Cancel");
				}else{
				//get the selected directory
					fSelectedFolder = fcFilechooser.getSelectedFile() ;
				}
				//System.out.println("fSelectedFolder   " + fSelectedFolder);
				//check if directory is selected
				if(fSelectedFolder != null){
					//get the selected directory path
					String strInputFolder = fSelectedFolder.getAbsolutePath();
					txtInputFile.setText(strInputFolder);
					INPUT_FILE_DIRECTORY = strInputFolder;
					//System.out.println("INPUT_FILE_DIRECTORY is .."+INPUT_FILE_DIRECTORY);
					pwPrintWriter.println("INPUT_FILE_DIRECTORY is .."+INPUT_FILE_DIRECTORY);
					
					//set the input LAS directory and destination directory
					txtInputFile.setText(INPUT_FILE_DIRECTORY);
					txtDestFile.setText(INPUT_FILE_DIRECTORY);
					DEST_FILE_DIRECTORY = INPUT_FILE_DIRECTORY;
					//System.out.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
					pwPrintWriter.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
					//display the file selection grid
					fileSelectionTableDetails("Button");			
				}else{
					//JOptionPane.showMessageDialog(null, " Please select input LAS file folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
					//System.out.println("cancel is clicked");
				}
			} catch(Exception eException){
					bnErrorFlag = true;
					System.out.println("The exception occured on Input File button clicking " + eException);
					pwPrintWriter.println("The exception occured on Input File button clicking " + eException);
				}
		}else if(ae.getSource() == txtInputFile){
			//if enter key is pressed in Input LAS file folder text field
			try {
			//get the selected directory path
				String strInputFolder = txtInputFile.getText();
				if(strInputFolder != null){
					INPUT_FILE_DIRECTORY = strInputFolder;
					System.out.println("INPUT_FILE_DIRECTORY is .."+INPUT_FILE_DIRECTORY);
					pwPrintWriter.println("INPUT_FILE_DIRECTORY is .."+INPUT_FILE_DIRECTORY);
					//set the input and destination directory(defaulted with input directory) 			
					txtInputFile.setText(INPUT_FILE_DIRECTORY);
					txtDestFile.setText(INPUT_FILE_DIRECTORY);
					DEST_FILE_DIRECTORY = INPUT_FILE_DIRECTORY;
					//System.out.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
					pwPrintWriter.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
					fileSelectionTableDetails("TextField");			
				}/*else{
					JOptionPane.showMessageDialog(null, " Please select input LAS file folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
				}*/
			} catch(Exception eException){
					bnErrorFlag = true;
					//System.out.println("The exception occured on clicking input file text field " + eException);
					pwPrintWriter.println("The exception occured on clicking input file text field " + eException);
		    }
		}
		else if(ae.getSource() == btnConvert){
		//the below code will be executed when Convert LAS to WITSML button is clicked on left hand menu
			try{
				//System.out.println("MAND_DATA_SELECTED before conversion"+MAND_DATA_SELECTED);
				pwPrintWriter.println("MAND_DATA_SELECTED before conversion"+MAND_DATA_SELECTED);
				
				//set string painted for progress bar
				progressBar.setStringPainted(false);
				
				//check if any curves are selected
				boolean blnIsRowSelected = false;
				int intRowsSelected = 0;
				if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
					ArrayList alSectionData = objLasDataFile.getLasData().getLogDefns();
					if (alSectionData != null) {
						//System.out.println(">>>> alSectionData size ..."+alSectionData.size());
						pwPrintWriter.println(">>>> alSectionData size ..."+alSectionData.size());
						for (int intSectionIndex = 0; intSectionIndex < alSectionData.size(); intSectionIndex++) {
							LasSectionInfo objLasSectionInfo = (LasSectionInfo) alSectionData.get(intSectionIndex);
							boolean blnSelected = (objLasSectionInfo.getIsSelected()).booleanValue();
							if (blnSelected) {
								//blnIsRowSelected = true;
								intRowsSelected++;
							}
						}
					}
					//System.out.println("CURVE SELECTED "+intRowsSelected);
					pwPrintWriter.println("CURVE SELECTED "+intRowsSelected);
					//if no curves are selected, inform the user
					//if (blnIsRowSelected == false) {
					if (intRowsSelected < 1) {
						JOptionPane.showMessageDialog(null, "No curves are selected, Hence all curves will be selected by default", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
						MAND_DATA_SELECTED = strMandCurveData;
						ParameterDetails();
					}
					else if (intRowsSelected < 2) {
						JOptionPane.showMessageDialog(null, "Atleast two Curves are to be selected, Please select another curve", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
						MAND_DATA_SELECTED = strMandCurveData;
						ParameterDetails();
					}
				}
				else if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
					if (jtMandatory != null) {
						for(int intSelectionIndex = 0; intSelectionIndex < jtMandatory.getRowCount(); intSelectionIndex++) {
							Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSelectionIndex,0);
							//curve is selected, set the Row Selected flag to true			
							if (blnSelected.booleanValue() == true) {
								//blnIsRowSelected = true;
								intRowsSelected++;
							}
						}
						//System.out.println("IS ROW SELECTED for curve data..."+intRowsSelected);
						pwPrintWriter.println("IS ROW SELECTED for curve data..."+intRowsSelected);
					}
					//if no curves are selected, inform the user
					//if (blnIsRowSelected == false) {
					if (intRowsSelected < 2) {
						JOptionPane.showMessageDialog(null, "Atleast two Curves are to be selected, Please select another curve", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
					}
				} 
				//if atleast two curves are selected, save the selections
				if (intRowsSelected >= 2) {
					//System.out.println(" Number of curves selected >= 2");
					pwPrintWriter.println(" Number of curves selected >= 2");
					//set the progress bar status to zero
					progressBar.setValue(0);
					//ensure that this component has a valid layout.
					validate();
					//display the conversion screen
					fnConvertFileSection();
					//Disable buttons except convert LAS to WITSML button, set the back ground colors for other buttons on left hand menu
					/*btnFileSelection.setBackground(new Color(255,245,219));
					btnMandData.setBackground(new Color(255,245,219));
					//jlMandData.setBackground(new Color(255,245,219));
					jlMandData.setBackground(new Color(239,247,251));
					btnConvertSummary.setBackground(new Color(255,245,219));
					btnConvert.setBackground(new Color(255,245,219));*/
					
					btnFileSelection.setBackground(new Color(239,247,251));
					btnMandData.setBackground(new Color(239,247,251));
					//jlMandData.setBackground(new Color(255,245,219));
					jlMandData.setBackground(new Color(239,247,251));
					btnConvertSummary.setBackground(new Color(239,247,251));
					btnConvert.setBackground(new Color(239,247,251));
					btnProcessConversion.requestFocusInWindow();
					validate();
					
					/*btnFileSelection.setBackground(new Color(183,255,255));
					btnMandData.setBackground(new Color(183,255,255));
					jlMandData.setBackground(new Color(183,255,255));
					btnConvertSummary.setBackground(new Color(183,255,255));
					btnConvert.setBackground(clActiveButton);*/
											
				}
			}catch(ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("The exception occured on clicking convert LAS to WITSML on the left hand menu " + eConverterException);
				pwPrintWriter.println("The exception occured on clicking convert LAS to WITSML on the left hand menu " + eConverterException);
			}			
		} else if(ae.getSource() == btnConvertSummary){
			try{
			//ensure that this component has a valid layout.
				validate();
				//display the conversion summary screen
				fnSummarySection();
				//Disable buttons except Conversion Summary button, set the back ground colors for other buttons on left hand menu
								
				btnFileSelection.setBackground(new Color(239,247,251));
				btnMandData.setBackground(new Color(239,247,251));
				//jlMandData.setBackground(new Color(255,245,219));
				jlMandData.setBackground(new Color(239,247,251));
				btnConvertSummary.setBackground(new Color(239,247,251));
				btnConvert.setBackground(new Color(239,247,251));
							
			}catch (ConverterException eConverterException) {
					bnErrorFlag = true;
					System.out.println("The exception occured on clicking conversion  summary button on the left hand menu " + eConverterException);
					pwPrintWriter.println("The exception occured on clicking conversion  summary button on the left hand menu " + eConverterException);
			}	
		} else if(ae.getSource() == btnDestFile) 	{
		
			DEST_FILE_DIRECTORY = null;
			//if Destination Folder button  is clicked
			try{
				String strInputFolder = txtInputFile.getText();
				if(strInputFolder != null){
					if (strInputFolder.trim().equals("")) {
						//System.out.println("strInputFolder is empty");
						showAndCheckFolder();
					} else {
						INPUT_FILE_DIRECTORY = strInputFolder;
						//System.out.println("INPUT_FILE_DIRECTORY is .."+INPUT_FILE_DIRECTORY);
						pwPrintWriter.println("INPUT_FILE_DIRECTORY is .."+INPUT_FILE_DIRECTORY);
						//set the input and destination directory(defaulted with input directory) 			
						txtInputFile.setText(INPUT_FILE_DIRECTORY);
						txtDestFile.setText(INPUT_FILE_DIRECTORY);
						//System.out.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
						if (DEST_FILE_DIRECTORY == null) {
							showAndCheckFolder();
						}
						//DEST_FILE_DIRECTORY = INPUT_FILE_DIRECTORY;
						//System.out.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
						pwPrintWriter.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
						fileSelectionTableDetails("TextField");		
					}
				} else if ((strInputFolder != null) && (((DEST_FILE_DIRECTORY == null) || (DEST_FILE_DIRECTORY.trim().equals("")))) || ((txtDestFile.getText() != null) || (txtDestFile.getText().trim().equals("") == false))){
					showAndCheckFolder();
				} else if ((DEST_FILE_DIRECTORY != null) && (DEST_FILE_DIRECTORY.trim().equals("") == false)){
					showAndCheckFolder();
					
				} else if(strInputFolder == null){
					//if the destination folder is not set upon selecting input LAS file folder	
					JOptionPane.showMessageDialog(null, "Please select Input LAS File Folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
					//setting focus to window
					txtInputFile.requestFocusInWindow(); 
					btnInputFile.doClick();
				} else{
					//if the destination folder is not set upon selecting input LAS file folder	
					JOptionPane.showMessageDialog(null, "Please select destination File Folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
					//setting focus to window
					txtDestFile.requestFocusInWindow(); 
				}				
			}catch(Exception eException){
				bnErrorFlag = true;
				//System.out.println("The exception occured on clicking destination folder button " + eException);
				pwPrintWriter.println("The exception occured on clicking destination folder button " + eException);
			}
		} else if(ae.getSource() == txtDestFile) {
			//if enter key is pressed after entering/selecting Destination Folder 
			try{
				//if the destination folder is already set
				if ((txtDestFile.getText() != null) && (txtDestFile.getText().trim().equals("") == false)){
					//create a file with the entered/selected destination folder
					File fSelectedFolder = new File(txtDestFile.getText()) ;
					//if the destination folder is already selected
					if(txtDestFile.getText() != null){
						//check if destination folder is a directory
						if (fSelectedFolder.isDirectory()) {
							//get the selected directory path
							String strDestFolder = fSelectedFolder.getCanonicalPath() ;
							//set the destination folder		
							txtDestFile.setText(strDestFolder);
							DEST_FILE_DIRECTORY = strDestFolder;
							INPUT_FILE_DIRECTORY = txtInputFile.getText();
							
							//check if input folder is selected
							if(txtInputFile.getText() != null){
								File fInputFolder = new File(txtInputFile.getText());
								if (!(fInputFolder.isDirectory())) {
									JOptionPane.showMessageDialog(null, "Specified input folder doesn't exist ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
									return;
								} else {
									fileSelectionTableDetails("TextField");
								}
								//get the current directory path
								File fCurrentFolder = new File(".");
								String strCurrentPath=new File(fCurrentFolder.getCanonicalPath()).getParent();
								//System.out.println("The CurrentPath is >>>>>>>>>>>>>> " + strCurrentPath);
								pwPrintWriter.println("The CurrentPath is >>>>>>>>>>>>>> " + strCurrentPath);
	//if the destination folder is different from the current directory where the application is running, display a message that style sheets will not be applied							
								if (!((strCurrentPath.trim()).equalsIgnoreCase(txtDestFile.getText().trim()))) {
								JOptionPane.showMessageDialog(null, "Destination folder is the not the same as the Application folder("+strCurrentPath+"),\n Hence stylesheets will not be applied for the generated files ", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
							
									return;
								}
							} else{
									//inform the user to select a folder
									JOptionPane.showMessageDialog(null, "Please select Input LAS File Folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
									txtDestFile.setText("");
									//setting focus to window
									txtDestFile.requestFocusInWindow(); 
							}			
						} else {
								//if the selected destination folder does not exist, display the below message
								JOptionPane.showMessageDialog(null, "Specified destination folder doesn't exist ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
								txtDestFile.requestFocusInWindow();
								return;
						}
						System.out.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
						pwPrintWriter.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
					}
				} if ((txtDestFile.getText() != null) && (txtDestFile.getText().trim().equals("") == true)){
					DEST_FILE_DIRECTORY = "";
					showAndCheckFolder();
				} else{
						//inform the user to select a folder
						JOptionPane.showMessageDialog(null, "Please select Input LAS File Folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						txtDestFile.setText("");
						DEST_FILE_DIRECTORY = "";
						//setting focus to window
						txtDestFile.requestFocusInWindow(); 
				}				
			}catch(Exception eException){
				bnErrorFlag = true;
				System.out.println("The exception occured on entering  destination folder " + eException);
				pwPrintWriter.println("The exception occured on entering  destination folder " + eException);
			}
		}else if(ae.getSource() == btnNext) 	{
			//check the input directory, destination directory and the file selection and display the mandatory data Parameter section screen
			try {
				//get the mapping file
				String strFileName=(String)cbnMappingFiles.getSelectedItem();
				//set the mapping file
				objLasDataFile.setMappingFile((String)cbnMappingFiles.getSelectedItem());
				pwPrintWriter.println("The New mapping File from ComboBox is  " + objLasDataFile.getMappingFile());
				MappingReader   mappingReader=new MappingReader();
				//get the LAS WITSML elements for the wells from the mapping file
				Hashtable htCustomizeWells = mappingReader.LASWITSMLElements("wells",strFileName);
				//get the LAS WITSML elements for the well logs from the mapping file
				Hashtable htCustomizeWellLogs = mappingReader.LASWITSMLElements("wellLogs",strFileName);
				//if no elements are found, inform the user that the mapping file is not valid
				if(htCustomizeWells == null && htCustomizeWellLogs == null)	{
					JOptionPane.showMessageDialog(null, "The selected mapping file is not valid. Please select another file", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 	
					//set the mapping to default mapping
					//cbnMappingFiles.setSelectedItem(strMappingFile+"\\Default.xml");
				} else{
					//set the destination folder
					if(txtDestFile.getText() != null){
						DEST_FILE_DIRECTORY = txtDestFile.getText();			
						//System.out.println("btnNext DEST_FILE_DIRECTORY"+DEST_FILE_DIRECTORY+"..");
					}
					//get the input LAS file folder
					File flInputFile = new File(txtInputFile.getText());
					//get the destinator folder
					File flDestinationFile = new File(txtDestFile.getText());
					//if input and destination folders are not selected
					if ((txtInputFile.getText() == null) || (txtInputFile.getText().equals(""))) {
						JOptionPane.showMessageDialog(null, "Please select input LAS file folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						txtInputFile.requestFocusInWindow(); 
					} else if (!(flInputFile.isDirectory())) { //if input LAS folder is not a valid directory
						JOptionPane.showMessageDialog(null, "Specified input folder doesn't exist ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						
						if (dmCurrentData != null) {
							int intRowCnt=jtCurrentTable.getRowCount();
							if(intRowCnt > 0) {
								dmCurrentData.setRowCount(0);
								LAS_FILE_SELECTED = "";
								intRowCnt = 0;
							}
						}
						txtInputFile.selectAll();
						txtInputFile.requestFocusInWindow(); 
						//setting focus to window
						//btnInputFile.requestFocusInWindow(); 
					} else if ((txtDestFile.getText() == null) || (txtDestFile.getText().equals(""))) { //if destination folder is not selected
						JOptionPane.showMessageDialog(null, "Please select Destination folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						txtDestFile.requestFocusInWindow(); 
					} else if (!(flDestinationFile.isDirectory())) { //if destination folder does not exist
						JOptionPane.showMessageDialog(null, "Specified destination folder doesn't exist ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						//setting focus to window
						btnDestFile.requestFocusInWindow(); 
					} else {
						String strInputFileSelected = "";
						//get the selected row the file current files grid
						ROW_SELECTED = jtCurrentTable.getSelectedRow();
						if (ROW_SELECTED != -1) {
							//get the current directory path
							File fCurrentFolder = new File(".");
							String strCurrentPath = new File(fCurrentFolder.getCanonicalPath()).getParent();
							//System.out.println("The CurrentPath is >>>>>>>>>>>>>> " + strCurrentPath);	
							pwPrintWriter.println("The CurrentPath is >>>>>>>>>>>>>> " + strCurrentPath);					
							//if the destination folder is different from the current directory where the application is running, display a message that style sheets will not be applied	
							if (!((strCurrentPath.trim()).equalsIgnoreCase(txtDestFile.getText().trim()))) {
								//JOptionPane.showMessageDialog(null, "Destination folder is the not the same as the current folder, Hence stylesheets will not be applied for the generated files ", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
								JOptionPane.showMessageDialog(null, "Destination folder is the not the same as the Application folder("+strCurrentPath+"),\n Hence stylesheets will not be applied for the generated files ", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
							}
							
							//get the file from the file details vector for the selected row, and process the file 
							if(vecFileDetails.size() > 0){
								for(int intFileIndex = 0; intFileIndex < vecFileDetails.size(); intFileIndex++){
									//make the file as selected
									if (intFileIndex == ROW_SELECTED) {
										((FileDetails) vecFileDetails.elementAt(intFileIndex)).setIsSelected(true);
									}
									else {
										((FileDetails) vecFileDetails.elementAt(intFileIndex)).setIsSelected(false);
									}
								}
							
							//Get the selected LAS file
							for(int intFileIndex =0; intFileIndex < vecFileDetails.size(); intFileIndex++){
								if (((FileDetails) vecFileDetails.elementAt(intFileIndex)).getIsSelected() == true){
									//Make the last selected LAS file as the previous LAS file that is selected
									if (PREV_SELECTED_LAS_FILE != null) {
										PREV_SELECTED_LAS_FILE = LAS_FILE_SELECTED;
									}
									//set the selected LAS file
									LAS_FILE_SELECTED = ((FileDetails) vecFileDetails.elementAt(intFileIndex)).getInputFile();
									//if the no LAS file is selected earlier, make the current LAS file as the previous LAS file
									if (PREV_SELECTED_LAS_FILE == null) {
										PREV_SELECTED_LAS_FILE = LAS_FILE_SELECTED;
									}
								}
							}
							System.out.println("LAS_FILE_SELECTED "+LAS_FILE_SELECTED);
							pwPrintWriter.println("LAS_FILE_SELECTED FileSelection Next"+LAS_FILE_SELECTED);
							//System.out.println("PREV_SELECTED_LAS_FILE FileSelection Next"+PREV_SELECTED_LAS_FILE);
							pwPrintWriter.println("PREV_SELECTED_LAS_FILE FileSelection Next"+PREV_SELECTED_LAS_FILE);
							
							
							//LAS file is selected, display the data from the LAS file in mandatory Parameter section screen
							if ((strInputFileSelected != null) && (LAS_FILE_SELECTED !=null)) {
								if(objLasData == null){
									objLasData = new LasData();
								}
															
								//objLasDataFile = new LasDataFile();
								//set the input LAS file
								objLasData.setInputLasFile(LAS_FILE_SELECTED);
								//get the LAS file name
								String strLasFileName=new File(LAS_FILE_SELECTED).getName();
								pwPrintWriter.println("The fileName is >>>>>>>>>>>>>>>>>>> " + strLasFileName);
								int intFileNameIndex = -1;
								//check if the file ends with .LAS or .las
								intFileNameIndex = strLasFileName.indexOf(".LAS");
								if (intFileNameIndex == -1) {
									intFileNameIndex = strLasFileName.indexOf(".las");
								}
								pwPrintWriter.println("intFileNameIndex"+intFileNameIndex);
								if (intFileNameIndex != -1) {
									//set the destination folder
									objLasData.setDestFile(DEST_FILE_DIRECTORY+"/"+strLasFileName.substring(0,intFileNameIndex));
								}
															
								System.out.println(" DEST_FILE_DIRECTORY is "+DEST_FILE_DIRECTORY);
								pwPrintWriter.println(" <<<<<<< DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY);
								//System.out.println("Loading InputLasFile "+objLasData.getInputLasFile());
								pwPrintWriter.println("Loading InputLasFile "+objLasData.getInputLasFile());
								
								//load file data, instantiate LasData object 
								objLasDataFile.LoadFileData(objLasData.getInputLasFile());
								
								//display the parameter section screen
								c.remove(pnNavFileSelection);
								//jlMandData.setBackground(btnMandData.getBackground());
								jlMandData.setBackground(new Color(239,247,251));
								jlMandData.setBorder(BorderFactory.createLineBorder(Color.black));
								pnMandData.removeAll();
								pnNavMandData.remove(pnMandData);
								validate();
								pnMandData.setLayout(new GridLayout(2,1));
								pnMandData.add(btnMandData);
								pnMandData.add(jlMandData);
								//pnNavMandData.setLayout(new GridLayout(5,1)); for displaying optional data
								pnNavMandData.setLayout(new GridLayout(4,1));
								pnNavMandData.setBackground(new Color(255,245,219));
								
																
								pnNavMandData.add(btnFileSelection);
								pnNavMandData.add(pnMandData);
								
								pnNavMandData.add(btnConvert);
								pnNavMandData.add(btnConvertSummary);
								//add the panel to container
								c.add(pnNavMandData,BorderLayout.WEST);
								
								pwPrintWriter.println(" @@@@@@@@@ PREV_SELECTED_LAS_FILE    "+PREV_SELECTED_LAS_FILE+".....");
								pwPrintWriter.println(" @@@@@@@@@ LAS_FILE_SELECTED   "+LAS_FILE_SELECTED+".....");
								//check the file selections and clear the previous files parameter and curve data
								if ((PREV_SELECTED_LAS_FILE != null) && (LAS_FILE_SELECTED != null)) {
									if (!(PREV_SELECTED_LAS_FILE.equals(LAS_FILE_SELECTED))) {
										pwPrintWriter.println(" selections are not the same");
										//SaveSelectionFile name is reseted to null if the selection lasfile name is different
										objLasDataFile.setSelectionFile(null);
										if (objLasDataFile != null) {
											//clear parameter data
											if (objLasDataFile.getLasData().getParameterSectionData() != null) {
												objLasDataFile.getLasData().getParameterSectionData().clear();
												objLasDataFile.getLasData().getRunParameterSectionData().clear();
												objLasDataFile.getLasData().getWellSectionData().clear();
												objLasDataFile.getLasData().getParameterData().getWellSectionData().clear();
											}
											alParameterData.clear();
											//clear curve data
											if (objLasDataFile.getLasData().getLogDefns() != null) {
												objLasDataFile.getLasData().getLogDefns().clear();
											}
											alCurveData.clear();
											pwPrintWriter.println("CLEARED ALL PREVIOUS DATA ");
										}
									} 
								}
								
								//set Parameter selected on the left menu
								MAND_DATA_SELECTED = strMandParameter;
								
								//display the parameter section screen
								ParameterDetails();
								//btnFileSelection.setBackground(new Color(183,255,255));
								btnFileSelection.setBackground(new Color(239,247,251));
								//make mandatory data as selected on the left menu
								btnMandData.setBackground(new Color(239,247,251));
								//jlMandData.setBackground(new Color(255,245,219));
								jlMandData.setBackground(new Color(239,247,251));
								
							
								//select parameter on left menu
								jlMandData.setSelectedIndex(0);					
								//enable other links on the left menu
								btnFileSelection.setEnabled(true);
								btnMandData.setEnabled(true);
								
								btnConvert.setEnabled(true);
								//disable conversion summary link
								btnConvertSummary.setEnabled(false); 
								//show the screen
								validate();
								repaint();
								}
							}else{
								//no LAS files exists in the selected input folder
								JOptionPane.showMessageDialog(null, "Please enter valid inputLAS file folder to load the files ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
							}
						
						} else {
							//no file is selected for conversion
							JOptionPane.showMessageDialog(null, "Please select a file for LAS WITSML conversion ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						}
					}
				}
			}catch (ConverterException eConverterException) {
				System.out.println(" Converter Exception occured on clicking Next button  "+eConverterException);
				pwPrintWriter.println(" Converter Exception occured on clicking Next button  "+eConverterException);
			} 
			catch(Exception e){
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking Next button " + e);
				pwPrintWriter.println("Exception occured on clicking Next button " + e);
			}
		} else if(ae.getSource() == btnExit) {
			//quit from the application
			System.exit(1);
		} else if(ae.getSource() == btnMandData){
			//mandatory data selection is selected on left menu
			try {
				//set the Parameter as selected on the menu
				MAND_DATA_SELECTED = strMandParameter;
				setMandatoryData();
				ParameterDetails();
				
				//set string painted for progress bar
				progressBar.setStringPainted(false);			
							
				
				btnFileSelection.setBackground(new Color(239,247,251));
				btnMandData.setBackground(new Color(239,247,251));
				//jlMandData.setBackground(new Color(255,245,219));
				jlMandData.setBackground(new Color(239,247,251));
				btnConvertSummary.setBackground(new Color(239,247,251));
				btnConvert.setBackground(new Color(239,247,251));
				//select parameter of mandatory data of left menu
				jlMandData.setSelectedIndex(0);					
				validate();
				repaint();
			}  catch (Exception eException) {
					bnErrorFlag = true;
					System.out.println("Exception occured on clicking Mandatory Data link " + eException);
					pwPrintWriter.println("Exception occured on clicking Mandatory Data link " + eException);
			}	
		}  //end if mandatory Data
		else if (ae.getSource() == btnParamNext){
			try{
				//System.out.println("clicked paramNext MAND_DATA_SELECTED"+MAND_DATA_SELECTED);
				pwPrintWriter.println("clicked paramNext MAND_DATA_SELECTED"+MAND_DATA_SELECTED);
				//set mandatory data from UI into an object
				setMandatoryData();
				MAND_DATA_SELECTED = strMandCurveData;
				//display next screen
				ParameterDetails();
				//select curve data of mandatory data of left menu
				jlMandData.setSelectedIndex(1);
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking Next button of Parameter data screen " + eConverterException);
				pwPrintWriter.println("Exception occured on clicking Next button of Parameter data screen " + eConverterException);
			} 
		} else if (ae.getSource() == btnParamPrev) {
			try{
				pwPrintWriter.println("clicked param Previous");
				MAND_DATA_SELECTED = strMandParameter;
				//set mandatory data from UI into an object
				setMandatoryData();
				
				pnCurrentFile.removeAll();
				pnMainPanel.removeAll();
				c.removeAll();
				validate();
				//get LAS files and display file selection grid
				fnDisplayFileSection();
				fileSelectionTableDetails("Button");
				//make file selection as active link of the left menu
				//btnFileSelection.setBackground(clActiveButton);
				btnFileSelection.setBackground(new Color(239,247,251));
				
				//enable file selection link and disable  other links of the left menu
				btnFileSelection.setEnabled(true);
				btnMandData.setEnabled(false);
				
				btnConvert.setEnabled(false);
				btnConvertSummary.setEnabled(false);
				validate();
				repaint();	
							
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking Prev button of Parameter data screen " + eConverterException);
				pwPrintWriter.println("Exception occured on clicking Prev button of Parameter data screen " + eConverterException);
			} 		
		} else if(ae.getSource() == btnBackFileSelection) {
			try{
				System.out.println("Back to File Selection HomePage");
				//get the previous selected LAS file
				PREV_SELECTED_LAS_FILE = LAS_FILE_SELECTED;
				pwPrintWriter.println("PREV_SELECTED_LAS_FILE    "+PREV_SELECTED_LAS_FILE);
				
				pnCurrentFile.removeAll();
				pnMainPanel.removeAll();
				c.removeAll();
				validate();
				//get LAS files and display file selection grid
				fnDisplayFileSection();
				fileSelectionTableDetails("Button");
				//make file selection as active link of the left menu
				//btnFileSelection.setBackground(clActiveButton);
				btnFileSelection.setBackground(new Color(239,247,251));
				
				//enable file selection link and disable  other links of the left menu
				btnFileSelection.setEnabled(true);
				btnMandData.setEnabled(false);
				
				btnConvert.setEnabled(false);
				btnConvertSummary.setEnabled(false);
				validate();
				repaint();
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking Back to File selection of Conversion summary screen " + eConverterException);
				pwPrintWriter.println("Exception occured on clicking Back to File selection of Conversion summary screen " + eConverterException);
			} 		
		}
		else if (ae.getSource() == btnCurvePrev){
			try{
				//System.out.println("clicked CurvePrev MAND_DATA_SELECTED"+MAND_DATA_SELECTED);
				pwPrintWriter.println("clicked CurvePrev MAND_DATA_SELECTED"+MAND_DATA_SELECTED);
				//set the mandatory data in  an object
				setMandatoryData();
				//display parameter data screen
				MAND_DATA_SELECTED = strMandParameter;
				ParameterDetails();
				//select Parameter of mandatory data of left menu
				jlMandData.setSelectedIndex(0);
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking Prev button of Curve data selection " + eConverterException);
				pwPrintWriter.println("Exception occured on clicking Prev button of Curve data selection " + eConverterException);
			} 	
		} else if (ae.getSource() == btnCurveNext){
			//System.out.println("clicked CurveNext MAND_DATA_SELECTED " +MAND_DATA_SELECTED);
			pwPrintWriter.println("clicked CurveNext MAND_DATA_SELECTED " +MAND_DATA_SELECTED);
			btnProcessConversion.requestFocusInWindow();
			try{
				boolean blnIsRowSelected=false;
				int intRowsSelected=0;
				//check if any rows are selected
				if (jtMandatory != null) {
					for(int intSelectionIndex = 0; intSelectionIndex < jtMandatory.getRowCount(); intSelectionIndex++) {
						Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSelectionIndex,0);
						//System.out.println("IS blnSelected ..."+blnSelected);
						//curve is selected			
						if (blnSelected.booleanValue() ==true) {
							//blnIsRowSelected = true;
							intRowsSelected++;
						}
					}
					pwPrintWriter.println("NUM CURVES SELECTED ..."+intRowsSelected);
				}
				//no curves are selected, inform user
				//if (blnIsRowSelected == false) {
				if (intRowsSelected < 2) {
					JOptionPane.showMessageDialog(null, "Atleast two Curves are to be selected, Please select another curve", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
					
				} 
				//curves are selected
				//if (blnIsRowSelected == true){
				else if (intRowsSelected >= 2){
					//set progress bar status value as zero
					progressBar.setValue(0);
					//set the curve data in  an object
					setMandatoryData();
					//display the conversion screen
					fnConvertFileSection();
					//set Curve as selected mandatory data
					MAND_DATA_SELECTED = strMandCurveData;
					//enable links of the left menu
					
												
					btnFileSelection.setBackground(new Color(239,247,251));
					btnMandData.setBackground(new Color(239,247,251));
					//jlMandData.setBackground(new Color(255,245,219));
					jlMandData.setBackground(new Color(239,247,251));
					btnConvertSummary.setBackground(new Color(239,247,251));
					btnConvert.setBackground(new Color(239,247,251));
					btnProcessConversion.requestFocusInWindow();
					validate();
				}
			} catch(ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking Next button of Curve data selection " + eConverterException);
				pwPrintWriter.println("Exception occured on clicking Next button of Curve data selection " + eConverterException);
			}
		} else if (ae.getSource() == btnProcessConversion) {
			try{
				pwPrintWriter.println(" VECTOR SIZE ... "+objLasDataFile.getOutputFiles());
				
				//set string painted for progress bar
				progressBar.setStringPainted(true);
				
				//check if there are any output files generated in previous run 	
				if (objLasDataFile.getOutputFiles() == null) {
					//no previous output files are present
					objLasDataFile.setOutputFiles(new Vector());
				}
				else if (objLasDataFile.getOutputFiles().size() > 0) {
					Vector vec = objLasDataFile.getOutputFiles();
					//clear previous output files
					vec.clear();
					objLasDataFile.setOutputFiles(vec);
				}
				setCursor(new Cursor(Cursor.WAIT_CURSOR));
				/*btnConverCustomizeMapping.setEnabled(false);
				btnConversionBack.setEnabled(false);*/
				//perform the conversion
				objSwingWorker = new SwingWorker() {
					public Object construct() {
						
								return doWork();
					}
					public void finished() {
							System.out.println("finished");
							pwPrintWriter.println("finished");
							setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
							/*btnConverCustomizeMapping.setEnabled(true);
							btnConversionBack.setEnabled(true);*/
					}
				};
		    objSwingWorker.start(); 
			}catch(Exception eException) {
				bnErrorFlag = true;
				System.out.println("The error in Process Conversion is " + eException.getMessage());
				pwPrintWriter.println("The error in Process Conversion is " + eException.getMessage());
			}
		} else if(ae.getSource() == btnFileSelection){
			try{
				pwPrintWriter.println("The file selection is clicked " +vecFileDetails.size());
				pwPrintWriter.println(" @@@@@@@@@ PREV_SELECTED_LAS_FILE    "+PREV_SELECTED_LAS_FILE+".....");
				pwPrintWriter.println(" @@@@@@@@@ LAS_FILE_SELECTED    "+LAS_FILE_SELECTED+".....");
				//check previous selected LAS file and the current LAS file
				if ((PREV_SELECTED_LAS_FILE != null) && (LAS_FILE_SELECTED != null)) {
					//if both the files are not same, do not clear parameter and curve data 
					if (!(PREV_SELECTED_LAS_FILE.equals(LAS_FILE_SELECTED))) {
						if (objLasDataFile != null) {
							if (objLasDataFile.getLasData().getParameterSectionData() != null) {
								objLasDataFile.getLasData().getParameterSectionData().clear();
							}
							if (objLasDataFile.getLasData().getLogDefns() != null) {
								objLasDataFile.getLasData().getLogDefns().clear();
							}
							pwPrintWriter.println("CLEARED ALL PREVIOUS DATA ");
						}
					} 
				} else if (PREV_SELECTED_LAS_FILE == null) {
					PREV_SELECTED_LAS_FILE = LAS_FILE_SELECTED;
				}
				pnCurrentFile.removeAll();
				pnMainPanel.removeAll();
				c.removeAll();
				validate();
				//get files and display file selection grid
				fnDisplayFileSection();
				//commented for removing unnecessary alert that is displayed when clicked on File Selection of left menu
				//fileSelectionTableDetails("Button"); 
				fileSelectionTableDetails("");
				//enable file selection and disable all other links of the left menu
				//btnFileSelection.setBackground(clActiveButton);
				btnFileSelection.setBackground(new Color(239,247,251));
													
				btnFileSelection.setEnabled(true);
				btnMandData.setEnabled(false);
				
				btnConvert.setEnabled(false);
				btnConvertSummary.setEnabled(false);
				validate();
				repaint();
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println("Exception occured on clicking File Selection button of the left menu " + eConverterException);
				pwPrintWriter.println("Exception occured on clicking File Selection button of the left menu " + eConverterException);
			} 		
		} else if(ae.getSource() == btnLoadSelection){
			try{
			//load the selections and display the load selection screen
			MappingSelector jdMappingSelector=new MappingSelector(this,"Load Selection");
			jdMappingSelector.setBounds(250,200,550,250);
			jdMappingSelector.loadSelections();
			jdMappingSelector.show(); 
			//System.out.println("The selectionFile name from loadselections " + objLasDataFile.getSelectionFile());
			} catch(Exception eException) {
				bnErrorFlag = true;
				System.out.println(" Exception in loading selections "+eException.getMessage());
				pwPrintWriter.println(" Exception in loading selections "+eException.getMessage());
			}
	} else if(ae.getSource() == btnSaveSelection){
		try{
			if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
				boolean blnIsCurveSelected = false;
				//check if any curves are selected for saving
				if (jtMandatory != null) {
					for(int intCurveIndex=0; intCurveIndex < jtMandatory.getRowCount(); intCurveIndex++) {
						boolean blnSelected = ((Boolean) jtMandatory.getValueAt(intCurveIndex,0)).booleanValue();
						if (blnSelected) {
							blnIsCurveSelected = true;
						}
					}
				}
				if (blnIsCurveSelected == false) {
					JOptionPane.showMessageDialog(null, "No curves are selected for saving ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
				} else {
					//save the selections
					saveSelections();
				}
			}
		} catch(Exception eException) {
			bnErrorFlag = true;
			System.out.println(" Exception in saving selections "+eException.getMessage());
			pwPrintWriter.println(" Exception in saving selections "+eException.getMessage());
		}
 	} else if(ae.getSource() == btnCustomizeMapping){
		//display the customize mapping screen and set the mapping file
		String strSelectedCmbName = null;
		try{
			if(jdCustomizeMapping != null){
				pwPrintWriter.println("The mapping in laswitsml is  :::::::::::::::::::::: " + jdCustomizeMapping.getMappingFile());
			}
			
			pwPrintWriter.println("The value is :::::::::  " + cbnMappingFiles.getSelectedItem());
			//if(cbnMappingFiles.getSelectedItem()!=null)
			if(cbnMappingFiles.getSelectedItem() != null)
			{
			objLasDataFile.setMappingFile((String)cbnMappingFiles.getSelectedItem());
			strSelectedCmbName = (String)cbnMappingFiles.getSelectedItem();
			}
			
			pwPrintWriter.println("TAfter setting the value  " + objLasDataFile.getMappingFile());
			
			jdCustomizeMapping = new CustomizeMapping(this,"Customize Mapping");
			jdCustomizeMapping.setBounds(250,200,650,250);
			jdCustomizeMapping.show(); 
			pwPrintWriter.println("The mapping in laswitsml is " + jdCustomizeMapping.getMappingFile());
			objLasDataFile.setMappingFile(jdCustomizeMapping.getMappingFile());
			pwPrintWriter.println("The mapping in latwitsml from lasdatafile  " + objLasDataFile.getMappingFile());
			//remove all items(if any) from the mappings combo box
			cbnMappingFiles.removeAllItems();
			//get mappings from the mappings folder
			File fCurrentFolder = new File(".");
			String strPath = new File(fCurrentFolder.getCanonicalPath()).getParent();
			String strMappingFile = strPath+"/mappings";
			
			File fMappingFile = new File(strMappingFile);
			String strList[] = fMappingFile.list();
			
			//add the mapping xml files to combo box
			for(int intFileIndex = 0; intFileIndex < strList.length; intFileIndex++) {
				File fTmp = new File(strList[intFileIndex]);
				String strFileName = strPath+"/"+fTmp.getName();
				if (strFileName != null)  {
					if((strFileName.endsWith("xml")) || (strFileName.endsWith("XML"))) {
						cbnMappingFiles.addItem(strMappingFile+"/"+strList[intFileIndex]);
					}
				}
			}
			pwPrintWriter.println("The mapping file is " + jdCustomizeMapping.getMappingFile());
			pwPrintWriter.println("The file name is " + strSelectedCmbName);
			if(jdCustomizeMapping.getMappingFile() != null){
				cbnMappingFiles.setSelectedItem(jdCustomizeMapping.getMappingFile());
			}else{
				cbnMappingFiles.setSelectedItem(strMappingFile+"/Default.xml");
				if(strSelectedCmbName != null){
					cbnMappingFiles.setSelectedItem(strSelectedCmbName);
				}
			}
			validate();
			repaint();
		} catch(Exception eException) {
			bnErrorFlag = true;
			System.out.println(" Exception in Customize Mapping selection "+eException);
			pwPrintWriter.println(" Exception in Customize Mapping selection "+eException);
		}
	} else if(ae.getSource() == cbnOutputFile){
			try {
				//get the selected item from the output files list box
				/*String strSelectedFile = (String)cbnOutputFile.getSelectedItem();
				if (strSelectedFile != null && !strSelectedFile.equalsIgnoreCase("View Generated Files")) {
					//open the output file with Internet Explorer
					Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + strSelectedFile);
					//System.out.println("The file is processed");
					pwPrintWriter.println("The file is processed");
				}else{
					pwPrintWriter.println("View generated files selected");
					//setting focus to JTable for removing issue of disappearing JComboBox
					//jtSummary.requestFocusInWindow(); 
				}*/
				//get the selected item from the output files list box
				String strSelectedFile = (String)cbnOutputFile.getSelectedItem();
				if (strSelectedFile != null && !strSelectedFile.equalsIgnoreCase("View Generated Files")) {
					boolean windows = isWindowsPlatform();
					String cmd = null;
					strSelectedFile = "file://"+ strSelectedFile;
					//open the output file with Internet Explorer
					/*String command = "C:\\Program Files\\Internet Explorer\\iexplore.exe "+strSelectedFile;
					Process child = Runtime.getRuntime().exec(command); */
					if (windows) {
						//Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + strSelectedFile);
						/*cmd = WIN_PATH + " " + WIN_FLAG + " " + strSelectedFile;
						Process p = Runtime.getRuntime().exec(cmd);*/
						System.out.println("In windows");
						Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + strSelectedFile);
						}
					else {
						// Under Unix, Netscape has to be running for the "-remote"
						// command to work.  So, we try sending the command and
						// check for an exit value.  If the exit command is 0,
						// it worked, otherwise we need to start the browser.
						// cmd = 'netscape -remote openURL(http://www.javaworld.com)'
						//cmd = UNIX_PATH + " " + UNIX_FLAG + "(" + strSelectedFile + ")";
						cmd = UNIX_PATH + " "  + strSelectedFile ;
						System.out.println("The unix path command is " + cmd);
						Process p = Runtime.getRuntime().exec(cmd);
						try {
							// wait for exit code -- if it's 0, command worked,
							// otherwise we need to start the browser up.
							int exitCode = p.waitFor();
							//System.out.println("EXITCODE<"+exitCode+">");
							if (exitCode != 0) {
								// Command failed, start up the browser
								// cmd = 'netscape http://www.javaworld.com'
								cmd = UNIX_PATH + " "  + strSelectedFile;
								p = Runtime.getRuntime().exec(cmd);
								}
						}
						catch(InterruptedException x) {
							System.err.println("Error bringing up browser, cmd='" + cmd + "'");
							System.err.println("Caught: " + x);
						}
					}	
					System.out.println("The file is processed");
					pwPrintWriter.println("The file is processed");
				}else{
					pwPrintWriter.println("View generated files selected");
					//setting focus to JTable for removing issue of disappearing JComboBox
					//jtSummary.requestFocusInWindow(); 
				}
				jtSummary.changeSelection(0,2,false,false); 
			} catch(IOException eIOException){
				//bnErrorFlag = true;
				System.out.println("Error in opening the file in browser " + eIOException);
				pwPrintWriter.println("Error in opening the file in browser " + eIOException);
			}	
	} else if(ae.getSource() == chCurveSelection) {
			// check if select all is selected
			if(chCurveSelection.isSelected() == true) {
				//select all the curves of the table
				for(int intSelectionIndex = 0; intSelectionIndex < jtMandatory.getRowCount(); intSelectionIndex++) {
					jtMandatory.setValueAt(new Boolean(true),intSelectionIndex,0);
				}
				//make select all as checked
				chCurveSelection.setSelected(true);
			}
			//select all is not selected
			if(chCurveSelection.isSelected() == false) {
				//deselect all the curves of the table
				for(int intSelectionIndex = 0; intSelectionIndex < jtMandatory.getRowCount(); intSelectionIndex++) {
					String strMnemonic = (String) jtMandatory.getValueAt(intSelectionIndex,1);
					//System.out.println("Mnemonic is ..."+strMnemonic);
					if ((strMnemonic.equalsIgnoreCase("DEPT")) || (strMnemonic.equalsIgnoreCase("TIME"))) {
						jtMandatory.setValueAt(new Boolean(true),intSelectionIndex,0);
					} else {
						jtMandatory.setValueAt(new Boolean(false),intSelectionIndex,0);
					}
				}
				//deselect select all check box
				chCurveSelection.setSelected(false);
			}
	}else if(ae.getSource() == chParameterSelection) {
			// check if select all is selected
			if(chParameterSelection.isSelected() == true) {
				//select all the curves of the table
				for(int intSelectionIndex = 0; intSelectionIndex < jtMandatory.getRowCount(); intSelectionIndex++) {
					jtMandatory.setValueAt(new Boolean(true),intSelectionIndex,0);
				}
				//make select all as checked
				chParameterSelection.setSelected(true);
			}
			//select all is not selected
			if(chParameterSelection.isSelected() == false) {
				pwPrintWriter.println("&&&& well param size ..."+objLasDataFile.getWellParamSize());
				//deselect all the curves of the table
				for(int intSelectionIndex = 0; intSelectionIndex < jtMandatory.getRowCount(); intSelectionIndex++) {
					String strMnemonic = (String) jtMandatory.getValueAt(intSelectionIndex,1);
					//System.out.println("Mnemonic is ..."+strMnemonic);
					if (objLasDataFile != null) {
						if (objLasDataFile.getWellParamSize() > 0) {
							if (intSelectionIndex < objLasDataFile.getWellParamSize()) {
								jtMandatory.setValueAt(new Boolean(true),intSelectionIndex,0);
							} else {
								jtMandatory.setValueAt(new Boolean(false),intSelectionIndex,0);
							}
						}
					}
					
				}
				//deselect select all check box
				chParameterSelection.setSelected(false);
			}
	}
	else if(ae.getSource() == btnConverCustomizeMapping) 	{
		try{
			pwPrintWriter.println("The mapping in latwitsml conversion mapping >>>>>>>>>>   " + objLasDataFile.getMappingFile());
			pwPrintWriter.println("The mapping file in conversion mapping >>>>>>>>>> is " + jdCustomizeMapping);
			
			//If Customize Mapping button in File Selection screen is not Clicked.
			if(jdCustomizeMapping == null){
				//show the customize mapping screen and set the mapping file
				jdCustomizeMapping = new CustomizeMapping(this,"Customize Mapping");
				jdCustomizeMapping.setBounds(250,200,650,250);
				jdCustomizeMapping.show(); 
				objLasDataFile.setMappingFile(jdCustomizeMapping.getMappingFile());
				pwPrintWriter.println("The mapping in latwitsml from lasdatafile  " + objLasDataFile.getMappingFile());
			}else{
				pwPrintWriter.println("The Customize Mapping is not null ");
				//user changes the mappings and try saving without giving a name
				if(jdCustomizeMapping.getMappingFile() == null){
					pwPrintWriter.println("The Customize Mapping  value is null and value is " + jdCustomizeMapping.getMappingFile());
					//show the customize mapping screen and set the mapping file
					jdCustomizeMapping = new CustomizeMapping(this,"Customize Mapping");
					jdCustomizeMapping.setBounds(250,200,650,250);
					jdCustomizeMapping.show(); 
					objLasDataFile.setMappingFile(jdCustomizeMapping.getMappingFile());
					pwPrintWriter.println("The mapping in latwitsml from lasdatafile  " + objLasDataFile.getMappingFile());
				}else{
					//user changes the mappings 
					pwPrintWriter.println("The Customize Mapping  value is not null and value is " + jdCustomizeMapping.getMappingFile());
					String strDefaultMappingPath = jdCustomizeMapping.getMappingFile();
					String strDefaultMappingName = strDefaultMappingPath.substring(strDefaultMappingPath.lastIndexOf("/")+1,strDefaultMappingPath.length());
					pwPrintWriter.println("The Customize Mapping  File name is " + strDefaultMappingName);
					if(strDefaultMappingName.trim().equalsIgnoreCase("Default.xml")){
						//show the customize mapping screen and set the mapping file
						jdCustomizeMapping = new CustomizeMapping(this,"Customize Mapping");
						jdCustomizeMapping.setBounds(250,200,650,250);
						jdCustomizeMapping.show(); 
						objLasDataFile.setMappingFile(jdCustomizeMapping.getMappingFile());
						pwPrintWriter.println("The mapping in latwitsml from lasdatafile  " + objLasDataFile.getMappingFile());
					}else{
					//show the customize mapping screen and set the mapping file
						pwPrintWriter.println("The mapping in latwitsml from lasdatafile  " + objLasDataFile.getMappingFile());
						jdConvCustomizeMapping = new ConversionCustomizeMapping(this,"Customize Mapping");
						jdConvCustomizeMapping.setBounds(250,200,650,250);
						jdConvCustomizeMapping.show(); 
						objLasDataFile.setMappingFile(jdConvCustomizeMapping.getMappingFile());
					}
				}
			}		
			//remove all mapping files from the combo box
			cbnMappingFiles.removeAllItems();
			//get the current path
			File fCurrentFolder = new File(".");
			String strPath=new File(fCurrentFolder.getCanonicalPath()).getParent();
			//set the mapping directory
			String strMappingFile = strPath+"/mappings";
			
			File fMappingFile = new File(strMappingFile);
			String strList[] = fMappingFile.list();
				
			//add the mapping xml files to combo box
			for(int intFileIndex = 0; intFileIndex < strList.length; intFileIndex++) {
				File fTmp = new File(strList[intFileIndex]);
				String strFileName = strPath+"/"+fTmp.getName();
				if (strFileName != null)  {
					if((strFileName.endsWith("xml")) || (strFileName.endsWith("XML"))) {
						cbnMappingFiles.addItem(strMappingFile+"/"+strList[intFileIndex]);
					}
				}
			}
			//set the selected mapping file
			cbnMappingFiles.setSelectedItem(jdCustomizeMapping.getMappingFile());
			validate();
			repaint();
		}catch(Exception eException) {
				bnErrorFlag = true;
				System.out.println(" Exception in Customize Mapping selection "+eException);
				pwPrintWriter.println(" Exception in Customize Mapping selection "+eException);
		}
	}else if(ae.getSource() == btnConversionBack){
			try{
				//System.out.println("btnConversionBack "+MAND_DATA_SELECTED);
				pwPrintWriter.println("btnConversionBack "+MAND_DATA_SELECTED);
				//set curve data
				setMandatoryData();
				MAND_DATA_SELECTED = strMandCurveData;
				//display the curve selection screen
				ParameterDetails();
				//select curve data of the left menu
				jlMandData.setSelectedIndex(1);
				//enable curve data selection				
				btnConvert.setBackground(new Color(255,245,219));
				btnMandData.setBackground(new Color(255,245,219));
				//jlMandData.setBackground(btnMandData.getBackground());
				jlMandData.setBackground(new Color(239,247,251));
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println(" Exception  "+eConverterException);
				pwPrintWriter.println(" Exception  "+eConverterException);
			} 	
	} else if(ae.getSource() == btnHelp){
		pwPrintWriter.println("Clicked on Help in Homepage");
		try {
			String url="FileSelection";
			showHelp(url);
		} catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println("IOException is .."+eException);
			pwPrintWriter.println("IOException is .."+eException);
		}
	}else if(ae.getSource() == btnParameterHelp){
		pwPrintWriter.println("Clicked on Help in Homepage");
		try {
			String url="ParameterData";
			showHelp(url);
		} catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println("IOException is .."+eException);
			pwPrintWriter.println("IOException is .."+eException);
		}
	}else if(ae.getSource() == btnCurveHelp){
		pwPrintWriter.println("Clicked on Help in Homepage");
		try {
			String url="CurveData";
			showHelp(url);
		} catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println("IOException is .."+eException);
			pwPrintWriter.println("IOException is .."+eException);
		}
	}else if(ae.getSource() == btnProcessHelp){
		pwPrintWriter.println("Clicked on Help in Homepage");
		try {
			String url="ConversionStatus";
			showHelp(url);
		} catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println("IOException is .."+eException);
			pwPrintWriter.println("IOException is .."+eException);
		}
	}else if(ae.getSource() == btnSummaryHelp){
		pwPrintWriter.println("Clicked on Help in Homepage");
		try {
			String url="SummaryReport";
			showHelp(url);
		} catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println("IOException is .."+eException);
			pwPrintWriter.println("IOException is .."+eException);
		}
	}
}//action performed

	/**
	* Try to determine whether this application is running under Windows
	* or some other platform by examing the "os.name" property.
	*
	* @return true if this application is running under a Windows OS
	*/

	public  boolean isWindowsPlatform() {
		String os = System.getProperty("os.name");
		System.out.println("The operating system is " + os);
		if ( os != null && os.startsWith(WIN_ID)){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	* Method doWork
	*This method performs the actual LAS to WITSML conversion
	* @return Object
	*
	*/
	Object doWork() {
	  try {
			//show the progress of the task
			for(int intProgressIndex = 0; intProgressIndex <=10; intProgressIndex+=4) {
				updateStatus(intProgressIndex);
				if (Thread.interrupted()) {
				    throw new InterruptedException();
				}
				Thread.sleep(500);
			}
			//process well section data
		  objLasDataFile.processWellSection(objLasData.getInputLasFile(),objLasData);
		  //show the progress of the task
			for(int intProgressIndex = 11; intProgressIndex <=45; intProgressIndex+=4) {
				updateStatus(intProgressIndex);
				if (Thread.interrupted()) {
					throw new InterruptedException();
				}
				Thread.sleep(500);
			}
			//process well Log data
		  //objLasDataFile.processLogSection(objLasData.getInputLasFile(),objLasData);
			//show the progress of the task
		  for(int intProgressIndex = 46; intProgressIndex <=76; intProgressIndex+=4) {
		    updateStatus(intProgressIndex);
				if (Thread.interrupted()) {
				    throw new InterruptedException();
				}
				Thread.sleep(500);
			}
			objLasDataFile.processLogSection(objLasData.getInputLasFile(),objLasData);
			//show the progress of the task
			 for(int intProgressIndex = 77; intProgressIndex <=96; intProgressIndex+=4) {
				 updateStatus(intProgressIndex);
				if (Thread.interrupted()) {
				    throw new InterruptedException();
				}
				Thread.sleep(500);
			}
			
			 for(int intProgressIndex = 97; intProgressIndex <=100; intProgressIndex++) {
				 updateStatus(intProgressIndex);
				if (Thread.interrupted()) {
				    throw new InterruptedException();
				}
				Thread.sleep(500);
			}
					
			//redirecting to the Summary Section
			btnConvertSummary.setEnabled(true);
			fnSummarySection();
		} catch (InterruptedException eInterruptedException) {
			updateStatus(0);
			return "Interrupted";  // SwingWorker.get() returns this
	  } catch (ConverterException eConverterException) {
			 bnErrorFlag = true; 
			System.out.println("The Converter Exception is " + eConverterException);
			pwPrintWriter.println("The Converter Exception is " + eConverterException);
	  }
	  return "All Done";         // or this
	}  
	/**
	* Method updateStatus
	*This method update the progress bar
	* @param The value to be displayed as the status
	* @return void
	*
	*/  
	public void updateStatus(final int intValue) {
		Runnable doSetProgressBarValue = new Runnable() {
			public void run() {
			progressBar.setValue(intValue);
			}
		};
		SwingUtilities.invokeLater(doSetProgressBarValue);
	}

	/**
	* Method fnSummarySection
	*This method displays the summary screen
	* @return void
	*
	*/    
	
	public void fnSummarySection() throws ConverterException{
		try{
			pnCurrentFile.removeAll();
			pnMainPanel.removeAll();
			c.remove(pnMainPanel);
			c.remove(pnNavMandData);
			
			pnNavMandData.remove(pnMandData);
			validate();
			
			//pnNavFileSelection.setLayout(new GridLayout(5,1)); //for optional data
			pnNavFileSelection.setLayout(new GridLayout(4,1));
			pnNavFileSelection.setBackground(new Color(255,245,219));
			
			btnFileSelection.setBackground(new Color(239,247,251));
			btnMandData.setBackground(new Color(239,247,251));
			btnConvertSummary.setBackground(new Color(239,247,251));
			btnConvert.setBackground(new Color(239,247,251));
			
						
			//add the menu links
			pnNavFileSelection.add(btnFileSelection);
			pnNavFileSelection.add(btnMandData);
			
			pnNavFileSelection.add(btnConvert);
			pnNavFileSelection.add(btnConvertSummary);
			
			//add FileNavigator panel to container
			c.add(pnNavFileSelection,BorderLayout.WEST); 
		
			//** starats here	
			//pnCurrentFileHdr.remove(lblCurrentFile);
			pnCurrentFileHdr.removeAll();
			pnMainPanel.setLayout(new GridLayout(3,1));
			if (lblParameterSelection != null) {
				lblParameterSelection.setText(" ");
			}
			//set the heading font and color and add to the panel
			lblParameterSelection = new JLabel(" Summary Report ",SwingConstants.LEFT);
			lblParameterSelection.setForeground(Color.white);
			lblParameterSelection.setFont(fnHdrFont);
			pnCurrentFile.setBackground(clrHeaderBg);
			pnCurrentFileHdr.add(lblParameterSelection,BorderLayout.WEST);
			pnCurrentFile.add(pnCurrentFileHdr,BorderLayout.NORTH);
			
			//data model for summary table
			dmSummary = new DefaultTableModel(strSummaryHeader,0);
			//create table with required features
			jtSummary = new JTable(dmSummary) {
				//disable cell editing
				public boolean isCellEditable(int rowIndex, int vColIndex) {
					if (vColIndex == 2) {
						return true;
					} else {
						return false;
					}
				}
				
				public TableCellRenderer getCellRenderer(int row, int column) {
					TableColumn tableColumn = getColumnModel().getColumn(column);
					TableCellRenderer renderer = tableColumn.getCellRenderer();
					if (renderer == null) {
						Class c = getColumnClass(column);
						if( c.equals(Object.class) ) {
							Object o = getValueAt(row,column);
							if( o != null )
								c = getValueAt(row,column).getClass();
						}
						renderer = getDefaultRenderer(c);
					}
					return renderer;
				}
				
				public TableCellEditor getCellEditor(int row, int column) {
					TableColumn tableColumn = getColumnModel().getColumn(column);
					TableCellEditor editor = tableColumn.getCellEditor();
					if (editor == null) {
						Class c = getColumnClass(column);
						if( c.equals(Object.class) ) {
							Object o = getValueAt(row,column);
							if( o != null )
								c = getValueAt(row,column).getClass();
						}
						editor = getDefaultEditor(c);
					}
					return editor;
				}
			};
			
			//get number of rows of summary table
			int intRowCnt = jtSummary.getRowCount();
			//get output files	
			vecFiles = objLasDataFile.getOutputFiles();
			//remove all items from output files combo box
			cbnOutputFile.removeAllItems();
			cbnOutputFile.addItem("View Generated Files");
			//add output files to the combo box
			for (int intFileIndex = intRowCnt; intFileIndex < vecFiles.size(); intFileIndex++) {
				cbnOutputFile.addItem(vecFiles.elementAt(intFileIndex));
			}
			
			String strMappingFullName = objLasDataFile.getMappingFile();
			String strMappingName = strMappingFullName.substring(strMappingFullName.lastIndexOf("/")+1,strMappingFullName.length());
			//added below code for getting saved selection file name in summary screen.
			String strSelectionFileName = objLasDataFile.getSelectionFile();
			if (strSelectionFileName == null)
			{
				strSelectionFileName = "defaultSelection";
			}
			//System.out.println("The selection file name used in application is " + strSelectionFileName+".xml");
			
			
			if (bnErrorFlag == false ){
			//add a row to summary table
				String[] strRowData = {"1",LAS_FILE_SELECTED,"","Done",strMappingName,strSelectionFileName+".xml"};
				dmSummary.addRow(strRowData);
			}else{
				String[] strRowData = {"1",LAS_FILE_SELECTED,"","Error",strMappingName,strSelectionFileName+".xml"};
				dmSummary.addRow(strRowData);
			}
			//dmSummary.addRow(strRowData);
			//add combo box as 3rd column
			jtSummary.setValueAt( cbnOutputFile, 0, 2 );
			jtSummary.setDefaultRenderer( JComponent.class, new JComponentCellRenderer() );
			jtSummary.setDefaultEditor( JComponent.class, new JComponentCellEditor() );
			//set row height
			jtSummary.setRowHeight(25);	
			//get table columns and set width	
			
			TableColumn snCol = jtSummary.getColumnModel().getColumn(0);//<-----------------
			//TableColumn inPutCol = jtSummary.getColumnModel().getColumn(1);
			
			int intColWidth = 50;
			int outColWidth = 250;
			int inputColWidth = 350;
			int statusColWidth = 50;
			
			//set column width
			/* commented the code to allow S.No to be resizable 16/6/2006
			snCol.setMinWidth(intColWidth);
			snCol.setMaxWidth(intColWidth);
			snCol.setPreferredWidth(intColWidth);*/
			
			//Commented code for allowing resizeing of columns as per UAT Testing comments.
			/***starts here ***/
			/*TableColumn outputCol = jtSummary.getColumnModel().getColumn(2);//<-----------------
			//TableColumn statusCol = jtSummary.getColumnModel().getColumn(3);//<-----------------
			//set column width
			outputCol.setMinWidth(outColWidth);
			outputCol.setMaxWidth(outColWidth);
			outputCol.setPreferredWidth(outColWidth);*/
			/***ends here ***/
			
			//disable re-ordering of columns
			jtSummary.getTableHeader().setReorderingAllowed(false);
			
			//set single row selection
			jtSummary.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			
			//disable row selection and enable column selection
			jtMandatory.setColumnSelectionAllowed(true);
			
					
			//set font, fore ground color and back ground color of jtCurrentTable header
			JTableHeader anHeader = jtSummary.getTableHeader();
			anHeader.setForeground(Color.blue);
			anHeader.setFont(fnHdrFont);
			anHeader.setBackground(new Color(255,204,153));
		
			scrollPane = new JScrollPane(jtSummary);
			scrollPane.getViewport().setBackground(new Color(246, 231, 215));
			jtSummary.setPreferredScrollableViewportSize(new Dimension(400, 200));
			pnCurrentFile.add(scrollPane,BorderLayout.CENTER);
			
			pnMainPanel.add(pnCurrentFile);
			pnButtonPanel.removeAll();
			//set shortcuts for exit and help buttons
			btnExit.setMnemonic('X');
			btnSummaryHelp.setMnemonic('H');
			btnBackFileSelection.setMnemonic('F');
			//add listener to Exit button		
			btnExit.addActionListener(this);
			
			//add buttons to buttons panel
			pnButtonPanel.add(btnBackFileSelection);
			for (int i = 0; i < 60; i++) {
				pnButtonPanel.add(new JLabel(" "));
			}
			pnButtonPanel.add(btnSummaryHelp);
			pnButtonPanel.add(btnExit);
			
			
			pnMainPanel.add(new JLabel(""));
			
			//add button panel to main panel
			pnMainPanel.add(pnButtonPanel);
			c.add(pnMainPanel,BorderLayout.CENTER);
			validate();
			repaint();
		}catch(Exception e) {
			bnErrorFlag = true;
			throw new ConverterException("Exception occured in displaying conversion summary details");
		}	
	}
	
	/**
	* Method fnConvertFileSection
	*This method displays the convert LAS to WITSML screen
	* @return void
	*
	*/   
	public void fnConvertFileSection() throws ConverterException{
		try{
			//remove all objects from the panel, and panel from the container
			pnCurrentFile.removeAll();
			pnMainPanel.removeAll();
			c.remove(pnMainPanel);
			c.remove(pnNavMandData);
			
			pnNavMandData.remove(pnMandData);
			validate();
			
			//pnNavFileSelection.setLayout(new GridLayout(5,1)); //for optional data
			pnNavFileSelection.setLayout(new GridLayout(4,1));
			pnNavFileSelection.setBackground(new Color(255,245,219));
	
			//set the background color for the buttons
			btnFileSelection.setBackground(new Color(239,247,251));
			btnMandData.setBackground(new Color(239,247,251));
			//jlMandData.setBackground(new Color(255,245,219));
			btnConvertSummary.setBackground(new Color(239,247,251));
			btnConvert.setBackground(new Color(239,247,251));
			
			pnNavFileSelection.add(btnFileSelection);
			pnNavFileSelection.add(btnMandData);
			
			pnNavFileSelection.add(btnConvert);
			pnNavFileSelection.add(btnConvertSummary);
			
			//add FileNavigator panel to container
			c.add(pnNavFileSelection,BorderLayout.WEST); 
				
			pnMainPanel.setLayout(new GridLayout(3,1));
			pnOptionalSelection = new JPanel();
			pnOptionalSelection.setLayout(new GridLayout(4,1));
			pnOptionalSelection.setBorder(BorderFactory.createLineBorder(Color.black,1));
			pnOptionalSelection.setForeground(Color.white);
			pnOptionalSelection.setBackground(Color.white);
			
			
			pnOptionalHeader = new JPanel();
			pnOptionalHeader.setBackground(Color.white);
			pnOptionalHeader.setLayout(new BorderLayout());
			
			JPanel pnOptionalEmpty = new JPanel();
			pnOptionalEmpty.setBackground(Color.white);
			pnOptionalEmpty.setLayout(new BorderLayout());
			
			JPanel pnOptionalTableEmpty = new JPanel();
			pnOptionalTableEmpty.setBackground(Color.white);
			pnOptionalTableEmpty.setLayout(new BorderLayout());
			
			JPanel pnConvertFile = new JPanel();
			pnConvertFile.setBackground(Color.white);
			pnConvertStatus = new JPanel();
			pnConvertStatus.setBackground(Color.white);
			
			JPanel pnConvertHeading = new JPanel();
			pnConvertHeading.setBackground(Color.white);
			pnConvertHeading.setLayout(new GridLayout(2,1));
			
			
			pnConvertFile.setBackground(Color.white);
			pnConvertStatus.setBackground(Color.white);
				
			pnConvertFile.setLayout(new FlowLayout());
			pnConvertStatus.setLayout(new FlowLayout());
			
			JLabel lblTopEmpty = new JLabel("  ");
						
			pnFileSelectHdr.removeAll();
			
			pnFileSelectHdr.setLayout(new BorderLayout());
			pnFileSelectHdr.setBackground(clrHeaderBg);
				
			lblConversionStatus.setForeground(Color.white);
			lblConversionStatus.setFont(fnHdrFont);
			pnFileSelectHdr.add(lblConversionStatus,BorderLayout.CENTER);
						
			pnConvertHeading.setBackground(Color.white);
			pnConvertHeading.add(pnFileSelectHdr);
			pnConvertHeading.add(lblTopEmpty);
			
			pnOptionalSelection.add(pnConvertHeading);
			
			
			pnConvertFile.add(lblFileProcessed);
			for(int intSpaceIndex = 0; intSpaceIndex < 2; intSpaceIndex++){
				pnConvertFile.add(new JLabel("   "));
			}
			pnConvertFile.add(txtFileProcessed);
			txtFileProcessed.setText(LAS_FILE_SELECTED);
			txtFileProcessed.setEnabled(false);
			
			pnOptionalSelection.add(pnConvertFile);
			
			pnConvertStatus.setSize(260,10);
			pnConvertStatus.add(lblFileStatus);
			for(int intSpaceIndex = 0; intSpaceIndex < 2; intSpaceIndex++){
				pnConvertStatus.add(new JLabel("   "));
			}
		
			pnConvertStatus.add(progressBar);
			for(int intSpaceIndex=0; intSpaceIndex < 6; intSpaceIndex++){
				pnConvertStatus.add(new JLabel("   "));
			}
			
			pnOptionalSelection.add(pnConvertStatus);
			
			
			JPanel pnConvertMsg = new JPanel();
			pnConvertMsg.setBackground(Color.white);
			pnConvertMsg.setLayout(new FlowLayout());
			
			//JLabel lblBottomEmpty = new JLabel(" ");
			JLabel lblProcessMsg = new JLabel("Click on Process or press Enter key to begin the conversion process");
			lblProcessMsg.setForeground(Color.blue);
			//pnOptionalSelection.add(lblBottomEmpty);
			pnConvertMsg.add(lblProcessMsg);
			
			pnOptionalSelection.add(pnConvertMsg);
			
			
			pnOptionalEmpty.add(new JLabel("         "),BorderLayout.NORTH);
			pnOptionalEmpty.add(new JLabel("             "),BorderLayout.EAST);
			pnOptionalEmpty.add(new JLabel("            "),BorderLayout.WEST);
			
			pnOptionalEmpty.add(pnOptionalSelection,BorderLayout.CENTER);
			//pnOptionalEmpty.add(pnOptionalSelection,BorderLayout.WEST);
			
			pnMainPanel.add(pnOptionalEmpty);
			pnMainPanel.add(new JLabel(" "));	
			
			pnButtonPanel.removeAll();
			//btnConvertExit.setMnemonic('X');
			btnProcessHelp.setMnemonic('H');
			btnProcessConversion.setMnemonic('P');
			
			//btnConvertExit.addActionListener(this);
			btnConvertHelp.addActionListener(this);
			
			//add buttons to buttons panel
			pnButtonPanel.add(btnConverCustomizeMapping);
			pnButtonPanel.add(btnConversionBack);
			pnButtonPanel.add(btnProcessConversion);
			for (int i = 0; i < 40; i++) {
					pnButtonPanel.add(new JLabel(" "));
				}
			pnButtonPanel.add(btnProcessHelp);
			pnButtonPanel.add(btnExit);
			
			pnMainPanel.add(pnButtonPanel);
			
			c.add(pnMainPanel,BorderLayout.CENTER);
			
			validate();
			repaint();
		}catch(Exception eException)
		{
			bnErrorFlag = true;
			throw new ConverterException("Exception in displaying convert LAS to WITSML screen");
		}
	}
	
	/**
	* Method fnDisplayFileSection
	*This method displays the file selection screen listing the LAS files from the selected Input folder
	* @return void
	*
	*/   
	public void fnDisplayFileSection() throws ConverterException{
		try{
			//get the container
			c = getContentPane();
			c.setFocusCycleRoot(true);
			//set layout for container
			c.setLayout(new BorderLayout());
			fnHdrFont = new Font("Arial",Font.BOLD,13);
			fnMenuFont = new Font("Arial",Font.BOLD,13);
			Font fnFont = new Font("Arial",Font.PLAIN,12);
			clrHeaderBg = new Color(109,102,154);
			//set cursor for menu links
			btnFileSelection.setCursor(new Cursor(Cursor.HAND_CURSOR));
			btnMandData.setCursor(new Cursor(Cursor.HAND_CURSOR));
			
			btnConvert.setCursor(new Cursor(Cursor.HAND_CURSOR));
			btnConvertSummary.setCursor(new Cursor(Cursor.HAND_CURSOR));
			
			//set string painted for progress bar
			progressBar.setStringPainted(false);
			
			//set borders for buttons
			btnFileSelection.setBorder(BorderFactory.createLineBorder(Color.black));
			btnMandData.setBorder(BorderFactory.createLineBorder(Color.black));
			
			btnConvert.setBorder(BorderFactory.createLineBorder(Color.black));
			btnConvertSummary.setBorder(BorderFactory.createLineBorder(Color.black));
			//set layout for curve selection panel
			pnCurveSelection.setLayout(new BorderLayout());
			pnCurveSelection.setBackground(clrHeaderBg);
			pnNavMandData.remove(pnMandData);
			validate();
			pnNavFileSelection.setLayout(new GridLayout(4,1));
			pnNavFileSelection.setBackground(new Color(255,245,219));
			//set background color for buttons
			btnFileSelection.setBackground(new Color(239,247,251));
			btnMandData.setBackground(new Color(239,247,251));
			btnConvert.setBackground(new Color(239,247,251));
			btnConvertSummary.setBackground(new Color(239,247,251));
			
							
			//add buttons to panel	
			pnNavFileSelection.add(btnFileSelection);
			pnNavFileSelection.add(btnMandData);
			
			pnNavFileSelection.add(btnConvert);
			pnNavFileSelection.add(btnConvertSummary);
			
			//add FileNavigator panel to container
			c.add(pnNavFileSelection,BorderLayout.WEST); 
			
			pnFileSelection.setLayout(new BorderLayout());
			//remove and add components to file selection panel
			pnFileSelectHdr.removeAll();
			pnFileSelectHdr.setLayout(new BorderLayout());
			pnFileSelectHdr.setBackground(clrHeaderBg);
			
			lblFileSelHeader.setForeground(Color.white);
			lblFileSelHeader.setFont(fnHdrFont);
			pnFileSelectHdr.add(lblFileSelHeader,BorderLayout.NORTH);
			
			pnFileSelection.setBorder(BorderFactory.createLineBorder(Color.black));
			pnFileSelection.add(pnFileSelectHdr,BorderLayout.NORTH);
				
			//set layout for file details panel
			pnFileSelectDtls.setLayout(new BorderLayout());
				
			//add input file details to panel
			pnInputFileSelectDtls.setLayout(new FlowLayout());
			
			//removing the empty spaces in the FileSelection Screen.
			pnInputFileSelectDtls.remove(lblEmpty);
			pnInputFileSelectDtls.remove(lblEmptyValue);
			
			//add input file contols to panel
			pnInputFileSelectDtls.add(lblInputFile);
			pnInputFileSelectDtls.add(lblEmptyValue);
			pnInputFileSelectDtls.add(txtInputFile);
			pnInputFileSelectDtls.add(btnInputFile);
			
			//add empty rows
			for(int intSpaceIndex = 0; intSpaceIndex < 10; intSpaceIndex++)
			{
				pnInputFileSelectDtls.add(lblEmpty);
			}
			pnFileSelectDtls.add(pnInputFileSelectDtls,BorderLayout.NORTH);
			//pnFileSelectDtls.add(pnInputFileSelectDtls,BorderLayout.CENTER);
			
			//add destination file details to panel		
			pnDestFileSelectDtls.setLayout(new FlowLayout());
			pnDestFileSelectDtls.add(lblDestFile);
			pnDestFileSelectDtls.add(txtDestFile);
			pnDestFileSelectDtls.add(btnDestFile);
			//add empty rows
			for(int intSpaceIndex = 0; intSpaceIndex < 10; intSpaceIndex++)
			{
				pnDestFileSelectDtls.add(lblEmpty);
			}
			pnFileSelectDtls.add(pnDestFileSelectDtls,BorderLayout.CENTER);
						
			//add file selection details to file selection panel
			pnFileSelection.add(pnFileSelectDtls,BorderLayout.WEST);
			
			//set layout for main panel
			pnMainPanel.setLayout(new GridLayout(4,1));
						
			//add file selection panel to main panel
			pnMainPanel.add(pnFileSelection);	
			
			
			pnCurveSelection.remove(lblCurveSelectAll);
			pnCurveSelection.removeAll();
			
			if (lblParameterSelection == null) {
				lblParameterSelection = new JLabel("  ",SwingConstants.LEFT);
			}
			pnCurrentFileHdr.remove(pnCurveSelection);
			pnCurrentFileHdr.remove(lblCurrentFile);
			pnCurrentFileHdr.remove(lblParameterSelection);
			pnCurrentFile.remove(pnCurrentFileHdr);
			validate();
			
			pnCurrentFile.setLayout(new BorderLayout());
			pnCurrentFile.setBorder(BorderFactory.createLineBorder(Color.black));
			
			pnCurrentFileHdr.setLayout(new BorderLayout());
			pnCurrentFileHdr.setBackground(clrHeaderBg);
			
			lblCurrentFile.setForeground(Color.white);
			lblCurrentFile.setFont(fnHdrFont);
			
			pnCurrentFileHdr.add(lblCurrentFile,BorderLayout.NORTH);
			//create mandatory data list	on the left menu
			jlMandData = new JList(strMandData);
			jlMandData.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			jListSelectionModel = jlMandData.getSelectionModel();
			jListSelectionModel.addListSelectionListener(new SharedListSelectionHandler());

			//jlMandData.setForeground(new Color(255,177,43));
			jlMandData.setForeground(new Color(1,74,100));
						
			//create a file details vector 
			vecFileDetails = new Vector();
			
			//create default table model
			dmCurrentData = new DefaultTableModel(strCurrentFileHeader,0);
			//create a table with the above default table model
			jtCurrentTable = new JTable(dmCurrentData){
				//disable cell editing
				public boolean isCellEditable(int rowIndex, int vColIndex) {
					if(vColIndex == 2) //changed 3 to 2 for removing mapping file
						return true;
					else 
						return false;
				}
				public Component prepareRenderer(TableCellRenderer renderer,
						 int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
					if (c instanceof JComponent) {
						JComponent jc = (JComponent)c;
						if (vColIndex == 1)
							jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
					}
					
					if (isCellSelected(rowIndex, 2)) {
						c.setBackground(clActiveButton);
						if (c instanceof JRadioButton) {
							JRadioButton jrb = (JRadioButton)c;
							jrb.setSelected(true);
						}
					} else {
						// If not shaded, match the table's background
						c.setBackground(getBackground());
						if (c instanceof JRadioButton) {
							JRadioButton jrb = (JRadioButton)c;
							jrb.setSelected(false);
						}
				       }
					return c;
				}
					
			};
			
			//disable re-ordering of columns
			jtCurrentTable.getTableHeader().setReorderingAllowed(false);
			//set single row selection
			jtCurrentTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			//enable row selection
			jtCurrentTable.setRowSelectionAllowed(true);
			//set font, fore ground color and back ground color of jtCurrentTable header
			JTableHeader anHeader = jtCurrentTable.getTableHeader();
			anHeader.setForeground(Color.blue);
			anHeader.setFont(fnHdrFont);
			anHeader.setBackground(new Color(255,204,153));
			
			//set column width for SN column
			int intColIndex = 0;
			TableColumn tblCol = jtCurrentTable.getColumnModel().getColumn(intColIndex);
			int intColWidth = 80;
			tblCol.setMinWidth(intColWidth);
			tblCol.setMaxWidth(intColWidth);
			tblCol.setPreferredWidth(intColWidth);
			
			
			//set column width for Select column
			intColIndex = 2; 
			tblCol = jtCurrentTable.getColumnModel().getColumn(intColIndex);
			intColWidth = 80;
			tblCol.setMinWidth(intColWidth);
			tblCol.setMaxWidth(intColWidth);
			tblCol.setPreferredWidth(intColWidth);
			
			//add jtCurrentTable to scroll pane
			scrollPane = new JScrollPane(jtCurrentTable);
			scrollPane.getViewport().setBackground(new Color(246, 231, 215));
			jtCurrentTable.setPreferredScrollableViewportSize(new Dimension(435, 400));
			
			pnCurrentFile.remove(scrollPane);
			pnCurrentFile.removeAll();
			pnMainPanel.remove(pnCurrentFile);
			
			pnComboPanel.remove(cbnMappingFiles);
			pnComboPanel.remove(lblMappingFile);
			pnComboPanel.removeAll();
			
			pnCustomizeMapping.remove(lblMappingEmpty);
			pnCustomizeMapping.remove(pnComboPanel);
			pnCustomizeMapping.removeAll();
			pnMainPanel.remove(pnCustomizeMapping);
			
			pnCurrentFile.add(pnCurrentFileHdr,BorderLayout.NORTH);
			pnCurrentFile.add(scrollPane,BorderLayout.CENTER);
			
			//add scrollpane to main panel		
			pnMainPanel.add(pnCurrentFile);
			
			cbnMappingFiles  =new JComboBox();
			
			File fCurrentFolder = new File(".");
			String strPath = new File(fCurrentFolder.getCanonicalPath()).getParent();
			String strMappingFile = strPath+"/mappings";
			
			File fMappingFile = new File(strMappingFile);
			String strList[] = fMappingFile.list();
					
			//add the mapping xml files to combo box
			for(int intFileIndex = 0; intFileIndex < strList.length; intFileIndex++) {
				File fTmp = new File(strList[intFileIndex]);
				String strFileName = strPath+"/"+fTmp.getName();
				if (strFileName != null)  {
					if((strFileName.endsWith("xml")) || (strFileName.endsWith("XML"))) {
						cbnMappingFiles.addItem(strMappingFile+"/"+strList[intFileIndex]);
					}
				}
			}
			cbnMappingFiles.setSelectedItem(strMappingFile+"/Default.xml");
				
			pnComboPanel = new JPanel();
			pnComboPanel.setLayout(new FlowLayout());
			pnComboPanel.add(lblMappingFile);
			pnComboPanel.add(cbnMappingFiles);
			
			
			pnComboPanel.setBackground(Color.white);
			pnCustomizeMapping.setLayout(new FlowLayout());
			pnCustomizeMapping.add(pnComboPanel);
			pnCustomizeMapping.setBackground(Color.white);
							
			for(int intSpaceIndex = 0; intSpaceIndex < 80; intSpaceIndex++){
				pnCustomizeMapping.add(new JLabel(" "));
			}
			
			//add pnCustomizeMapping
			pnMainPanel.add(pnCustomizeMapping);
				
			//set short cut keys for buttons
			btnNext.setMnemonic('N');
			btnExit.setMnemonic('X');
			btnHelp.setMnemonic('H');
			
			btnExit.addActionListener(this);
			
			pnButtonPanel.removeAll();
			pnMainPanel.remove(pnButtonPanel);
			
			//pnButtonPanel.setBorder(BorderFactory.createLineBorder(Color.black));
			btnCustomizeMapping.setMnemonic('C');
			
			//add buttons to buttons panel
			
			pnButtonPanel.add(btnCustomizeMapping);
			pnButtonPanel.add(btnNext);
			for(int i = 0; i < 50; i++){
				pnButtonPanel.add(new JLabel(" "));
			}
			pnButtonPanel.add(btnHelp);
			pnButtonPanel.add(btnExit);
			
						
			//add buttons to main panel
			pnMainPanel.add(pnButtonPanel); 
				
			//add main panel to content pane
			c.add(pnMainPanel,BorderLayout.CENTER);
			
			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		}catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println(" Exception in fnDisplayFileSection "+eException);
			pwPrintWriter.println(" Exception in fnDisplayFileSection "+eException);
			throw new ConverterException("Converter Exception in fnDisplayFileSection");
		}
	}

	/**
	* Method displaySectionData
	*This method displays the mandatory section data
	* @param The section for which the data is to be displayed, can be Parameter or Curve
	* @return void
	*
	*/
	public void displaySectionData(String strSection) throws ConverterException{ 
		try{
			ArrayList alMandatoryData = null;
			pwPrintWriter.println(">>>>> displaySectionData strSection >>>>> "+strSection);
			pwPrintWriter.println("LAS_FILE_SELECTED in displaySectionData"+LAS_FILE_SELECTED);
			pwPrintWriter.println("PREV_SELECTED_LAS_FILE in displaySectionData"+PREV_SELECTED_LAS_FILE);
			
			//get 
			if (strSection.equalsIgnoreCase(strMandParameter)) {
				
				if ((LAS_FILE_SELECTED != null) && (PREV_SELECTED_LAS_FILE != null)) {
					if ((PREV_SELECTED_LAS_FILE.equals(LAS_FILE_SELECTED))) {
						pwPrintWriter.println("files are same..");
						//alMandatoryData = objLasDataFile.getLasData().getParameterSectionData();
						if ((alParameterData != null) && (alParameterData.size() > 0)) {
							//System.out.println("parameters size is "+alParameterData.size());
							alMandatoryData = alParameterData;
							
						} else {
							alMandatoryData = objLasDataFile.getLasData().getParameterSectionData();
							
						}
					} else {
						pwPrintWriter.println("files are not same..");
						if ((alParameterData != null) && (alParameterData.size() > 0)) {
								//System.out.println("curve size is "+alParameterData.size());
								alMandatoryData = alParameterData;
							
						} else {
							objLasDataFile.readParametersSectionData(objLasData.getInputLasFile(),objLasData);
							alMandatoryData = objLasDataFile.getLasData().getParameterSectionData();
							alParameterData = alMandatoryData;
							
						}
					}
				}
				//alMandatoryData = objLasDataFile.getLasData().getParameterSectionData();
							
				if ((alMandatoryData != null) && (alMandatoryData.size() == 0)) {
					pwPrintWriter.println("alMandatoryData != null");
					objLasDataFile.readParametersSectionData(objLasData.getInputLasFile(),objLasData);
					alMandatoryData = objLasDataFile.getLasData().getParameterSectionData();
					alParameterData = alMandatoryData;
				}
				
							
				//System.out.println("after reading parameters size is "+alMandatoryData.size());
			} else if (strSection.equalsIgnoreCase(strMandCurveData)) {
				if ((LAS_FILE_SELECTED != null) && (PREV_SELECTED_LAS_FILE != null)) {
						if ((PREV_SELECTED_LAS_FILE.equals(LAS_FILE_SELECTED))) {
							pwPrintWriter.println("files are same..");
							if ((alCurveData != null) && (alCurveData.size() > 0)) {
								//System.out.println("curve size is "+alCurveData.size());
								alMandatoryData = alCurveData;
							} else {
								alMandatoryData = objLasDataFile.getLasData().getLogDefns();
							}
						} else {
							pwPrintWriter.println("files are not same..");
							
							if ((alCurveData != null) && (alCurveData.size() > 0)) {
								//System.out.println("curve size is "+alCurveData.size());
								alMandatoryData = alCurveData;
							} else {
								objLasDataFile.readLogDefinitionsData(objLasData.getInputLasFile(),null,"~C",objLasData);
								alMandatoryData = objLasDataFile.getLasData().getLogDefns();
								alCurveData = alMandatoryData;
							}
							//PREV_SELECTED_LAS_FILE = LAS_FILE_SELECTED;
						}
				}
				
												
				if ((alMandatoryData != null) && (alMandatoryData.size() == 0)) {
					pwPrintWriter.println("alMandatoryData != null");
					objLasDataFile.readLogDefinitionsData(objLasData.getInputLasFile(),null,"~C",objLasData);
					alMandatoryData = objLasDataFile.getLasData().getLogDefns();
					alCurveData = alMandatoryData;
				}
			}
				
			//System.out.println("alMandatoryData .."+alMandatoryData);
			
			if(alMandatoryData  != null) {
				//add the data to table
				for(int intSectionIndex = 0; intSectionIndex < alMandatoryData.size(); intSectionIndex++) {
					
					LasSectionInfo objLasSectionInfo = (LasSectionInfo) alMandatoryData.get(intSectionIndex); 
					boolean blnSelected = true;
					blnSelected = (objLasSectionInfo.getIsSelected()).booleanValue();
					dmMandatory.addRow(objLasSectionInfo.createRow(strSection,intSectionIndex+1, blnSelected));
				}
			}
			
			if (strSection.equalsIgnoreCase(strMandCurveData)) {
				if(alMandatoryData  != null) {
					int intNumRowsSelected = 0;
					for(int intSectionIndex = 0; intSectionIndex < jtMandatory.getRowCount(); intSectionIndex++) {
						Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSectionIndex,0);
						//check if check box is checked
						if (blnSelected.booleanValue()==true) {
							intNumRowsSelected++;
						}
					}
					//if row count is equal to number of rows selected, check the Select All check box
					if (intNumRowsSelected == jtMandatory.getRowCount()) {
						chCurveSelection.setSelected(true);
					}else{
						chCurveSelection.setSelected(false);
					}
				}	
			}
			
		}catch(Exception eException)
		{
			bnErrorFlag = true;
			throw new ConverterException("Exception in displaying mandatory data");
		}	
	}
	/**
	* Method ParameterDetails
	*This method prepares the mandatory data screen UI and populates the grid with the LAS file details
	* @return void
	*
	*/
	public void ParameterDetails() throws ConverterException {
		try {
		
			pnMainPanel.remove(pnFileSelection);
			pnCurrentFile.removeAll();
			pnMainPanel.removeAll();
			
			pnMandData.removeAll();		
			c.remove(pnNavFileSelection);
			c.remove(pnMainPanel);
					
			fnMandList();
			
			pnCurrentFileHdr.remove(lblCurrentFile);
			pnMainPanel.setLayout(new GridLayout(3,1));
			pwPrintWriter.println(" >>>>>>> parameterDetails() MAND_DATA_SELECTED   "+MAND_DATA_SELECTED);
			pwPrintWriter.println("LAS_FILE_SELECTED in parameterDetails"+LAS_FILE_SELECTED);
			pwPrintWriter.println("PREV_SELECTED_LAS_FILE in parameterDetails"+PREV_SELECTED_LAS_FILE);
			//set the table headers
			if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
				if (lblParameterSelection != null) {
					lblParameterSelection.setText(" ");
				}
				pnCurveSelection.removeAll();
				lblParameterSelection = new JLabel(" Parameter Data ",SwingConstants.LEFT);
				dmMandatory = new DefaultTableModel(strParameterHeader,0);
			} else if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
					if (lblParameterSelection != null) {
						lblParameterSelection.setText(" ");
					}
					lblParameterSelection = new JLabel(" Curve Data Selection ",SwingConstants.LEFT);
					lblCurveSelectAll.setForeground(Color.white);
					lblCurveSelectAll.setFont(fnHdrFont);
					
					dmMandatory = new DefaultTableModel(strCurveHeader,0);
			}
			lblParameterSelection.setForeground(Color.white);
			lblParameterSelection.setFont(fnHdrFont);
			
			pnCurrentFile.setBackground(clrHeaderBg);
					
			//pnCurrentFileHdr.add(lblParameterSelection,BorderLayout.WEST);
			pnCurrentFileHdr.add(lblParameterSelection,BorderLayout.NORTH);
			
					
			if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
				pnCurveSelection.removeAll();	
							
				if (jtMandatory != null) {
				int intNumRowsSelected = 0;
					for(int intSectionIndex = 0; intSectionIndex < jtMandatory.getRowCount(); intSectionIndex++) {
						Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSectionIndex,0);
						if (blnSelected.booleanValue()==true) {
							intNumRowsSelected++;
						}
					}
				
				//check row count and number of rows selected and display Select ALL checkbox as selected
					if (jtMandatory.getRowCount() == intNumRowsSelected) {
						chParameterSelection.setSelected(true);
					} else {
						chParameterSelection.setSelected(false);
					}
				}
				//add select all check box to panel	
				pnCurveSelection.add(chParameterSelection);
				//pnCurrentFile.add(pnCurveSelection,BorderLayout.SOUTH);			
				pnCurrentFileHdr.add(pnCurveSelection,BorderLayout.SOUTH);			
			}
			
			
			if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
				pnCurveSelection.removeAll();	
				
				if (jtMandatory != null) {
					int intNumRowsSelected = 0;
					for(int intSectionIndex = 0; intSectionIndex < jtMandatory.getRowCount(); intSectionIndex++) {
						Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSectionIndex,0);
						if (blnSelected.booleanValue()==true) {
							intNumRowsSelected++;
						}
					}
					//check row count and number of rows selected and display Select ALL checkbox as selected
					if (jtMandatory.getRowCount() == intNumRowsSelected) {
							chCurveSelection.setSelected(true);
					} else {
						chCurveSelection.setSelected(false);
					}
				}
				//add select all check box to panel	
				pnCurveSelection.add(chCurveSelection);
				//pnCurrentFile.add(pnCurveSelection,BorderLayout.SOUTH);
				pnCurrentFileHdr.add(pnCurveSelection,BorderLayout.SOUTH);		
			}
			
			
			//pnCurrentFileHdr.add(pnCheckPanel,BorderLayout.SOUTH);
			
			pnCurrentFile.add(pnCurrentFileHdr,BorderLayout.NORTH);
			
			//pnCurveSelection.add(chParameterSelection);
			
			jtMandatory = new JTable(dmMandatory){
			//disable cell editing
				public boolean isCellEditable(int rowIndex, int vColIndex) {
					if(vColIndex == 0) {
						if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
							boolean blnIsSelected = true;
							if (jtMandatory != null) {
								String strMnemonic = (String) jtMandatory.getValueAt(rowIndex,1);
								if ((strMnemonic.equalsIgnoreCase("DEPT")) || (strMnemonic.equalsIgnoreCase("TIME"))) {
									blnIsSelected = false;
								}
							} else 
								blnIsSelected = true;
							return blnIsSelected;
						}else {
							boolean blnIsSelected = true;
							if (jtMandatory != null) {
								if (objLasDataFile != null) {
										if (objLasDataFile.getWellParamSize() > 0) {
											if (rowIndex < objLasDataFile.getWellParamSize()) {
												blnIsSelected = false;
											}
										}
									}
								//} //commented code to select all well info by default starts here
							} else 
								blnIsSelected = true;
							return blnIsSelected;
						} 
						/*else {
							return false;
						}*/
					} else 
						return false;
				
				}
				public Component prepareRenderer(TableCellRenderer renderer,
						 int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
					if (c instanceof JComponent) {
						JComponent jc = (JComponent)c;
						//if (vColIndex == 1) {
							if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
								String strMnemonic = (String) jtMandatory.getValueAt(rowIndex,1);
								
								if ((strMnemonic.equalsIgnoreCase("DEPT")) || (strMnemonic.equalsIgnoreCase("TIME"))) {
									
									Object objValue;
									boolean bFlag = false;
									objValue= jtMandatory.getValueAt(rowIndex, 0);
									bFlag = Boolean.valueOf(String.valueOf(objValue).trim()).booleanValue();
									if (bFlag) {
										c.setBackground(Color.lightGray);
									}
								} else {
									c.setBackground(jtMandatory.getBackground());
								}
							}
							if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
								if (objLasDataFile != null){
									if (objLasDataFile.getWellParamSize() != -1) {
										if (rowIndex < objLasDataFile.getWellParamSize()) {
											c.setBackground(Color.lightGray);
										} else {
											c.setBackground(jtMandatory.getBackground());
										}
									}
								}
							}
						//}
						
						//ADDED BELOW LINES FOR DISPLAYING CHECKBOXES FOR PARAMETER DATA 9/2/2006 STARTS HERE
						if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
							if ((vColIndex == 1) || (vColIndex == 2))
							jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
						} else if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
							if ((vColIndex == 1) || (vColIndex == 2))
							jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
						}
						//ADDED BELOW LINES FOR DISPLAYING CHECKBOXES FOR PARAMETER DATA 9/2/2006 ENDS HERE
				       }
					return c;
				}
			};
			TableColumn chkCol = jtMandatory.getColumnModel().getColumn(0);//<-----------------
			chkCol.setCellEditor(jtMandatory.getDefaultEditor(Boolean.class));//<-----------------
			chkCol.setCellRenderer(jtMandatory.getDefaultRenderer(Boolean.class));//<-----------------
			
			int intColWidth = 80;
			//set column width
			chkCol.setMinWidth(intColWidth);
			chkCol.setMaxWidth(intColWidth);
			chkCol.setPreferredWidth(intColWidth);
	
			//removed condition as we need to add for both parameter and curve data section
			//if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) { //added 19.1.2006 
					dmMandatory.addTableModelListener(this);
			//}
			
			//disable re-ordering of columns
			jtMandatory.getTableHeader().setReorderingAllowed(false);
			//set single row selection
			jtMandatory.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			//enable row selection 
			jtMandatory.setRowSelectionAllowed(true);
			
			//set font, fore ground color and back ground color of jtCurrentTable header
			JTableHeader anHeader = jtMandatory.getTableHeader();
			anHeader.setForeground(Color.blue);
			anHeader.setFont(fnHdrFont);
			anHeader.setBackground(new Color(255,204,153));
	
			if (MAND_DATA_SELECTED != null) {
				displaySectionData(MAND_DATA_SELECTED);
			}
			
			scrollPane = new JScrollPane(jtMandatory);
			scrollPane.getViewport().setBackground(new Color(246, 231, 215));
			jtMandatory.setPreferredScrollableViewportSize(new Dimension(400, 200));
			pnCurrentFile.add(scrollPane,BorderLayout.CENTER);
						
			pnMainPanel.add(pnCurrentFile);
			pnButtonPanel.removeAll();
			//set shortcut keys for buttons
			btnExit.setMnemonic('X');
			btnParameterHelp.setMnemonic('H');
			btnCurveHelp.setMnemonic('H');
			btnExit.addActionListener(this);
			
			//add buttons to buttons panel
			if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
				//set shortcut keys for buttons
				btnParamPrev.setMnemonic('F');
				btnParamNext.setMnemonic('N');
				pnButtonPanel.add(btnParamPrev); 
				pnButtonPanel.add(btnParamNext);
				for (int i = 0; i < 55; i++) {
					pnButtonPanel.add(new JLabel(" "));
				}
				pnButtonPanel.add(btnParameterHelp);
			} else  if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
				//set shortcut keys for buttons
				btnCurvePrev.setMnemonic('B');
				btnCurveNext.setMnemonic('N');
				btnLoadSelection.setMnemonic('L');
				btnSaveSelection.setMnemonic('S');
				//add buttons to panel
				pnButtonPanel.add(btnLoadSelection);
				pnButtonPanel.add(btnSaveSelection);
				pnButtonPanel.add(btnCurvePrev); 
				pnButtonPanel.add(btnCurveNext);
				for (int i = 0; i < 30; i++) {
					pnButtonPanel.add(new JLabel(" "));
				}
				pnButtonPanel.add(btnCurveHelp);
			}
			pnButtonPanel.add(btnExit);
			pnMainPanel.add(new JLabel(""));
			pnMainPanel.add(pnButtonPanel);
			c.add(pnMainPanel,BorderLayout.CENTER);
			validate();
			repaint();
		} catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println(" Converter Exception "+eConverterException);
				pwPrintWriter.println(" Converter Exception "+eConverterException);
		} catch (Exception eException) {
				bnErrorFlag = true;
				System.out.println(" Exception in Parameter Details data "+eException);
				throw new ConverterException("Converter Exception");
		}
	}
	
	/**
	* Method tableChanged
	*This method respondes to the table change events
	* @param TableModelEvent, that has caused that table value change
	* @return void
	*
	*/
	public void tableChanged(TableModelEvent e) { 
		if (e.getType() == TableModelEvent.UPDATE) {
			//System.out.println(" Table updated ");
			int intNumRowsSelected = 0;
			for(int intSectionIndex = 0; intSectionIndex < jtMandatory.getRowCount(); intSectionIndex++) {
				Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSectionIndex,0);
				//check if check box is checked
				if (blnSelected.booleanValue()==true) {
					intNumRowsSelected++;
				}
			}
			//if row count is equal to number of rows selected, check the Select All check box
			if (intNumRowsSelected == jtMandatory.getRowCount()) {
				chCurveSelection.setSelected(true);
				chParameterSelection.setSelected(true);
			}else{
				chCurveSelection.setSelected(false);
				chParameterSelection.setSelected(false);
			}
		} 
	}
	
	
	/**
	* Method keyPressed
	*Invoked when a key has been released
	* @param KeyEvent
	*
	*/
	
	public void keyPressed(KeyEvent ke)
	{
		//System.out.println("key pressed");
		if (ke.getKeyCode() == KeyEvent.VK_ENTER){
			//System.out.println("Enter key pressed ");
			btnProcessConversion.doClick();
		}
		
	}
	
	/**
	* Method keyTyped
	*Invoked when a key has been typed
	* @param KeyEvent
	*
	*/
	
	public void keyTyped(KeyEvent ke1)
	{
		//System.out.println("key typed");
	}
	
	/**
	* Method keyReleased
	*Invoked when a key has been pressed
	* @param KeyEvent
	*
	*/
	public void keyReleased(KeyEvent ke2)
	{
		//System.out.println(" Key released ");
	}
	
	/**
	* Method fnMandList
	*This method checks whether Parameter or Curve data is selected in the left menu and displays the appropriate screen
	* @return void
	*
	*/
	public void fnMandList() throws ConverterException {
	  try{
			MouseListener mouseListener = new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					try{
						 if (e.getClickCount() == 1) {
							int index = jlMandData.locationToIndex(e.getPoint());
							if(index == 0)  {
								//System.out.println("Double clicked on Item " + index);
								pwPrintWriter.println("Double clicked on Item " + index);
								MAND_DATA_SELECTED=strMandParameter;
							} else if(index == 1) {
							    // System.out.println("Double clicked on Item " + index);
							     pwPrintWriter.println("Double clicked on Item " + index);
							     MAND_DATA_SELECTED=strMandCurveData;
							}
							ParameterDetails();
						}
					}catch (Exception eException) {
								bnErrorFlag = true;
								System.out.println(" Exception in fnMandList  "+eException);
								pwPrintWriter.println(" Exception in fnMandList  "+eException);
					} 	
				}
			};
			//add mouse listener to mandatory data menu
			jlMandData.addMouseListener(mouseListener);
	
			//jlMandData.setBackground(btnMandData.getBackground());
			jlMandData.setBackground(new Color(239,247,251));
			jlMandData.setBorder(BorderFactory.createLineBorder(Color.black));
			
			pnMandData.setLayout(new GridLayout(2,1));
			pnMandData.add(btnMandData);
			pnMandData.add(jlMandData);
			
			//pnNavMandData.setLayout(new GridLayout(5,1)); //for optional data
			pnNavMandData.setLayout(new GridLayout(4,1));
			pnNavMandData.setBackground(new Color(255,245,219));
			
			pnNavMandData.add(btnFileSelection);
			pnNavMandData.add(pnMandData);
			
			pnNavMandData.add(btnConvert);
			pnNavMandData.add(btnConvertSummary);
			
			c.add(pnNavMandData,BorderLayout.WEST);
		}catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println(" Exception  "+eException);
			pwPrintWriter.println(" Exception  "+eException);
			throw new ConverterException("Converter Exception");
		} 
	}
	
	/**
	* Method setMandatoryData
	*This method gets the data selections on the parameter/curve data selection screen and sets the selections in the definitons of the LAS file
	* @return void
	*
	*/
	public void setMandatoryData() throws ConverterException{
		try{
			//check for  parameter selection
			pwPrintWriter.println("calling set mandatory data MAND_DATA_SELECTED"+MAND_DATA_SELECTED);
			Hashtable htSectionData = null;	
			Hashtable htWellData = null;	
			Hashtable htRunData = null;	
			if (jtMandatory != null) {
			
				//get parameter and curve data depending on the item selected on the mandatory data menu	
				if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
					//htSectionData = objLasDataFile.getLasData().getParameterData().getParameterSectionData();
					htWellData = objLasDataFile.getLasData().getParameterData().getWellSectionData();
					htRunData = objLasDataFile.getLasData().getParameterData().getRunParameterSectionData();
					//System.out.println("setMandatoryData() for parameters htWellData size ..."+htWellData.size());
					//System.out.println("setMandatoryData() for parameters htRunData size ..."+htRunData.size());
				} else  if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
					//objLasDataFile.readLogDefinitionsData(objLasData.getInputLasFile(),null,"~C",objLasData);
					htSectionData = objLasDataFile.getLasData().getParameterData().getCurveSectionData();
					//System.out.println("setMandatoryData() for CURVE htSectionData size ..."+htSectionData.size());
				}
				
				//set the selections from table to parameter and curve data of the object
				for(int intSectionIndex = 0; intSectionIndex < jtMandatory.getRowCount(); intSectionIndex++) {
					Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSectionIndex,0);
					//System.out.println("blnSelected"+blnSelected);
					//get the mnemonic in column 1			
					String strLasName = (String)jtMandatory.getValueAt(intSectionIndex,1);
					
					if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
						if (intSectionIndex < htWellData.size()) {
							htSectionData = htWellData;
						} else {
							htSectionData = htRunData;
						}
					}					
					//if the mnemonic is present in the hastable, get the LasSectionInfo object 
					if (htSectionData.containsKey(strLasName)) {
						
						LasSectionInfo objLasSectionInfo = (LasSectionInfo) htSectionData.get(strLasName);
						if (objLasSectionInfo != null) {
							//set the curve as selected depending on the item selected on the mandatory data menu	
							objLasSectionInfo.setIsSelected(blnSelected);
							if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
								//objLasDataFile.getLasData().getParameterData().setParameterSectionData(objLasSectionInfo);
								htWellData = objLasDataFile.getLasData().getParameterData().getWellSectionData();
								htRunData = objLasDataFile.getLasData().getParameterData().getRunParameterSectionData();
							
							}else  if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
								objLasDataFile.getLasData().getParameterData().setCurveSectionData(objLasSectionInfo);
							}
						}	
					}
				}
				ArrayList alSectionData = null;
				//update the curve and parameter definitions with the above selections
				if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
					htSectionData = objLasDataFile.getLasData().getParameterData().getCurveSectionData();
					alSectionData = objLasDataFile.getLasData().getLogDefns();
				} else if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
					//htSectionData = objLasDataFile.getLasData().getParameterData().getParameterSectionData();
					htWellData = objLasDataFile.getLasData().getParameterData().getWellSectionData();
					htRunData = objLasDataFile.getLasData().getParameterData().getRunParameterSectionData();
					
					alSectionData = objLasDataFile.getLasData().getParameterSectionData();
				}
				//get the curve and parameter defintions
				if (htSectionData != null) {
					if (alSectionData != null) {
						pwPrintWriter.println(">>>> alSectionData size ..."+alSectionData.size());
						for (int intSectionIndex = 0; intSectionIndex < alSectionData.size(); intSectionIndex++) {
							LasSectionInfo objLasSectionInfo = (LasSectionInfo) alSectionData.get(intSectionIndex);
							
							if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandParameter)) {
								if (intSectionIndex < htWellData.size()) {
									htSectionData = htWellData;
								} else {
									htSectionData = htRunData;
								}
							}	
							
							if (htSectionData.containsKey(objLasSectionInfo.getMnemonic())) {
								//set the curve selection in the curve defns list
								LasSectionInfo objNewLasSectionInfo = (LasSectionInfo) htSectionData.get(objLasSectionInfo.getMnemonic());
								//System.out.println(":::::::::: is curve selected "+objNewLasSectionInfo.getIsSelected());
								alSectionData.set(intSectionIndex, objNewLasSectionInfo);
							}
						}
					}
				}
				
				if (MAND_DATA_SELECTED.equalsIgnoreCase(strMandCurveData)) {
					alSectionData = objLasDataFile.getLasData().getLogDefns();
					for (int k=0; k<alSectionData.size(); k++) {
						LasSectionInfo objLasSectionInfo = (LasSectionInfo) alSectionData.get(k);
						//System.out.println(":::::::::: after updation .... is curve selected "+objLasSectionInfo.getMnemonic()+"..."+objLasSectionInfo.getIsSelected());
					}
				}
			} else {
				throw new ConverterException("Converter Exception in setMandatoryData, No mandatory section data");
			}
			
		}catch(Exception eException) {
			bnErrorFlag = true;
			System.out.println("Converter Exception " + eException);
			pwPrintWriter.println("Converter Exception " + eException);
			throw new ConverterException("Converter Exception in setMandatoryData"+eException);
		}
	}
	/**
	* Method fileSelectionTableDetails
	*This method gets the input folder and displays the LAS files
	* @param The source where the folder input is entered
	* @return void
	*
	*/
	
	public void fileSelectionTableDetails(String strSource)  {
		try {
			//System.out.println(" INPUT_FILE_DIRECTORY setting text"+INPUT_FILE_DIRECTORY);
			pwPrintWriter.println(" INPUT_FILE_DIRECTORY setting text"+INPUT_FILE_DIRECTORY);
			//GET ALL FILES FFROM SOURCE DIRECTORY
			if (INPUT_FILE_DIRECTORY == null || INPUT_FILE_DIRECTORY.equals("")) {
				//System.out.println(" strSource setting text"+strSource+":::::");
				if ((strSource != null) &&  (!(strSource.equals("")))) {
					JOptionPane.showMessageDialog(null, "Please select input LAS file folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
					txtInputFile.requestFocusInWindow(); 
				} else if (strSource.equals("")) {
					txtInputFile.requestFocusInWindow(); 
				}
				//return;
			} else if (INPUT_FILE_DIRECTORY != null )   { 
		
				File fFiles = new File(INPUT_FILE_DIRECTORY);
				
				if (fFiles.isDirectory()) {	
					String files[] = fFiles.list();
					pwPrintWriter.println("Number of files in input file directory "+files.length);
					int intSno = 0;
										
					if(vecFileDetails.size() >0){
						vecFileDetails.clear();
					}
					//System.out.println(" +++++++ vector size .."+vecFileDetails.size());
					 for (int intFileIndex = 0; intFileIndex < files.length; intFileIndex++) {
					
						if (vecFileDetails != null )
							intSno = vecFileDetails.size()+1;
						else
							intSno = 1;
						
						File fTmp = new File(files[intFileIndex]);
						
						String strInputFileName = INPUT_FILE_DIRECTORY+"/"+fTmp.getName();
						if (strInputFileName != null)  {
							if((strInputFileName.endsWith("las")) || (strInputFileName.endsWith("LAS"))) {
								//set data in FileDetails object
								FileDetails objFileDetails = new FileDetails();
								objFileDetails.setSNo(intSno);
								objFileDetails.setInputFile(strInputFileName);
								objFileDetails.setIsSelected(false);
								
								//add FileDetails object to vector
								vecFileDetails.add(objFileDetails);
							}
						}
					}
					if (vecFileDetails.size() == 0) {
						JOptionPane.showMessageDialog(null, "No LAS files exists in the selected input folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						txtInputFile.setText("");
						txtDestFile.setText("");
						txtInputFile.requestFocusInWindow(); 
						btnInputFile.doClick();
					}
					
					int intRowCnt = jtCurrentTable.getRowCount();
					
					if(intRowCnt > 0) {
						dmCurrentData.setRowCount(0);
						LAS_FILE_SELECTED = "";
						intRowCnt = 0;
					}
					
					int intSelectedRow = -1;
					for (int intFileIndex = intRowCnt; intFileIndex < vecFileDetails.size(); intFileIndex++) {
						jtCurrentTable.getColumn("Select").setCellRenderer(new ButtonRenderer());
						jtCurrentTable.getColumn("Select").setCellEditor(new ButtonEditor(new JCheckBox()));
						FileDetails objFileDetails = (FileDetails) vecFileDetails.elementAt(intFileIndex);
						String strFileSelected = (String) objFileDetails.getInputFile();
						//get the selected row count
						if (strFileSelected.equalsIgnoreCase(LAS_FILE_SELECTED)) {
							intSelectedRow = intFileIndex;
						}
						
						//create a new row with these File Details
						dmCurrentData.addRow(objFileDetails.createRow());
						
						//the below code is for selecting the lasfile when user clicks on back.
						//System.out.println(" selected row is "+intSelectedRow);
						if (jtCurrentTable != null) {
							boolean toggle = false;
							boolean extend = false;
							jtCurrentTable.changeSelection(intSelectedRow, 2, toggle, extend);
							//System.out.println(" value in 3rd column"+jtCurrentTable.getValueAt(intSelectedRow,3));
											
						}
					}
				} else {
					JOptionPane.showMessageDialog(null, "Please select a valid folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
					if (strSource != null) {
						if (strSource.equalsIgnoreCase("Button")) {
							txtInputFile.setText("");
							txtDestFile.setText("");
							txtInputFile.requestFocusInWindow(); 
						} else if (strSource.equalsIgnoreCase("TextField")) {
							txtInputFile.selectAll();
							txtInputFile.requestFocusInWindow(); 
						}
					}
					if (dmCurrentData != null) {
						int intRowCnt = jtCurrentTable.getRowCount();
						if(intRowCnt > 0) {
							dmCurrentData.setRowCount(0);
							LAS_FILE_SELECTED = "";
							intRowCnt = 0;
						}
					}
				}
			}
		} catch (Exception eException) {
			bnErrorFlag = true;
			System.out.println("Exception in fileSelectionTableDetails"+eException);
			pwPrintWriter.println("Exception in fileSelectionTableDetails"+eException);
		}
	}
	
	/**
	* Method checkFolder
	*This method checks the selected input and destination folders
	* @return void
	*
	*/
	public void showAndCheckFolder() {
		try {
			System.out.println(" in showAndCheckFolder INPUT_FILE_DIRECTORY"+INPUT_FILE_DIRECTORY+"::::");
			File fCurrentFolder = null;
			if (DEST_FILE_DIRECTORY != null) {
				System.out.println(" dest not null");
				if (!(DEST_FILE_DIRECTORY.equals(""))) {
					fCurrentFolder = new File(DEST_FILE_DIRECTORY);
				}
			}else  if (INPUT_FILE_DIRECTORY != null) {
				System.out.println(" in not null");
				fCurrentFolder = new File(INPUT_FILE_DIRECTORY);
			} else  if ((INPUT_FILE_DIRECTORY == null) && (DEST_FILE_DIRECTORY == null)) {
				System.out.println(" both are null");
				fCurrentFolder = new File(new File(new File(".").getCanonicalPath()).getParent());
			}
			//if the destination folder is already set after selecting input LAS file folder
			//instantiate JFileChooser for displaying the File Chooser dialog box
			JFileChooser fcFilechooser = new JFileChooser();
			
			// Set the current directory
			fcFilechooser.setCurrentDirectory(fCurrentFolder);
			// Set the directory selection modes
			fcFilechooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			//Show the file chooser dialog box
			int intReturnVal = fcFilechooser.showOpenDialog(null);
			
			File fSelectedFolder = null;
			if (intReturnVal == JFileChooser.CANCEL_OPTION){
				//System.out.println("User clicked on Cancel");
			}else{
			//get the selected directory
				fSelectedFolder = fcFilechooser.getSelectedFile() ;
			}
							
			//get the selected directory
			//File fSelectedFolder = fcFilechooser.getSelectedFile() ;
			//System.out.println("The selected folder is " + fSelectedFolder);
			if(fSelectedFolder != null){
				//if the selected folder is a directory
				if (fSelectedFolder.isDirectory()) {
					//get the selected directory path
					String strDestFolder = fSelectedFolder.getCanonicalPath() ;
					//set the destination folder		
					txtDestFile.setText(strDestFolder);
					DEST_FILE_DIRECTORY=strDestFolder;
					//check if input folder is selected
					if(INPUT_FILE_DIRECTORY != null){
					
						File fInputFolder = new File(INPUT_FILE_DIRECTORY);
						if (!(fInputFolder.isDirectory())) {
							JOptionPane.showMessageDialog(null, "Specified input folder doesn't exist ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
							txtInputFile.requestFocusInWindow();
							return;
						}
						//get the current directory path
						File fCurrentPath=new File(".");
						String strCurrentPath = new File(fCurrentPath.getCanonicalPath()).getParent();
						pwPrintWriter.println("The CurrentPath is >>>>>>>>>>>>>> " + strCurrentPath);						
						//if the destination folder is different from the current directory where the application is running, display a message that style sheets will not be applied
						if (!((strCurrentPath.trim()).equalsIgnoreCase(txtDestFile.getText().trim()))) {
							//JOptionPane.showMessageDialog(null, "Destination folder is the not the same as the current folder, Hence stylesheets will not be applied for the generated files ", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
							JOptionPane.showMessageDialog(null, "Destination folder is the not the same as the Application folder("+strCurrentPath+"), \nHence stylesheets will not be applied for the generated files ", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
							return;
						}
					} else {
						JOptionPane.showMessageDialog(null, "Please select Input LAS File Folder ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
						//setting focus to window
						btnInputFile.requestFocusInWindow(); 
						btnInputFile.doClick();
					}
				} else {
					//if the selected destination folder does not exist, display the below message
					JOptionPane.showMessageDialog(null, "Specified destination folder doesn't exist ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 
					txtDestFile.requestFocusInWindow();
				}
				pwPrintWriter.println("DEST_FILE_DIRECTORY is .."+DEST_FILE_DIRECTORY+"..");
			}
		} catch (Exception e) {
			bnErrorFlag = true;
			System.out.println("Exception is ..."+e);
		}
	}
	/**
	* Method saveSelections
	*This method saves the curve selections in to an xml file
	* @return void
	*
	*/
	public void saveSelections() {
		try{
			//System.out.println("in save selections");
			int intMinCurvesSelected = 0;
			if (jtMandatory != null) {
				for(int intCurveIndex = 0; intCurveIndex < jtMandatory.getRowCount(); intCurveIndex++) {
					Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intCurveIndex,0);
					if (blnSelected.booleanValue()==true) {
						intMinCurvesSelected++;
					}
				}
			}
			//check if alteast 2 curves are selected
			if (intMinCurvesSelected < 2) {
				JOptionPane.showMessageDialog(null, "Atleast two Curves are to be selected, Please select another curve", "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 
			}  else {
				/*showInputDialog returns null if we clicked on Cancel... It returns String if we clicked on OK 
				 The String may be empty or entered value */
				
				//strSelectionFileName = JOptionPane.showInputDialog("Please enter file name for saving selections"); 
				String strWellName = objLasDataFile.getLasData().getWell();
				if (strWellName == null) {
					
					//add the data to table
					for(int intSectionIndex = 0; intSectionIndex < alParameterData.size(); intSectionIndex++) {
						
						LasSectionInfo objLasSectionInfo = (LasSectionInfo) alParameterData.get(intSectionIndex); 
						if (objLasSectionInfo.getMnemonic().equalsIgnoreCase("WELL")) {
							strWellName = objLasSectionInfo.getValue();
						}
					}
					
				}
				
				//checking for double codes and replacing with empty
				strWellName = strWellName.replace('"',' ');
				
				//checking for double codes and replacing with empty
				strWellName = strWellName.replace('#',' ');
				
				strSelectionFileName = strWellName+"_"+objLasDataFile.getRunNumber(); 

				if(strSelectionFileName!=null && strSelectionFileName.equals("null") == false && strSelectionFileName.equals("") == false) {
					strSelectionFileName = strSelectionFileName.trim();
					strSelectionName = strSelectionFileName.substring(strSelectionFileName.lastIndexOf("/")+1,strSelectionFileName.length());
					objLasDataFile.setSelectionFile(strSelectionName);
					
					//if no file name, inform user to enter a file name
					if (strSelectionFileName.equals("")) {
						JOptionPane.showMessageDialog(null, "Enter a file name for saving selections ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 	
						return;
					} else {
						File fCurrentFolder = new File(".");
						String strPath = new File(fCurrentFolder.getCanonicalPath()).getParent();
						pwPrintWriter.println("The strSelectionFileName is >>>>>>>>>>>>>> " + strSelectionFileName);
						strSelectionFileName = strPath+"/selections/"+strSelectionFileName+".xml";
						pwPrintWriter.println("The strSelectionFileName is >>>>>>>>>>>>>> " + strSelectionFileName);
						int intResponse = -1;
						boolean blnResponse = true;
						if ((new File(strSelectionFileName)).exists()) {
						intResponse = JOptionPane.showConfirmDialog(null, "Selections already exists with this name\n, Previous selections will be overwritten, Do you wish to save", "LAS to WITSML Conversion",JOptionPane.YES_NO_OPTION); 
							if (intResponse == JOptionPane.YES_OPTION) {
								blnResponse = true;
							} else {
								blnResponse = false;
								return;
							}							
						}
						if (blnResponse == true) {
							//Obtain a new instance of a DocumentBuilderFactory
							DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
							
							//Creates a new instance of a DocumentBuilder using the currently configured parameters.
							DocumentBuilder builder = factory.newDocumentBuilder();
							
							//Obtain a new instance of a DOM Document object to build a DOM tree with.
							org.w3c.dom.Document selectionDoc = builder.newDocument();
							//create <las2witsml> tag
							String strLas2witsml = "las2witsml";
							Element elLas2witsml = selectionDoc.createElement(strLas2witsml);
							selectionDoc.appendChild(elLas2witsml);
							//create <selections> tag
							String strSelection = "selections";
							Element elSelections = selectionDoc.createElement(strSelection);
							elLas2witsml.appendChild(elSelections);
							
							if (jtMandatory != null) {
								pwPrintWriter.println("Number of curves "+jtMandatory.getRowCount());
								//save the selections			
								for(int intSectionIndex = 0; intSectionIndex < jtMandatory.getRowCount(); intSectionIndex++) {
								//create <logCurveInfo> tag
									Element elLogCurveInfo = selectionDoc.createElement("logCurveInfo");
												
									String strLasName = (String)jtMandatory.getValueAt(intSectionIndex,1);
									if (strLasName != null) {
										//set Mnemonic attribute
										elLogCurveInfo.setAttribute("mnemonic",strLasName);
									}
									Boolean blnSelected = (Boolean) jtMandatory.getValueAt(intSectionIndex,0);
									String isSelected = "N";
									if (blnSelected.booleanValue()==true) {
										isSelected = "Y";
									}
									//set selected attribute
									elLogCurveInfo.setAttribute("selected",isSelected);
									//append logcurveinfo node to selections list
									elSelections.appendChild(elLogCurveInfo);
								}
							}
							
							// Get an instance of a TransformerFactory
							TransformerFactory transFactory =  TransformerFactory.newInstance();
						
							// Get a transformer using the factory
							Transformer transformer = transFactory.newTransformer();
							
							transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
							transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "5");
							
							//prepare domsource
							DOMSource objSource = new DOMSource(selectionDoc);
							//StreamResult objResult = new  StreamResult(new File(strSelectionFileName));
							StreamResult objResult = new  StreamResult(strSelectionFileName);
							//transform the selections DOM object to XML
							transformer.transform(objSource,objResult);
							//System.out.println("CREATED DOCUMENT");
							JOptionPane.showMessageDialog(null, "Selections are saved with name "+strSelectionFileName, "LAS to WITSML Conversion", JOptionPane.INFORMATION_MESSAGE); 	
						}
					}
				}else if( strSelectionFileName == null){
					//System.out.println("user clicked on Cancel");
				}
				else if(strSelectionFileName.trim().equals("") == true) {
					JOptionPane.showMessageDialog(null, "Enter a file name for saving selections ", "LAS to WITSML Conversion", JOptionPane.ERROR_MESSAGE); 	
					
				}
			}
		} catch(Exception eException) {
				bnErrorFlag = true;
				System.out.println(" Exception in saving selections "+eException.getMessage());
				pwPrintWriter.println(" Exception in saving selections "+eException.getMessage());
		}
	}
	
	
	public void showHelp(String section){
		try {
			File f = new File(".");
			String strPath = new File(f.getCanonicalPath()).getParent();
			//String url="file:\\"+strPath+"\\help\\LAS2WITSMLConverter_Help.html#"+section;
			String url="file:"+strPath+"/help/LAS2WITSMLConverter_Help.html#"+section;
			JEditorPane epEditorPane = new JEditorPane(url);
			epEditorPane.setEditable(false);
			JScrollPane spScrollPane = new JScrollPane(epEditorPane);
			JFrame frame = new JFrame();
			frame.getContentPane().add(spScrollPane, BorderLayout.CENTER);
			frame.setSize(800, 400);
			frame.setVisible(true);
		} catch (IOException eIOException) {
			bnErrorFlag = true;
			System.out.println("IOException is .."+eIOException);
			pwPrintWriter.println("IOException is .."+eIOException);
		}
	}

	//class for implementing ListSelectionListenere to check which element of the mandatory list is selected
	class SharedListSelectionHandler implements ListSelectionListener {
	/**
	* Method valueChanged
	*This method saves the curve selections in to an xml file
	* @return void
	* @param ListSelectionEvent
	*
	*/
		public void valueChanged(ListSelectionEvent e) {
			try{
				ListSelectionModel lsm = (ListSelectionModel)e.getSource();
				if (e.getValueIsAdjusting() == false) {
		
					if (lsm.getMinSelectionIndex() == -1) {
						//No selection, disable fire button.
						pwPrintWriter.println("no element selected ");
					} else if (lsm.getMinSelectionIndex() == 0) {
						pwPrintWriter.println("element 1 selected ");
						MAND_DATA_SELECTED = strMandParameter;
						ParameterDetails();
					} else if (lsm.getMinSelectionIndex() == 1) {
						pwPrintWriter.println("element 2 selected ");
						MAND_DATA_SELECTED = strMandCurveData;
						ParameterDetails();
					}
			  }
			}catch (ConverterException eConverterException) {
				bnErrorFlag = true;
				System.out.println(" Exception in displaying mandatory data "+eConverterException);
				pwPrintWriter.println(" Exception in displaying mandatory data "+eConverterException);
			} 	
		}
	}
	
	/**
	* Method main
	*This is the main method for invoking the LAS to WITSML converter
	* @param args
	* @return void
	*
	*/
	public static void main(String[] args) {
		String strVersion = System.getProperty("java.version");
		System.out.println("The version is " + strVersion);
		LasWitsml objLasWitsml = new LasWitsml(); 
		objLasWitsml.setTitle("LAS to WITSML Converter");
		objLasWitsml.pack();
		objLasWitsml.setBounds(50,50,900,650);
		objLasWitsml.setVisible(true);
		objLasWitsml.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
	}
}
	/**
	* class JComponentCellRenderer
	*This is the mainclass for implementing the table cell renderer method required for displaying cells in a JTable
	*/
	class JComponentCellRenderer implements TableCellRenderer {
		public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
			return (JComponent)value;
		}
	}