/**
 * <p>Title: LasWitsml.java</p>
 * <p>Description: This class converts LAS file to WITSML file</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: Satyam Computer Services Ltd.</p>
 * @author Ravi Domakonda, Lalitha Yengaldas
 * @version 1.0
 */
package com.las2witsml;

//import packages
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.*; 

import org.apache.xpath.XPathAPI;
import org.apache.xpath.objects.XObject;


// LasWitsmlBatch.java
// A quick application to convert batch of LAS to WITSML.

public class LasWitsmlBatch {
	//declare variables
	public String INPUT_LAS_DIRECTORY;
	public String DEST_FILE_DIRECTORY;
	public String MAPPING_FILE;
	public String SELECTION_FILE;
	
	LasDataFile objLasDataFile = null;
	LasData objLasData = null;
	ArrayList alParameterData = null;
	ArrayList alCurveData = null;
	
	public LasWitsmlBatch(String strInputLasFolder, String strDestWitsmlFolder, String strMappingFile, String strSelectionFile) {
		INPUT_LAS_DIRECTORY = strInputLasFolder;
		DEST_FILE_DIRECTORY = strDestWitsmlFolder;
		MAPPING_FILE = strMappingFile;
		SELECTION_FILE = strSelectionFile;
		System.out.println(" Input LAS Folder .."+INPUT_LAS_DIRECTORY);
		System.out.println(" Destination WITSML Folder .."+DEST_FILE_DIRECTORY );
		System.out.println(" Mapping File .."+MAPPING_FILE);
		System.out.println(" Selection File.."+SELECTION_FILE);
	}
	
	
	/**
	* Method checkArguments
	*Description: This method checks all the arguments for validity
	* @return boolean
	*
	*/
	public boolean checkArguments() throws ConverterException{
		boolean blnValidArgument=true;
		try {
			//System.out.println("Checking arguments ..");
			
			if (INPUT_LAS_DIRECTORY == null || INPUT_LAS_DIRECTORY.equals("")) {
				blnValidArgument = false;
				throw new ConverterException("Please provide an input LAS Folder ");
			} else if (DEST_FILE_DIRECTORY == null || DEST_FILE_DIRECTORY.equals("")) {
				blnValidArgument = false;
				throw new ConverterException("Please provide Destinator Folder "); 
			} else if (MAPPING_FILE == null || MAPPING_FILE.equals("")) {
				blnValidArgument = false;
				throw new ConverterException("Please provide mapping file "); 
			} else if (SELECTION_FILE == null || SELECTION_FILE.equals("")) {
				blnValidArgument = false;
				throw new ConverterException("Please provide selection file "); 
			}
			if (INPUT_LAS_DIRECTORY != null) {
				File fInputDirectory = new File(INPUT_LAS_DIRECTORY);
				if (!(fInputDirectory.isDirectory())) {
					blnValidArgument = false;
					throw new ConverterException("Not a valid input LAS File folder "); 
				}
			} 
			if (DEST_FILE_DIRECTORY != null) {
				File fDestDirectory = new File(DEST_FILE_DIRECTORY);
				if (!(fDestDirectory.isDirectory())) {
					blnValidArgument = false;
					throw new ConverterException("Not a valid destination folder "); 
				}
			}
			if (MAPPING_FILE != null) {
				File fMappingFile = new File(MAPPING_FILE);
				if (!(fMappingFile.isDirectory())) {
				    if (!fMappingFile.exists()) {
					blnValidArgument = false;
					throw new ConverterException("Mapping file does not exist"); 
				    }
				} else {
					blnValidArgument = false;
					throw new ConverterException(" Not a valid mapping file"); 
				}
			} 
			if (SELECTION_FILE != null) {
				File fSelectionFile = new File(SELECTION_FILE);
				if (!(fSelectionFile.isDirectory())) {
				    if (!fSelectionFile.exists()) {
					blnValidArgument = false;
					throw new ConverterException("Selection file does not exist"); 
				    }
				} else {
					blnValidArgument = false;
					throw new ConverterException(" Not a valid selection file"); 
				}
			}
			return blnValidArgument;
		} catch (ConverterException ce) {
			System.out.println("Exception in check arguments .."+ce.getMessage());
		} catch (Exception e) {
			System.out.println("Exception in check arguments .."+e.getMessage());
		}
		return blnValidArgument;
	}

	/**
	* Method performConversion
	*Description: This method converts all the LAS files from the Input LAS folder into WITSML files 
	* using the mapping and selection provided and places the generated files in Destination folder
	* @return void
	*
	*/
	public void performConversion() {
		try {
			boolean blnCheckArgument = checkArguments();
			//System.out.println(" blnCheckArgument "+blnCheckArgument);
			
			if (blnCheckArgument == false) {
				throw new ConverterException("Please provide valid inputs for conversion "); 
			} else {
				//opening the FileWriter with currentpath
				File fCurrentPath = new File(".");
				String strPath  = new File(fCurrentPath.getCanonicalPath()).getParent();
				FileWriter fwFileWriter = new FileWriter(strPath+"/log/LASWITSML_BATCH.log",false);
				PrintWriter pwPrintWriter = new PrintWriter(fwFileWriter,true);
				Vector vecLasFiles = getLASFiles();
				if (vecLasFiles == null) {
					throw new ConverterException("Input LAS folder is empty "); 
				} else if (vecLasFiles.size() == 0) {
					throw new ConverterException("No LAS files exists in the selected input folder "); 
				} else {
					System.out.println("Total Number of LAS files in input file directory "+vecLasFiles.size());
					for (int intNumFiles=0; intNumFiles<vecLasFiles.size(); intNumFiles++) {
						String strInputLasFile = (String) vecLasFiles.elementAt(intNumFiles);
						
						objLasDataFile = new LasDataFile();
					
						objLasDataFile.setLogger(pwPrintWriter);

						alParameterData = new ArrayList();
						alCurveData = new ArrayList();
						if (objLasDataFile != null) {
						
							objLasDataFile.setMappingFile(MAPPING_FILE);
							//call LoadFileData first,  because this creates an instance of LasData
							//System.out.println("before load file data ");
							objLasDataFile.LoadFileData(strInputLasFile);
							//System.out.println("after load file data ");
							LasData objLasData = objLasDataFile.getLasData();
							//set Input LAS file
							objLasData.setInputLasFile(strInputLasFile);
							
							//set Destination File
							String strFileName=new File(strInputLasFile).getName();
							System.out.println("The fileName is >>>>>>>>>>>>>>>>>>> " + strFileName);
							int intFileNameIndex = -1;
							intFileNameIndex = strFileName.indexOf(".LAS");
							if (intFileNameIndex == -1) {
								intFileNameIndex = strFileName.indexOf(".las");
							}
							System.out.println("intFileNameIndex"+intFileNameIndex);
							if (intFileNameIndex != -1) {
								objLasData.setDestFile(DEST_FILE_DIRECTORY+"/"+strFileName.substring(0,intFileNameIndex));
							}
							
							//set Selection file
							objLasDataFile.setSelectionFile(SELECTION_FILE);
							
							System.out.print(" INPUT "+objLasData.getInputLasFile());
							System.out.print(">>>DESTINATION "+objLasData.getDestFile());
							System.out.print(">>>MAPPING "+objLasDataFile.getMappingFile());
							System.out.println(">>>SELECTION "+objLasDataFile.getSelectionFile());
							
							//read Well and Parameter section
							objLasDataFile.readParametersSectionData(objLasData.getInputLasFile(),objLasData);
							alParameterData = objLasDataFile.getLasData().getParameterSectionData();
							System.out.println("Parameter data size .."+alParameterData.size());
							
							//read Curve Section data
							objLasDataFile.readLogDefinitionsData(objLasData.getInputLasFile(),null,"~C",objLasData);
							//alCurveData=objLasDataFile.getLasData().getLogDefns();
							//System.out.println("Curve size before applying selections is "+alCurveData.size());
														
							applySelection(objLasDataFile.getSelectionFile());
							alCurveData=objLasDataFile.getLasData().getLogDefns();
							System.out.println("Curve Defns after applying selections is "+alCurveData.size());
							int intNumCurveSelected = 0;
							boolean blnDefaultCurveSelected = false;
							for (int k=0; k<alCurveData.size(); k++) {
								LasSectionInfo objLasSectionInfo = (LasSectionInfo) alCurveData.get(k);
								//System.out.println("::::::::::  curve selected is .."+objLasSectionInfo.getIsSelected());
								if ( objLasSectionInfo.getIsSelected().booleanValue()) {
									intNumCurveSelected++;
									String strMnemonic = objLasSectionInfo.getMnemonic();
									//System.out.println("Mnemonic is ..."+strMnemonic);
									if ((strMnemonic.equalsIgnoreCase("DEPT")) || (strMnemonic.equalsIgnoreCase("TIME"))) {
										blnDefaultCurveSelected = true;
									}
								}
							}
							
							//
							//if (alCurveData.size() > 0) {
							if (intNumCurveSelected >= 2) {
								if (blnDefaultCurveSelected == false)
								{
									pwPrintWriter.println("Default Curves(DEPT/TIME) not selected for conversion.\n Please proceed manually or modify the selections file");
									System.out.println("Default Curves(DEPT/TIME) not selected for conversion.\n Please proceed manually or modify the selections file");
								}else{
									objLasDataFile.setOutputFiles(new Vector());
									//process well section data
									objLasDataFile.processWellSection(objLasData.getInputLasFile(),objLasData);
									//process log section data
									objLasDataFile.processLogSection(objLasData.getInputLasFile(),objLasData);
								}
							}else if (intNumCurveSelected < 2 ){
								pwPrintWriter.println("Atleast two curves should  be selected for conversion.\n Please proceed manually or modify the selections file");
								System.out.println("Atleast two curves should  be selected for conversion.\n Please proceed manually or modify the selections file");
							}
							else {
								throw new ConverterException("Cannot convert the LAS file "+objLasData.getInputLasFile());
							}
						}
					}
				}
			}
		} catch (ConverterException ce) {
			System.out.println("ConverterException in performConversion .."+ce.getMessage());
		} catch (Exception e) {
			System.out.println("Exception in performConversion .."+e.getMessage());
		}
	}
	/**
	* Method getLASFiles
	*Description: This method gets all the LAS files from the Input LAS folder 
	* @return Vector
	*
	*/
	public Vector getLASFiles() {
		try {
			if (INPUT_LAS_DIRECTORY != null )   { 
				File fInputFiles = new File(INPUT_LAS_DIRECTORY);
				if (fInputFiles.isDirectory()) {
					Vector vecFileDetails = new Vector();			
					String strFiles[] =fInputFiles.list();
					//System.out.println("Total Number of files in input file directory "+files.length);
														
					if(vecFileDetails.size() >0){
						vecFileDetails.clear();
					}
					//System.out.println(" +++++++ vector size .."+vecFileDetails.size());
					 for (int i=0; i < strFiles.length; i++) {
					
						File fTmp = new File(strFiles[i]);
						
						String strInputFileName = INPUT_LAS_DIRECTORY+"/"+fTmp.getName();
						//System.out.println(" <<<<<<<<<<< strInputFileName "+strInputFileName);
						if (strInputFileName != null)  {
							if((strInputFileName.endsWith("las")) || (strInputFileName.endsWith("LAS"))) {
								//add FileName to vector
								vecFileDetails.add(strInputFileName);
							}
						}
					}
					return vecFileDetails;
				} else {
					throw new ConverterException("Input LAS folder is not a valid folder "); 
				}
			} else {
				throw new ConverterException("Input LAS folder is not a valid folder "); 
			}
		} catch (Exception e) {
			System.out.println("Exception in getLASFiles "+e.getMessage());
		}
		return null;
	}
	
	
	public void applySelection(String strSelectionFile) {
		ArrayList alCurveInfoList = new ArrayList();
		try {
			//Obtain a new instance of a DocumentBuilderFactory
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			//Creates a new instance of a DocumentBuilder using the currently configured parameters.
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			//Parse the content of the given URI as an XML document and return a new DOM Document object
			Document selectedDocument = builder.parse(strSelectionFile);
			System.out.println(" selected document "+selectedDocument);
			
			//search for selected curves in selected .xml
			String strCurveInfoPath = "//las2witsml/selections/logCurveInfo[@selected='Y']";
			
			// Get LogCurveInfo List
			//NodeList nlLogCurveInfo = org.apache.xpath.XPathAPI.selectNodeList(selectedDocument, strCurveInfoPath);
			NodeList nlLogCurveInfo = XPathAPI.selectNodeList(selectedDocument, strCurveInfoPath);
			System.out.println(" Number of Curves selected  .."+nlLogCurveInfo.getLength());
			
			//get selected curves
			if ((nlLogCurveInfo != null) && (nlLogCurveInfo.getLength() > 0)) {
				// Process the curves in the curve list
				for (int i=0; i<nlLogCurveInfo.getLength(); i++) {
					Element elLogCurveInfo = (Element) nlLogCurveInfo.item(i);
					//System.out.println("ndLogCurveInfo node name"+elLogCurveInfo.getTagName());
					
					//get the Mnemonic name 					
					String strMnemonic = elLogCurveInfo.getAttributeNode("mnemonic").getValue();
					//System.out.println(" SELECTED MNEMONIC IS .."+strMnemonic);
					
					if (strMnemonic != null) {
						alCurveInfoList.add(strMnemonic);
					}
				}
					
			} //end for nlLogCurveInfo.getLength
			System.out.println(" LOAD SELECTIONS CURVE SIZE .."+alCurveInfoList.size());
		
			//Read curve data section
			ArrayList alCurveDefns = objLasDataFile.getLasData().getLogDefns();
			if ((alCurveDefns != null) && (alCurveDefns.size() > 0)) {
				for (int k=0; k<alCurveDefns.size(); k++) {
					LasSectionInfo objLasSectionInfo = (LasSectionInfo) alCurveDefns.get(k);
					
					if (alCurveInfoList.contains(objLasSectionInfo.getMnemonic())) {
						//System.out.println("::::::::::  curve selected is .."+objLasSectionInfo.getMnemonic());
						objLasSectionInfo.setIsSelected(new Boolean(true));
					} else {
						objLasSectionInfo.setIsSelected(new Boolean(false));
					}
					alCurveDefns.set(k, objLasSectionInfo);
				} 
			}
		} catch (javax.xml.parsers.ParserConfigurationException pce) {
			System.out.println("Error loading selections .ParserConfigurationException .."+pce.getMessage());
		} catch (org.xml.sax.SAXException saxe) {
			System.out.println("Error loading selections .SAXException.."+saxe.getMessage());
		} catch (javax.xml.transform.TransformerException te) {
			System.out.println("Error loading selections .TransformerException.."+te);
		} catch(Exception e) {
			System.out.println("Exception in applying selections ");
		}
	}
	//main method
	public static void main(String[] args) throws ConverterException{
		String strInputLasFolder = "";
		String strDestWitsmlFolder = "";
		String strMappingFile = "";
		String strSelectionFile = "";

		try {
			
			System.out.println(" Number of arguments provided "+args.length);
			
			/*for (int i = 0; i < args.length; i++)
			    System.out.println(args[i]);*/
			
			if (args.length < 4) {
				System.err.println("Usage: LasWitsmlBatch InputLASFolder DestinationWITSMLFolder MappingFile CurveSelectionFile");
				return;
			} else {
				if (args[0] != null) {
					strInputLasFolder = args[0];
					//System.out.println(" Input LAS Folder .."+strInputLasFolder);
				}
				if (args[1] != null) {
					strDestWitsmlFolder = args[1];
					//System.out.println(" Destination WITSML Folder .."+strDestWitsmlFolder);
				}
				if (args[2] != null) {
					strMappingFile = args[2];
					//System.out.println(" Mapping File .."+strMappingFile);
				}
				if (args[3] != null) {
					strSelectionFile = args[3];
					//System.out.println(" Curve Selection File .."+strSelectionFile);
				}
			}
			//call constructor and initialize the arguments
			LasWitsmlBatch objLasWitsmlBatch = new LasWitsmlBatch(strInputLasFolder, strDestWitsmlFolder, strMappingFile, strSelectionFile); 
			//call perform conversion method to initiate the LAS-WITSML conversion
			objLasWitsmlBatch.performConversion();
		} catch (Exception e) {
			System.out.println("Exception in main method .."+e.getMessage());
		}
	}
}
