package com.las2witsml;

//import classes
import java.io.File;
import java.util.Hashtable;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Enumeration;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.ProcessingInstruction;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException; 

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.OutputKeys;

import org.apache.xpath.XPathAPI;
import org.apache.xpath.objects.XObject;

/**
 * Title: MappingReader.java
 * Description:  This class reads the mapping file and creates DOM tree by parsing the mapping document 
 * Copyright: Copyright (c) 2006
 * Company: Satyam Computer Services Ltd.
  * @version 1.0
 * @author Ravi Domakonda, Lalitha Yengaldas
 */

public class MappingReader{
	//declare variables 
	Hashtable htWellNodeData=null; //hashtable for WELL section mnemonics
	Hashtable htLogNodeData=null; //hashtable for WELL Log section mnemonics
	org.w3c.dom.Document newXMLDoc; //new xml documented to be created
	/** 
	*  Constructor MappingReader
	*/
	MappingReader() {
		
	}

	/**
	* Method createDocument
	*This method reads the mapping xml file and creates an XML document 
	* @param strSection
	* @return Document
	*
	*/	
	public org.w3c.dom.Document createDocument(String strSection,String strMappingFileName){
		//declare variables 
		String strStyleSheet="";
		String strPath="";
		String strMappingFile="";
		String strStyleSheetPath="";
		String strAncillaryPath="";
		Element docRoot= null;
		
		try {
			//get the current directory path
			File fCurrentPath=new File(".");
			strPath=new File(fCurrentPath.getCanonicalPath()).getParent();
			
			//System.out.println("The mapping file path is :::::::::::::::::::::: " + strMappingFileName);
			
			//set the mapping file
			if(strMappingFileName!=null && strMappingFileName.equals("null") == false && strMappingFileName.equals("") == false){
				strMappingFile=strMappingFileName;
			}else{
				//set the default mapping from the current directory
				strMappingFile=strPath+"/mappings/Default.xml";
			}
			
			//set the style sheet path from the current directory
			strStyleSheetPath=strPath+"/stylesheets/";
			//System.out.println("  <<<<<< strStyleSheetPath"+strStyleSheetPath);
			
			//Obtain a new instance of a DocumentBuilderFactory
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			//Creates a new instance of a DocumentBuilder using the currently configured parameters.
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			//Parse the content of the given URI as an XML document and return a new DOM Document object
			org.w3c.dom.Document mappingDocument = builder.parse(strMappingFile);
					
			//System.out.println(" createDocument for section "+strSection);	
			
			//Obtain a new instance of a DOM Document object to build a DOM tree with.
			newXMLDoc = builder.newDocument();
			
			// check for child nodes and add the style sheet
			if (!(newXMLDoc.hasChildNodes())) {
				//create root element
				docRoot = newXMLDoc.createElement(strSection);
				
				//append root to xml document
				newXMLDoc.appendChild(docRoot);
				
				// Add a PI at the beginning of the document
				Element element = newXMLDoc.getDocumentElement();
				
				//set the style sheet name depending on section
				if(strSection.equalsIgnoreCase("wells")){
					strStyleSheet = "well.xsl";
				}else if(strSection.equalsIgnoreCase("wellLogs")){
					strStyleSheet = "wellLog.xsl";
				}
				
				//form the style sheet name along with path
				strStyleSheet=strStyleSheetPath + strStyleSheet;
				
				//System.out.println("The style sheet is >>>>>>>>>> " + strStyleSheet);
	
				//create a processing instruction for style sheet
				ProcessingInstruction pi = newXMLDoc.createProcessingInstruction("xml-stylesheet", "href=\""+strStyleSheet+"\" type=\"text/xsl\" media=\"screen\"");
				//insert the above processing instruction before the root node
				element.getParentNode().insertBefore(pi, docRoot);
				
			}
			
			//search for attributes for section in mapping .xml
			String strAttributePath = "//las2witsml/mappings/section[@name='"+strSection+"']/attributes/attribute";
			
			// Get attibutes list
			Element elRoot = mappingDocument.getDocumentElement();
			NodeList nlAttributes = XPathAPI.selectNodeList(mappingDocument, strAttributePath, elRoot);
					
				
			//set attributes for root node
			for (int intAttribute = 0; intAttribute < nlAttributes.getLength(); intAttribute++) {
				Element elAttributeNode = (Element) nlAttributes.item(intAttribute);
				//get name
				String strAttribute = elAttributeNode.getAttributeNode("name").getValue();
				//get value
				String strAttributeValue = elAttributeNode.getAttributeNode("value").getValue();
				//set attribute for document root
				docRoot.setAttribute(strAttribute,strAttributeValue);
			}
			
			//search for relations in mapping
			String strRelationPath = "//las2witsml/mappings/section[@name='"+strSection+"']/relations/elem";
			// Get relations elements
			NodeList nlRelation = XPathAPI.selectNodeList(mappingDocument, strRelationPath, elRoot);
						
			//create relation elements
			for (int intRelation = 0; intRelation < nlRelation.getLength(); intRelation++) {
			
				Element elElemNode = (Element) nlRelation.item(intRelation);
				//get name
				String strElementName = elElemNode.getAttributeNode("name").getValue();
				//get parent
				String strElementParent = elElemNode.getAttributeNode("parent").getValue();
				
				//create a new element 
				Node childNode = newXMLDoc.createElement(strElementName);
				
				//get the nodelist for the above element
				NodeList nlParentList  = newXMLDoc.getElementsByTagName(strElementParent);
				//check for child nodes in the above list
				if (nlParentList.getLength() > 0) {
					Node ndParentNode = (Element)nlParentList.item(0);
					//append this node to the parent list
					ndParentNode.appendChild(childNode);
				} else {
					//append this node to the document root
					docRoot.appendChild(childNode);
				}
			}
		
			if(strSection.equalsIgnoreCase("wells")){
			
				//search for mandatoryelements in mapping
				String strMandatoryPath = "//las2witsml/mappings/section[@name='"+strSection+"']/mandatoryelements";
				//get mapping elements
				NodeList nlMandatoryElements = XPathAPI.selectNodeList(mappingDocument, strMandatoryPath, elRoot);
					
				//create mandatory elements
				if (nlMandatoryElements.getLength() > 0) {
					// Process the elements in the mandatory elements list
					for (int intMandatory = 0; intMandatory < nlMandatoryElements.getLength(); intMandatory++) {
					
						Element elDataNode = (Element) nlMandatoryElements.item(intMandatory);
						
						//get the WITSML elements
						NodeList nlWitsmlName  = elDataNode.getElementsByTagName("WITSML");
						
						for (int m=0; m<nlWitsmlName.getLength(); m++) {
							Element elWitsml = (Element)nlWitsmlName.item(m);
							//get the WITSML element name 					
							String strWitsmlName = elWitsml.getAttributeNode("elemname").getValue();
							//get the WITSML element parent name 	
							String strWitsmlParent = elWitsml.getAttributeNode("elemparent").getValue();
															
							//create a new element 
							Node ndWitsml = newXMLDoc.createElement(strWitsmlName);
							//get the elements under the parent node
							NodeList nlWitsmlParent  = newXMLDoc.getElementsByTagName(strWitsmlParent);
							
							//create nodes
							if (nlWitsmlParent.getLength() > 0) {
								Node ndParentNode = (Element)nlWitsmlParent.item(0);
								//create a text node and append to the parent node
								Text text = newXMLDoc.createTextNode("");
								ndWitsml.appendChild(text);
								ndParentNode.appendChild(ndWitsml);
							} else {
								//append the node to document root
								docRoot.appendChild(ndWitsml);
							}
						}
					}
				}
			}
			
			
			
			//search for documentheader in mapping
			String strDocHeaderPath = "//las2witsml/mappings/section[@name='"+strSection+"']/documentheader/elem";
			// Get documentheader elements
			NodeList nlDocHeader = XPathAPI.selectNodeList(mappingDocument, strDocHeaderPath, elRoot);
			
			//create documentheader elements
			for (int intDocument = 0; intDocument < nlDocHeader.getLength(); intDocument++) {
			
				Element elElemNode = (Element) nlDocHeader.item(intDocument);
				//get name
				String strElementName = elElemNode.getAttributeNode("name").getValue();
				//get parent
				String strElementParent = elElemNode.getAttributeNode("parent").getValue();
								
				//create a new element 
				Node childNode = newXMLDoc.createElement(strElementName);
				//get nodes under the parent
				NodeList nlParentList  = newXMLDoc.getElementsByTagName(strElementParent);
				//check for child nodes in the above list
				if (nlParentList.getLength() > 0) {
					Node ndParentNode = (Element)nlParentList.item(0);
					//append this node to the parent list
					ndParentNode.appendChild(childNode);
				} else {
					//append this node to the document root
					docRoot.appendChild(childNode);
				}
			}
			
			//search for nodedata in mapping
			String strNodeDataPath = "//las2witsml/mappings/section[@name='"+strSection+"']/nodedata";
			// Get nodedata elements
			NodeList nlNodeData = XPathAPI.selectNodeList(mappingDocument, strNodeDataPath, elRoot);
					
			//create nodedata elements
			if (nlNodeData.getLength() > 0) {
				//check the section and create hashtables for storing data
				if(strSection.equalsIgnoreCase("wells")){
					htWellNodeData = new Hashtable();
				}else if(strSection.equalsIgnoreCase("wellLogs")){
					htLogNodeData = new Hashtable();
				}
				// Process the elements in the node data list
				for (int intNode = 0; intNode < nlNodeData.getLength(); intNode++) {
					Element elDataNode = (Element) nlNodeData.item(intNode);
					//get LAS elements from the list
					NodeList nlLasName = elDataNode.getElementsByTagName("LAS");
					
					for (int intLasName = 0; intLasName < nlLasName.getLength(); intLasName++) {
						Element elLas = (Element)nlLasName.item(intLasName);
						//get the LAS name 
						String strLasName = elLas.getAttributeNode("name").getValue();
						
						//get the WITSML elements
						NodeList nlWitsmlName  = elLas.getElementsByTagName("WITSML");
						for (int intWitsml = 0; intWitsml < nlWitsmlName.getLength(); intWitsml++) {
							Element elWitsml = (Element)nlWitsmlName.item(intWitsml);
							//get the WITSML element name 					
							String strWitsmlName = elWitsml.getAttributeNode("elemname").getValue();
							//get the WITSML element parent name 					
							String strWitsmlParent = elWitsml.getAttributeNode("elemparent").getValue();
							
							//get the Default Attribute
							String strDefault = elWitsml.getAttributeNode("default").getValue();
							
							
							if (strDefault.equalsIgnoreCase("Y")) {
								
								//create a new element 
								Node ndWitsml = newXMLDoc.createElement(strWitsmlName);
								//get elements for the parent
								NodeList nlWitsmlParent  = newXMLDoc.getElementsByTagName(strWitsmlParent);
															
								//create nodes
								if (nlWitsmlParent.getLength() > 0) {
									Node ndParentNode = (Element)nlWitsmlParent.item(0); 
									//create a text node and append to the parent node
									Text text = newXMLDoc.createTextNode("--");
									ndWitsml.appendChild(text);
									ndParentNode.appendChild(ndWitsml);
								} else {
									//append the node to document root
									docRoot.appendChild(ndWitsml);
								}
								
								//add the mapping element to hash table depending on section to be processed
								if(strSection.equalsIgnoreCase("wells")){
									htWellNodeData.put(strLasName,strWitsmlName);
								}else if(strSection.equalsIgnoreCase("wellLogs")){
									htLogNodeData.put(strLasName,strWitsmlName);
								}
							}
							
						}	
					}
				}
				if(strSection.equalsIgnoreCase("wells")){
					setMappings(htWellNodeData,strSection);
				
				}else if(strSection.equalsIgnoreCase("wellLogs")){
					setMappings(htLogNodeData,strSection);
					//System.out.println("############ Mappings in create doc "+getMappings("wellLogs"));
				}
				
				
			}
			
				
			
			if(strSection.equalsIgnoreCase("wellLogs")){
			
				//search for mandatoryelements in mapping
				String strMandatoryPath = "//las2witsml/mappings/section[@name='"+strSection+"']/mandatoryelements";
				//get mapping elements
				NodeList nlMandatoryElements = XPathAPI.selectNodeList(mappingDocument, strMandatoryPath, elRoot);
					
				//create mandatory elements
				if (nlMandatoryElements.getLength() > 0) {
					// Process the elements in the mandatory elements list
					for (int intMandatory = 0; intMandatory < nlMandatoryElements.getLength(); intMandatory++) {
					
						Element elDataNode = (Element) nlMandatoryElements.item(intMandatory);
						
						//get the WITSML elements
						NodeList nlWitsmlName  = elDataNode.getElementsByTagName("WITSML");
						
						for (int m=0; m<nlWitsmlName.getLength(); m++) {
							Element elWitsml = (Element)nlWitsmlName.item(m);
							//get the WITSML element name 					
							String strWitsmlName = elWitsml.getAttributeNode("elemname").getValue();
							//get the WITSML element parent name 	
							String strWitsmlParent = elWitsml.getAttributeNode("elemparent").getValue();
															
							//create a new element 
							Node ndWitsml = newXMLDoc.createElement(strWitsmlName);
							//get the elements under the parent node
							NodeList nlWitsmlParent  = newXMLDoc.getElementsByTagName(strWitsmlParent);
							
							//create nodes
							if (nlWitsmlParent.getLength() > 0) {
								Node ndParentNode = (Element)nlWitsmlParent.item(0);
								//create a text node and append to the parent node
								Text text = newXMLDoc.createTextNode("");
								ndWitsml.appendChild(text);
								ndParentNode.appendChild(ndWitsml);
							} else {
								//append the node to document root
								docRoot.appendChild(ndWitsml);
							}
						}
					}
				}
			}
			if(strSection.equalsIgnoreCase("wells")){
				//get the WITSML elements
				NodeList nlWell  = newXMLDoc.getElementsByTagName("well");
				//System.out.println("nlWell size >>>>>>>>>>>>>>is .."+nlWell.getLength());
				Element elWell= (Element)nlWell.item(0);
				//System.out.println("Element Name .."+elWell.getNodeName());
				Node ndWellLocation = (Node) elWell.getFirstChild();
				//System.out.println("Node Name .."+ndWellLocation.getNodeName());
				Node ndCommonData = (Node) ndWellLocation.getNextSibling();
				//System.out.println("Common Data Node Name .."+ndCommonData.getNodeName());
				Node ndCustomData = ndCommonData.getNextSibling();
				//System.out.println("Custom Data Node Name .."+ndCustomData.getNodeName());
				elWell.removeChild(ndWellLocation);
				elWell.removeChild(ndCommonData);
				elWell.removeChild(ndCustomData);
				//System.out.println("after removing wellLocation ");
				elWell.appendChild(ndWellLocation);
				elWell.appendChild(ndCommonData);
				elWell.appendChild(ndCustomData);
				//System.out.println("after appending wellLocation ");
				
			}
			

		} catch (Exception e ) {
			System.out.println(" Exception in createDocument .."+e.getMessage());
		}
		return newXMLDoc;
	}
	
	/**
	* Method LASWITSMLElements
	*This method reads the mapping xml file and returns a hashtable of LAS and WITSML elements
	* @param Section to be searched for in the mapping xml
	* @return FullPath of the mapping file
	*
	*/	
	public Hashtable LASWITSMLElements(String strSection,String strFullPath) 	{
		Element docRoot= null;
		org.w3c.dom.Document newXMLDoc;
		Hashtable htWellNodeData=null; //hashtable for WELL section mnemonics
		Hashtable htLogNodeData=null;
		String strMappingFileName=null;
		
		try{
			if(strFullPath == null){
				File fCurrentPath=new File(".");
				String Path=new File(fCurrentPath.getCanonicalPath()).getParent();
				//set the default mapping from the current directory
				strMappingFileName=Path+"/mappings/Default.xml";
			}else{
				strMappingFileName=strFullPath;
			}
			
			//Obtain a new instance of a DocumentBuilderFactory
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			//Creates a new instance of a DocumentBuilder using the currently configured parameters.
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			//Parse the content of the given URI as an XML document and return a new DOM Document object
			org.w3c.dom.Document mappingDocument = builder.parse(strMappingFileName);
			
			//Obtain a new instance of a DOM Document object to build a DOM tree with.
			newXMLDoc = builder.newDocument();			
			
			docRoot = newXMLDoc.createElement(strSection);
		//search for nodedata in mapping
			String strNodeDataPath = "//las2witsml/mappings/section[@name='"+strSection+"']/nodedata";
			// Get nodedata elements
			Element elRoot = mappingDocument.getDocumentElement();
			NodeList nlNodeData = XPathAPI.selectNodeList(mappingDocument, strNodeDataPath, elRoot);
						
			//create nodedata elements
			if (nlNodeData.getLength() > 0) {
				//create hashtables for storing data depending on section
				if(strSection.equalsIgnoreCase("wells")){
					htWellNodeData = new Hashtable();
				}else if(strSection.equalsIgnoreCase("wellLogs")){
					htLogNodeData = new Hashtable();
				}
				// Process the elements in the node data list
				for (int intNode=0; intNode < nlNodeData.getLength(); intNode++) {
					Element elDataNode = (Element) nlNodeData.item(intNode);
					//get LAS elements from the list
					NodeList nlLasName = elDataNode.getElementsByTagName("LAS");
									
					for (int intLas=0; intLas < nlLasName.getLength(); intLas++) {
						Element elLas = (Element)nlLasName.item(intLas);
						//get the LAS name 
						String strLasName = elLas.getAttributeNode("name").getValue();
						
						
						//get the WITSML elements
						NodeList nlWitsmlName  = elLas.getElementsByTagName("WITSML");
						/*
						if(strLasName.equalsIgnoreCase("SRVC"))
						{
							System.out.println("nlWitsmlName.getLength() " + nlWitsmlName.getLength());
						}*/
						ArrayList alWitsmlList=new ArrayList();
						for (int intWitsml = 0; intWitsml<nlWitsmlName.getLength(); intWitsml++) {
							Element elWitsml = (Element)nlWitsmlName.item(intWitsml);
							//get the WITSML element name 					
							String strWitsmlName = elWitsml.getAttributeNode("elemname").getValue();
							//get the WITSML element editable name 					
							
							//Commented as we have removed editable feature
							/*String strEditable = elWitsml.getAttributeNode("editable").getValue();
							if(strEditable!=null && strEditable.equals("")==false){
								if(strEditable.trim().equalsIgnoreCase("Y")){*/
									//add the WITSML element to arraylist
									alWitsmlList.add(strWitsmlName);
									//add the mapping element to hash table depending on section to be processed
									if(strSection.equalsIgnoreCase("wells")){
										htWellNodeData.put(strLasName,alWitsmlList);
									}else if(strSection.equalsIgnoreCase("wellLogs")){
										htLogNodeData.put(strLasName,alWitsmlList);
									}
								/*}
							}*/
						}	
					}
				}			
			}
			//return the stored data depending on the section
			if(strSection.equalsIgnoreCase("wells")){
				return htWellNodeData;
			}else if(strSection.equalsIgnoreCase("wellLogs")){
				return htLogNodeData;
			}
		}catch(Exception e) {
			System.out.println("The error is " + e);
		}
		return null;	
	}

	
	/**
	* Method UpdateWITSMLElements
	*This method updates the mapping xml file as per the new mappings provided by the user
	* @param Path of the mapping xml file
	* @return hashtable of LAS - WITSML elements
	*
	*/	
	public void UpdateWITSMLElements(String strPath,Hashtable htTableElements) {
		Element docRoot= null;
		org.w3c.dom.Document newXMLDoc;
		Hashtable htWellNodeData=null; //hashtable for WELL section mnemonics
		Hashtable htLogNodeData=null;
		
		try{
		String strMappingFileName=strPath;
		
		//System.out.println("The strMappingfileName is " + strMappingFileName);
		
		//Obtain a new instance of a DocumentBuilderFactory
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		
		//Creates a new instance of a DocumentBuilder using the currently configured parameters.
		DocumentBuilder builder = factory.newDocumentBuilder();
		
		//Parse the content of the given URI as an XML document and return a new DOM Document object
		org.w3c.dom.Document mappingDocument = builder.parse(strMappingFileName);
		
		//Obtain a new instance of a DOM Document object to build a DOM tree with.
		newXMLDoc = mappingDocument;		
		
		String strSection=null;
		for(int intSection = 0; intSection < 2; intSection++){
			if(intSection == 0){
				strSection="wellLogs";
			}else{
				strSection="wells";
			}
			//create a document 
			docRoot = newXMLDoc.createElement(strSection);
			//search for nodedata in mapping
			String strNodeDataPath = "//las2witsml/mappings/section[@name='"+strSection+"']/nodedata";
			// Get nodedata elements
			Element elRoot = mappingDocument.getDocumentElement();
			NodeList nlNodeData = XPathAPI.selectNodeList(mappingDocument, strNodeDataPath, elRoot);
			
			//create nodedata elements
			if (nlNodeData.getLength() > 0) {
				//create a hashtable to store mapping depending on section
				if(strSection.equalsIgnoreCase("wells")){
					htWellNodeData = new Hashtable();
				}else if(strSection.equalsIgnoreCase("wellLogs")){
					htLogNodeData = new Hashtable();
				}
				// Process the elements in the node data list
					for (int intNode = 0; intNode < nlNodeData.getLength(); intNode++) {
					
						Element elDataNode = (Element) nlNodeData.item(intNode);
						//get LAS elements from the list
						NodeList nlLasName = elDataNode.getElementsByTagName("LAS");
						
						
												
						for (int intLas=0; intLas<nlLasName.getLength(); intLas++) {
							Element elLas = (Element)nlLasName.item(intLas);
							
							//get the LAS name 
							String strLasName = elLas.getAttributeNode("name").getValue();
							
							//get the WITSML elements
							NodeList nlWitsmlName  = elLas.getElementsByTagName("WITSML");
							
							//System.out.println("The witsml length is " + nlWitsmlName.getLength()+"for element"+elLas.getAttribute("name"));
							for (int intWt=0; intWt<nlWitsmlName.getLength(); intWt++) {
								Element elWit = (Element)nlWitsmlName.item(intWt);
								//System.out.println("The elwit is " + elWit);
								//get the LAS name 
								String strWitsmlName = elWit.getAttributeNode("elemname").getValue();
								String strWitsmlElement = (String) htTableElements.get(strLasName); 
								if (strWitsmlName.equalsIgnoreCase(strWitsmlElement)){
									//System.out.println("Both are equal");
									elWit.setAttribute("default","Y");
									if (nlWitsmlName.getLength() > 1) {
										//System.out.println("Node name " + elLas.getFirstChild().getNodeName());
										if (elLas.getFirstChild().getNodeName().equalsIgnoreCase("WITSML")){
											//System.out.println("In witsml");
											Text text = newXMLDoc.createTextNode("");
											
											elLas.insertBefore(text,elLas.getFirstChild());
											//System.out.println("after apending");
											//System.out.println("Node name11 " + elLas.getFirstChild().getNodeName());
										}
										elLas.insertBefore(elWit, elLas.getFirstChild()); 
									}
								} else {
									//System.out.println("Both are not equal");
									elWit.setAttribute("default","N");
								}
							}
							/*System.out.println("before document");
							//elLasDumyS.appendChild(elLasDumyN);
							newXMLDoc.appendChild(elLasDumyS);
							System.out.println("after document");*/
							//elLas.appendChild(elDumyElement);
							
							//Element elWitsml = (Element)nlWitsmlName.item(0);
							
							
							
							/*
							Commented the below code to update the default mapping as per witsml element provided in customize mapping STARTS here
							Enumeration e = htTableElements.keys(); 
							int intNumElements=1;
							while(e.hasMoreElements() ) { 
								String strWitsmlElement="";
								String strLasElement="";
								strLasElement = (String)e.nextElement();
								strWitsmlElement=(String) htTableElements.get(strLasElement); 
								System.out.println("strLasElement "+strLasElement);
								if(strLasElement.equalsIgnoreCase("SRVC")){
									System.out.println("The strWitsmlElement element is " + strWitsmlElement);
								}
									if(strLasName.trim().equalsIgnoreCase(strLasElement)){
										//get the WITSML element name 	
										elWitsml.setAttribute("elemname",strWitsmlElement);
										String strWitsmlName = elWitsml.getAttributeNode("elemname").getValue();
										break;
									}
								intNumElements++;
								Commented the below code to update the default mapping as per witsml element provided in customize mapping ENDS here
							}*/
						}
					}			
				}
			}
			// Get an instance of a TransformerFactory
			TransformerFactory transFactory = TransformerFactory.newInstance();
			// Get a transformer using the factory
			Transformer transformer = transFactory.newTransformer();
			
			transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "5");
			
			DOMSource objSource = new DOMSource(newXMLDoc);
			//StreamResult objResult = new  StreamResult(new File(strPath));
			StreamResult objResult = new  StreamResult(strPath);
			transformer.transform(objSource,objResult);
		}catch(Exception e) 	{
			System.out.println("The error is " + e);
		}
	}
	
	/**
	* Method setMappings
	*This method sets the LAS-WITSML mappings that are read from the mapping xml file
	* @param mapping elements
	* @param section
	* @return void
	*
	*/	
	public void setMappings(Hashtable htMappedElements,String strSection) {
		//check for the section and set the mappings
		if(strSection.equalsIgnoreCase("wells")){
			htWellNodeData = htMappedElements;
		}else if(strSection.equalsIgnoreCase("wellLogs")){
			htLogNodeData = htMappedElements;
		}
		
	}
	/**
	* Method getMappings
	*This method returns the mapping from the hash table
	* @param strSection
	* @return Hashtable
	*
	*/	
	public Hashtable getMappings(String strSection) {
		//check for the section and get the mappings
		if(strSection.equalsIgnoreCase("wells")){
			return htWellNodeData;
		}else if(strSection.equalsIgnoreCase("wellLogs")){
			return htLogNodeData;
		}
		return null;
	}
}